@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -16,10 +16,10 @@ import itertools
 
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
 from heat.engine import resource
+from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 from heat.tests import utils
 
 from ..resources import auto_scale  # noqa
@@ -99,7 +99,7 @@ class FakeAutoScale(object):
         """Update the launch configuration on a scaling group."""
         allowed = ['launch_config_type', 'server_name', 'image', 'flavor',
                    'disk_config', 'metadata', 'personality', 'networks',
-                   'load_balancers', 'key_name']
+                   'load_balancers', 'key_name', 'user_data', 'config_drive']
         self._check_args(kwargs, allowed)
         self._get_group(group_id).kwargs = kwargs
 
@@ -164,7 +164,7 @@ class FakeAutoScale(object):
         webhook.kwargs['metadata'] = metadata
 
 
-class ScalingGroupTest(HeatTestCase):
+class ScalingGroupTest(common.HeatTestCase):
 
     group_template = template_format.parse('''
     HeatTemplateFormatVersion: "2012-12-12"
@@ -191,6 +191,8 @@ class ScalingGroupTest(HeatTestCase):
                             key_name: my-key
                             metadata:
                                 server: metadata
+                            personality:
+                                /tmp/testfile: "dGVzdCBjb250ZW50"
                             networks:
                                 - uuid: "00000000-0000-0000-0000-000000000000"
                                 - uuid: "11111111-1111-1111-1111-111111111111"
@@ -198,12 +200,11 @@ class ScalingGroupTest(HeatTestCase):
 
     def setUp(self):
         super(ScalingGroupTest, self).setUp()
-        utils.setup_dummy_db()
         for res_name, res_class in auto_scale.resource_mapping().items():
             resource._register_class(res_name, res_class)
         self.fake_auto_scale = FakeAutoScale()
-        self.patch(clients.OpenStackClients,
-                   'auto_scale', lambda resource: self.fake_auto_scale)
+        self.patchobject(auto_scale.Group, 'auto_scale',
+                         return_value=self.fake_auto_scale)
 
     def _setup_test_stack(self):
         self.stack = utils.parse_stack(self.group_template)
@@ -222,6 +223,74 @@ class ScalingGroupTest(HeatTestCase):
         self.assertEqual(
             {
                 'cooldown': 60,
+                'config_drive': False,
+                'user_data': None,
+                'disk_config': None,
+                'flavor': 'flavor-ref',
+                'image': 'image-ref',
+                'launch_config_type': 'launch_server',
+                'load_balancers': None,
+                'key_name': "my-key",
+                'max_entities': 25,
+                'group_metadata': {'group': 'metadata'},
+                'metadata': {'server': 'metadata'},
+                'min_entities': 1,
+                'name': 'My Group',
+                'networks': [{'uuid': '00000000-0000-0000-0000-000000000000'},
+                             {'uuid': '11111111-1111-1111-1111-111111111111'}],
+                'personality': [{
+                        'path': u'/tmp/testfile',
+                        'contents': u'dGVzdCBjb250ZW50'}],
+                'server_name': u'autoscaled-server'},
+            self.fake_auto_scale.groups['0'].kwargs)
+
+        resource = self.stack['my_group']
+        self.assertEqual('0', resource.FnGetRefId())
+
+    def test_group_create_no_personality(self):
+
+        template = template_format.parse('''
+HeatTemplateFormatVersion: "2012-12-12"
+Description: "Rackspace Auto Scale"
+Parameters: {}
+Resources:
+    my_group:
+        Type: Rackspace::AutoScale::Group
+        Properties:
+            groupConfiguration:
+                name: "My Group"
+                cooldown: 60
+                minEntities: 1
+                maxEntities: 25
+                metadata:
+                    group: metadata
+            launchConfiguration:
+                type: "launch_server"
+                args:
+                    server:
+                        name: autoscaled-server
+                        flavorRef: flavor-ref
+                        imageRef: image-ref
+                        key_name: my-key
+                        metadata:
+                            server: metadata
+                        networks:
+                            - uuid: "00000000-0000-0000-0000-000000000000"
+                            - uuid: "11111111-1111-1111-1111-111111111111"
+''')
+
+        self.stack = utils.parse_stack(template)
+        self.stack.create()
+        self.assertEqual(
+            ('CREATE', 'COMPLETE'), self.stack.state,
+            self.stack.status_reason)
+
+        self.assertEqual(1, len(self.fake_auto_scale.groups))
+        self.assertEqual(
+            {
+                'cooldown': 60,
+                'config_drive': False,
+                'user_data': None,
                 'disk_config': None,
                 'flavor': 'flavor-ref',
                 'image': 'image-ref',
@@ -250,8 +319,11 @@ class ScalingGroupTest(HeatTestCase):
         self._setup_test_stack()
 
         resource = self.stack['my_group']
-        new_template = copy.deepcopy(resource.parsed_template())
-        new_template['Properties']['groupConfiguration']['minEntities'] = 5
+        uprops = copy.deepcopy(dict(resource.properties.data))
+        uprops['groupConfiguration']['minEntities'] = 5
+        new_template = rsrc_defn.ResourceDefinition(resource.name,
+                                                    resource.type(),
+                                                    uprops)
         scheduler.TaskRunner(resource.update, new_template)()
 
         self.assertEqual(1, len(self.fake_auto_scale.groups))
@@ -266,9 +338,13 @@ class ScalingGroupTest(HeatTestCase):
         self._setup_test_stack()
 
         resource = self.stack['my_group']
-        new_template = copy.deepcopy(resource.parsed_template())
-        lcargs = new_template['Properties']['launchConfiguration']['args']
+        uprops = copy.deepcopy(dict(resource.properties.data))
+        lcargs = uprops['launchConfiguration']['args']
         lcargs['loadBalancers'] = [{'loadBalancerId': '1', 'port': 80}]
+        new_template = rsrc_defn.ResourceDefinition(resource.name,
+                                                    resource.type(),
+                                                    uprops)
+
         scheduler.TaskRunner(resource.update, new_template)()
 
         self.assertEqual(1, len(self.fake_auto_scale.groups))
@@ -309,7 +385,7 @@ class ScalingGroupTest(HeatTestCase):
             if count < 3:
                 raise auto_scale.Forbidden("Not empty!")
 
-        self.patch(self.fake_auto_scale, 'delete', delete)
+        self.patchobject(self.fake_auto_scale, 'delete', side_effect=delete)
         resource = self.stack['my_group']
         scheduler.TaskRunner(resource.delete)()
         # It really called delete until it succeeded:
@@ -325,14 +401,14 @@ class ScalingGroupTest(HeatTestCase):
         def delete(group_id):
             1 / 0
 
-        self.patch(self.fake_auto_scale, 'delete', delete)
+        self.patchobject(self.fake_auto_scale, 'delete', side_effect=delete)
         resource = self.stack['my_group']
         err = self.assertRaises(
             exception.ResourceFailure, scheduler.TaskRunner(resource.delete))
         self.assertIsInstance(err.exc, ZeroDivisionError)
 
 
-class PolicyTest(HeatTestCase):
+class PolicyTest(common.HeatTestCase):
     policy_template = template_format.parse('''
     HeatTemplateFormatVersion: "2012-12-12"
     Description: "Rackspace Auto Scale"
@@ -350,12 +426,11 @@ class PolicyTest(HeatTestCase):
 
     def setUp(self):
         super(PolicyTest, self).setUp()
-        utils.setup_dummy_db()
         for res_name, res_class in auto_scale.resource_mapping().items():
             resource._register_class(res_name, res_class)
         self.fake_auto_scale = FakeAutoScale()
-        self.patch(clients.OpenStackClients,
-                   'auto_scale', lambda resource: self.fake_auto_scale)
+        self.patchobject(auto_scale.ScalingPolicy, 'auto_scale',
+                         return_value=self.fake_auto_scale)
 
     def _setup_test_stack(self, template):
         self.stack = utils.parse_stack(template)
@@ -441,9 +516,12 @@ class PolicyTest(HeatTestCase):
         """
         self._setup_test_stack(self.policy_template)
         resource = self.stack['my_policy']
-        template = copy.deepcopy(resource.parsed_template())
-        template['Properties']['changePercent'] = 50
-        del template['Properties']['change']
+        uprops = copy.deepcopy(dict(resource.properties.data))
+        uprops['changePercent'] = 50
+        del uprops['change']
+        template = rsrc_defn.ResourceDefinition(resource.name,
+                                                resource.type(),
+                                                uprops)
 
         scheduler.TaskRunner(resource.update, template)()
         self.assertEqual(
@@ -475,7 +553,7 @@ class PolicyTest(HeatTestCase):
         self.assertEqual({}, self.fake_auto_scale.policies)
 
 
-class WebHookTest(HeatTestCase):
+class WebHookTest(common.HeatTestCase):
     webhook_template = template_format.parse('''
     HeatTemplateFormatVersion: "2012-12-12"
     Description: "Rackspace Auto Scale"
@@ -492,12 +570,11 @@ class WebHookTest(HeatTestCase):
 
     def setUp(self):
         super(WebHookTest, self).setUp()
-        utils.setup_dummy_db()
         for res_name, res_class in auto_scale.resource_mapping().items():
             resource._register_class(res_name, res_class)
         self.fake_auto_scale = FakeAutoScale()
-        self.patch(clients.OpenStackClients,
-                   'auto_scale', lambda resource: self.fake_auto_scale)
+        self.patchobject(auto_scale.WebHook, 'auto_scale',
+                         return_value=self.fake_auto_scale)
 
     def _setup_test_stack(self, template):
         self.stack = utils.parse_stack(template)
@@ -532,9 +609,12 @@ class WebHookTest(HeatTestCase):
     def test_update(self):
         self._setup_test_stack(self.webhook_template)
         resource = self.stack['my_webhook']
-        template = copy.deepcopy(resource.parsed_template())
-        template['Properties']['metadata']['a'] = 'different!'
-        template['Properties']['name'] = 'newhook'
+        uprops = copy.deepcopy(dict(resource.properties.data))
+        uprops['metadata']['a'] = 'different!'
+        uprops['name'] = 'newhook'
+        template = rsrc_defn.ResourceDefinition(resource.name,
+                                                resource.type(),
+                                                uprops)
 
         scheduler.TaskRunner(resource.update, template)()
         self.assertEqual(
