@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,15 +14,16 @@
 import routes
 from webob import Request
 
+from oslo import i18n
+
 from heat.api.cloudwatch import versions
 from heat.api.cloudwatch import watch
 from heat.api.middleware.version_negotiation import VersionNegotiationFilter
 from heat.common import wsgi
-from heat.openstack.common import gettextutils
-from heat.openstack.common import log as logging
 
-logger = logging.getLogger(__name__)
-gettextutils.install('heat')
+# FIXME(elynn): Since install() is deprecated, we should remove it in the
+# future.
+i18n.install('heat')
 
 
 class API(wsgi.Router):
