@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,6 +14,7 @@
 from webob import exc
 
 from heat.api.openstack.v1 import util
+from heat.common import serializers
 from heat.common import wsgi
 from heat.rpc import client as rpc_client
 
@@ -27,19 +27,23 @@ class ActionController(object):
     # Define request scope (must match what is in policy.json)
     REQUEST_SCOPE = 'actions'
 
-    ACTIONS = (SUSPEND, RESUME) = ('suspend', 'resume')
+    ACTIONS = (
+        SUSPEND, RESUME, CHECK, CANCEL_UPDATE
+    ) = (
+        'suspend', 'resume', 'check', 'cancel_update'
+    )
 
     def __init__(self, options):
         self.options = options
         self.rpc_client = rpc_client.EngineClient()
 
     @util.identified_stack
-    def action(self, req, identity, body={}):
+    def action(self, req, identity, body=None):
         """
         Performs a specified action on a stack, the body is expecting to
         contain exactly one item whose key specifies the action
         """
-
+        body = body or {}
         if len(body) < 1:
             raise exc.HTTPBadRequest(_("No action specified"))
 
@@ -54,6 +58,10 @@ class ActionController(object):
             self.rpc_client.stack_suspend(req.context, identity)
         elif ac == self.RESUME:
             self.rpc_client.stack_resume(req.context, identity)
+        elif ac == self.CHECK:
+            self.rpc_client.stack_check(req.context, identity)
+        elif ac == self.CANCEL_UPDATE:
+            self.rpc_client.stack_cancel_update(req.context, identity)
         else:
             raise exc.HTTPInternalServerError(_("Unexpected action %s") % ac)
 
@@ -63,5 +71,5 @@ def create_resource(options):
     Actions action factory method.
     """
     deserializer = wsgi.JSONRequestDeserializer()
-    serializer = wsgi.JSONResponseSerializer()
+    serializer = serializers.JSONResponseSerializer()
     return wsgi.Resource(ActionController(options), deserializer, serializer)
