@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -16,12 +15,17 @@
 Stack endpoint for Heat v1 ReST API.
 """
 
+import six
+from six.moves.urllib import parse
 from webob import exc
 
 from heat.api.openstack.v1 import util
 from heat.api.openstack.v1.views import stacks_view
 from heat.common import environment_format
+from heat.common.i18n import _LW
 from heat.common import identifier
+from heat.common import param_utils
+from heat.common import serializers
 from heat.common import template_format
 from heat.common import urlfetch
 from heat.common import wsgi
@@ -29,7 +33,7 @@ from heat.openstack.common import log as logging
 from heat.rpc import api as engine_api
 from heat.rpc import client as rpc_client
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class InstantiationData(object):
@@ -53,9 +57,15 @@ class InstantiationData(object):
         'files',
     )
 
-    def __init__(self, data):
-        """Initialise from the request object."""
+    def __init__(self, data, patch=False):
+        """
+        Initialise from the request object.
+        If called from the PATCH api, insert a flag for the engine code
+        to distinguish.
+        """
         self.data = data
+        if patch:
+            self.data[engine_api.PARAM_EXISTING] = True
 
     @staticmethod
     def format_parse(data, data_type):
@@ -70,7 +80,7 @@ class InstantiationData(object):
             else:
                 return template_format.parse(data)
         except ValueError as parse_ex:
-            mdict = {'type': data_type, 'error': parse_ex}
+            mdict = {'type': data_type, 'error': six.text_type(parse_ex)}
             msg = _("%(type)s not in valid format: %(error)s") % mdict
             raise exc.HTTPBadRequest(msg)
 
@@ -84,20 +94,34 @@ class InstantiationData(object):
 
     def template(self):
         """
-        Get template file contents, either inline or from a URL, in JSON
-        or YAML format.
+        Get template file contents, either inline, from stack adopt data or
+        from a URL, in JSON or YAML format.
         """
-        if self.PARAM_TEMPLATE in self.data:
+        if engine_api.PARAM_ADOPT_STACK_DATA in self.data:
+            adopt_data = self.data[engine_api.PARAM_ADOPT_STACK_DATA]
+            try:
+                adopt_data = template_format.simple_parse(adopt_data)
+
+                if not isinstance(adopt_data, dict):
+                    raise exc.HTTPBadRequest(
+                        _('Adopt data %s invalid. Adopt data must be a dict.')
+                        % adopt_data)
+
+                return adopt_data['template']
+            except (ValueError, KeyError) as ex:
+                err_reason = _('Invalid data: %s') % ex
+                raise exc.HTTPBadRequest(err_reason)
+        elif self.PARAM_TEMPLATE in self.data:
             template_data = self.data[self.PARAM_TEMPLATE]
             if isinstance(template_data, dict):
                 return template_data
         elif self.PARAM_TEMPLATE_URL in self.data:
             url = self.data[self.PARAM_TEMPLATE_URL]
-            logger.debug('TemplateUrl %s' % url)
+            LOG.debug('TemplateUrl %s' % url)
             try:
                 template_data = urlfetch.get(url)
             except IOError as ex:
-                err_reason = _('Could not retrieve template: %s') % str(ex)
+                err_reason = _('Could not retrieve template: %s') % ex
                 raise exc.HTTPBadRequest(err_reason)
         else:
             raise exc.HTTPBadRequest(_("No template specified"))
@@ -154,16 +178,38 @@ class StackController(object):
         filter_whitelist = {
             'status': 'mixed',
             'name': 'mixed',
+            'action': 'mixed',
+            'tenant': 'mixed',
+            'username': 'mixed',
+            'owner_id': 'mixed',
         }
         whitelist = {
             'limit': 'single',
             'marker': 'single',
             'sort_dir': 'single',
             'sort_keys': 'multi',
+            'show_deleted': 'single',
+            'show_nested': 'single',
         }
         params = util.get_allowed_params(req.params, whitelist)
         filter_params = util.get_allowed_params(req.params, filter_whitelist)
 
+        show_deleted = False
+        if engine_api.PARAM_SHOW_DELETED in params:
+            params[engine_api.PARAM_SHOW_DELETED] = param_utils.extract_bool(
+                params[engine_api.PARAM_SHOW_DELETED])
+            show_deleted = params[engine_api.PARAM_SHOW_DELETED]
+        show_nested = False
+        if engine_api.PARAM_SHOW_NESTED in params:
+            params[engine_api.PARAM_SHOW_NESTED] = param_utils.extract_bool(
+                params[engine_api.PARAM_SHOW_NESTED])
+            show_nested = params[engine_api.PARAM_SHOW_NESTED]
+        # get the with_count value, if invalid, raise ValueError
+        with_count = False
+        if req.params.get('with_count'):
+            with_count = param_utils.extract_bool(
+                req.params.get('with_count'))
+
         if not filter_params:
             filter_params = None
 
@@ -173,15 +219,17 @@ class StackController(object):
                                              **params)
 
         count = None
-        if req.params.get('with_count'):
+        if with_count:
             try:
                 # Check if engine has been updated to a version with
                 # support to count_stacks before trying to use it.
                 count = self.rpc_client.count_stacks(req.context,
                                                      filters=filter_params,
-                                                     tenant_safe=tenant_safe)
+                                                     tenant_safe=tenant_safe,
+                                                     show_deleted=show_deleted,
+                                                     show_nested=show_nested)
             except AttributeError as exc:
-                logger.warning("Old Engine Version: %s" % str(exc))
+                LOG.warn(_LW("Old Engine Version: %s") % exc)
 
         return stacks_view.collection(req, stacks=stacks, count=count,
                                       tenant_safe=tenant_safe)
@@ -263,6 +311,10 @@ class StackController(object):
         if path:
             location = '/'.join([location, path])
 
+        params = req.params
+        if params:
+            location += '?%s' % parse.urlencode(params, True)
+
         raise exc.HTTPFound(location=location)
 
     @util.identified_stack
@@ -312,6 +364,22 @@ class StackController(object):
 
         raise exc.HTTPAccepted()
 
+    @util.identified_stack
+    def update_patch(self, req, identity, body):
+        """
+        Update an existing stack with a new template by patching the parameters
+        Add the flag patch to the args so the engine code can distinguish
+        """
+        data = InstantiationData(body, patch=True)
+        self.rpc_client.update_stack(req.context,
+                                     identity,
+                                     data.template(),
+                                     data.environment(),
+                                     data.files(),
+                                     data.args())
+
+        raise exc.HTTPAccepted()
+
     @util.identified_stack
     def delete(self, req, identity):
         """
@@ -378,8 +446,31 @@ class StackController(object):
         """
         return self.rpc_client.generate_template(req.context, type_name)
 
+    @util.identified_stack
+    def snapshot(self, req, identity, body):
+        name = body.get('name')
+        return self.rpc_client.stack_snapshot(req.context, identity, name)
 
-class StackSerializer(wsgi.JSONResponseSerializer):
+    @util.identified_stack
+    def show_snapshot(self, req, identity, snapshot_id):
+        snapshot = self.rpc_client.show_snapshot(
+            req.context, identity, snapshot_id)
+        return {'snapshot': snapshot}
+
+    @util.identified_stack
+    def delete_snapshot(self, req, identity, snapshot_id):
+        self.rpc_client.delete_snapshot(req.context, identity, snapshot_id)
+        raise exc.HTTPNoContent()
+
+    @util.identified_stack
+    def list_snapshots(self, req, identity):
+        return {
+            'snapshots': self.rpc_client.stack_list_snapshots(
+                req.context, identity)
+        }
+
+
+class StackSerializer(serializers.JSONResponseSerializer):
     """Handles serialization of specific controller method responses."""
 
     def _populate_response_header(self, response, location, status):
