@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -11,12 +11,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import uuid
 import itertools
-import sqlalchemy
+import uuid
 
 import migrate.changeset.constraint as constraint
-from heat.openstack.common import timeutils
+from oslo.utils import timeutils
+import sqlalchemy
 
 
 def upgrade(migrate_engine):
@@ -45,7 +45,8 @@ def upgrade(migrate_engine):
     fake_autoincrement = itertools.count(1)
 
     event_list = event_table.select().order_by(
-        sqlalchemy.sql.expression.asc(event_table.c.created_at)).execute()
+        sqlalchemy.sql.expression.asc(
+            event_table.c.created_at)).execute().fetchall()
     for event in event_list:
         values = {'tmp_id': fake_autoincrement.next(), 'uuid': event.id}
         update = event_table.update().where(
@@ -144,6 +145,9 @@ def downgrade(migrate_engine):
     cons.create()
 
     event_table.c.tmp_id.alter('id', default=lambda: str(uuid.uuid4))
+    if migrate_engine.name == 'postgresql':
+        sequence = sqlalchemy.Sequence('evt')
+        sqlalchemy.schema.DropSequence(sequence, bind=migrate_engine).execute()
 
 
 def downgrade_sqlite(migrate_engine):
@@ -176,7 +180,7 @@ def downgrade_sqlite(migrate_engine):
     event_table.create()
 
     prev_event_table = sqlalchemy.Table('event', meta, autoload=True)
-    event_list = prev_event_table.select().execute()
+    event_list = prev_event_table.select().execute().fetchall()
     for event in event_list:
         values = {
             'id': event.uuid,
