@@ -17,12 +17,12 @@ import itertools
 import sys
 
 from oslo.config import cfg
+import six
 
-from heat.openstack.common import log
 from heat.common import plugin_loader
+from heat.openstack.common import log
 
-
-logger = log.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 class PluginManager(object):
@@ -94,15 +94,15 @@ class PluginMapping(object):
                 try:
                     mapping_dict = mapping_func(*self.args, **self.kwargs)
                 except Exception:
-                    logger.error(_('Failed to load %(mapping_name)s '
-                                   'from %(module)s') % fmt_data)
+                    LOG.error(_('Failed to load %(mapping_name)s '
+                                'from %(module)s') % fmt_data)
                     raise
                 else:
                     if isinstance(mapping_dict, collections.Mapping):
                         return mapping_dict
                     elif mapping_dict is not None:
-                        logger.error(_('Invalid type for %(mapping_name)s '
-                                       'from %(module)s') % fmt_data)
+                        LOG.error(_('Invalid type for %(mapping_name)s '
+                                    'from %(module)s') % fmt_data)
 
         return {}
 
@@ -112,4 +112,5 @@ class PluginMapping(object):
         Mappings are returned as a list of (key, value) tuples.
         '''
         mod_dicts = plugin_manager.map_to_modules(self.load_from_module)
-        return itertools.chain.from_iterable(d.iteritems() for d in mod_dicts)
+        return itertools.chain.from_iterable(six.iteritems(d) for d
+                                             in mod_dicts)
