@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,18 +12,13 @@
 #    under the License.
 
 from heat.common import exception
-from heat.db import api as db_api
-from heat.engine import clients
+from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import properties
 from heat.engine import resource
 from heat.engine.resources.neutron import neutron
-from heat.engine.resources import nova_utils
 from heat.engine import scheduler
-
-if clients.neutronclient is not None:
-    from neutronclient.common.exceptions import NeutronClientException
-    from neutronclient.neutron import v2_0 as neutronV20
+from heat.engine import support
 
 
 class HealthMonitor(neutron.NeutronResource):
@@ -40,6 +34,16 @@ class HealthMonitor(neutron.NeutronResource):
         'http_method', 'expected_codes', 'url_path',
     )
 
+    ATTRIBUTES = (
+        ADMIN_STATE_UP_ATTR, DELAY_ATTR, EXPECTED_CODES_ATTR, HTTP_METHOD_ATTR,
+        MAX_RETRIES_ATTR, TIMEOUT_ATTR, TYPE_ATTR, URL_PATH_ATTR, TENANT_ID,
+        SHOW,
+    ) = (
+        'admin_state_up', 'delay', 'expected_codes', 'http_method',
+        'max_retries', 'timeout', 'type', 'url_path', 'tenant_id',
+        'show',
+    )
+
     properties_schema = {
         DELAY: properties.Schema(
             properties.Schema.INTEGER,
@@ -96,26 +100,42 @@ class HealthMonitor(neutron.NeutronResource):
         ),
     }
 
-    update_allowed_keys = ('Properties',)
-
     attributes_schema = {
-        'admin_state_up': _('The administrative state of this health '
-                            'monitor.'),
-        'delay': _('The minimum time in seconds between regular connections '
-                   'of the member.'),
-        'expected_codes': _('The list of HTTP status codes expected in '
-                            'response from the member to declare it healthy.'),
-        'http_method': _('The HTTP method used for requests by the monitor of '
-                         'type HTTP.'),
-        'max_retries': _('Number of permissible connection failures before '
-                         'changing the member status to INACTIVE.'),
-        'timeout': _('Maximum number of seconds for a monitor to wait for a '
-                     'connection to be established before it times out.'),
-        'type': _('One of predefined health monitor types.'),
-        'url_path': _('The HTTP path used in the HTTP request used by the '
-                      'monitor to test a member health.'),
-        'tenant_id': _('Tenant owning the health monitor.'),
-        'show': _('All attributes.'),
+        ADMIN_STATE_UP_ATTR: attributes.Schema(
+            _('The administrative state of this health monitor.')
+        ),
+        DELAY_ATTR: attributes.Schema(
+            _('The minimum time in seconds between regular connections '
+              'of the member.')
+        ),
+        EXPECTED_CODES_ATTR: attributes.Schema(
+            _('The list of HTTP status codes expected in response '
+              'from the member to declare it healthy.')
+        ),
+        HTTP_METHOD_ATTR: attributes.Schema(
+            _('The HTTP method used for requests by the monitor of type HTTP.')
+        ),
+        MAX_RETRIES_ATTR: attributes.Schema(
+            _('Number of permissible connection failures before changing '
+              'the member status to INACTIVE.')
+        ),
+        TIMEOUT_ATTR: attributes.Schema(
+            _('Maximum number of seconds for a monitor to wait for a '
+              'connection to be established before it times out.')
+        ),
+        TYPE_ATTR: attributes.Schema(
+            _('One of predefined health monitor types.')
+        ),
+        URL_PATH_ATTR: attributes.Schema(
+            _('The HTTP path used in the HTTP request used by the monitor '
+              'to test a member health.')
+        ),
+        TENANT_ID: attributes.Schema(
+            _('Tenant owning the health monitor.')
+        ),
+        SHOW: attributes.Schema(
+            _('All attributes.')
+        ),
     }
 
     def handle_create(self):
@@ -138,8 +158,8 @@ class HealthMonitor(neutron.NeutronResource):
     def handle_delete(self):
         try:
             self.neutron().delete_health_monitor(self.resource_id)
-        except NeutronClientException as ex:
-            self._handle_not_found_exception(ex)
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
         else:
             return self._delete_task()
 
@@ -150,10 +170,10 @@ class Pool(neutron.NeutronResource):
     """
 
     PROPERTIES = (
-        PROTOCOL, SUBNET_ID, LB_METHOD, NAME, DESCRIPTION,
+        PROTOCOL, SUBNET_ID, SUBNET, LB_METHOD, NAME, DESCRIPTION,
         ADMIN_STATE_UP, VIP, MONITORS,
     ) = (
-        'protocol', 'subnet_id', 'lb_method', 'name', 'description',
+        'protocol', 'subnet_id', 'subnet', 'lb_method', 'name', 'description',
         'admin_state_up', 'vip', 'monitors',
     )
 
@@ -173,6 +193,14 @@ class Pool(neutron.NeutronResource):
         'type', 'cookie_name',
     )
 
+    ATTRIBUTES = (
+        ADMIN_STATE_UP_ATTR, NAME_ATTR, PROTOCOL_ATTR, SUBNET_ID_ATTR,
+        LB_METHOD_ATTR, DESCRIPTION_ATTR, TENANT_ID, VIP_ATTR,
+    ) = (
+        'admin_state_up', 'name', 'protocol', 'subnet_id',
+        'lb_method', 'description', 'tenant_id', 'vip',
+    )
+
     properties_schema = {
         PROTOCOL: properties.Schema(
             properties.Schema.STRING,
@@ -183,10 +211,17 @@ class Pool(neutron.NeutronResource):
             ]
         ),
         SUBNET_ID: properties.Schema(
+            properties.Schema.STRING,
+            support_status=support.SupportStatus(
+                support.DEPRECATED,
+                _('Use property %s.') % SUBNET),
+            required=False
+        ),
+        SUBNET: properties.Schema(
             properties.Schema.STRING,
             _('The subnet for the port on which the members '
               'of the pool will be connected.'),
-            required=True
+            required=False
         ),
         LB_METHOD: properties.Schema(
             properties.Schema.STRING,
@@ -281,26 +316,41 @@ class Pool(neutron.NeutronResource):
         ),
     }
 
-    update_allowed_keys = ('Properties',)
-
     attributes_schema = {
-        'admin_state_up': _('The administrative state of this pool.'),
-        'name': _('Name of the pool.'),
-        'protocol': _('Protocol to balance.'),
-        'subnet_id': _('The subnet for the port on which the members '
-                       'of the pool will be connected.'),
-        'lb_method': _('The algorithm used to distribute load between the '
-                       'members of the pool.'),
-        'description': _('Description of the pool.'),
-        'tenant_id': _('Tenant owning the pool.'),
-        'vip': _('Vip associated with the pool.'),
+        ADMIN_STATE_UP_ATTR: attributes.Schema(
+            _('The administrative state of this pool.')
+        ),
+        NAME_ATTR: attributes.Schema(
+            _('Name of the pool.')
+        ),
+        PROTOCOL_ATTR: attributes.Schema(
+            _('Protocol to balance.')
+        ),
+        SUBNET_ID_ATTR: attributes.Schema(
+            _('The subnet for the port on which the members of the pool '
+              'will be connected.')
+        ),
+        LB_METHOD_ATTR: attributes.Schema(
+            _('The algorithm used to distribute load between the members '
+              'of the pool.')
+        ),
+        DESCRIPTION_ATTR: attributes.Schema(
+            _('Description of the pool.')
+        ),
+        TENANT_ID: attributes.Schema(
+            _('Tenant owning the pool.')
+        ),
+        VIP_ATTR: attributes.Schema(
+            _('Vip associated with the pool.')
+        ),
     }
 
     def validate(self):
         res = super(Pool, self).validate()
         if res:
             return res
-
+        self._validate_depr_property_required(
+            self.properties, self.SUBNET, self.SUBNET_ID)
         session_p = self.properties[self.VIP].get(self.VIP_SESSION_PERSISTENCE)
         if session_p is None:
             # session persistence is not configured, skip validation
@@ -319,6 +369,8 @@ class Pool(neutron.NeutronResource):
         properties = self.prepare_properties(
             self.properties,
             self.physical_resource_name())
+        self.client_plugin().resolve_subnet(
+            properties, self.SUBNET, 'subnet_id')
         vip_properties = properties.pop(self.VIP)
         monitors = properties.pop(self.MONITORS)
         client = self.neutron()
@@ -341,49 +393,53 @@ class Pool(neutron.NeutronResource):
         vip_arguments['protocol'] = self.properties[self.PROTOCOL]
 
         if vip_arguments.get(self.VIP_SUBNET) is None:
-            vip_arguments['subnet_id'] = self.properties[self.SUBNET_ID]
+            vip_arguments['subnet_id'] = properties[self.SUBNET_ID]
         else:
-            vip_arguments[
-                'subnet_id'] = neutronV20.find_resourceid_by_name_or_id(
-                    self.neutron(),
-                    'subnet',
-                    vip_arguments.pop(self.VIP_SUBNET))
+            vip_arguments['subnet_id'] = self.client_plugin().resolve_subnet(
+                vip_arguments, self.VIP_SUBNET, 'subnet_id')
 
         vip_arguments['pool_id'] = pool['id']
         vip = client.create_vip({'vip': vip_arguments})['vip']
 
-        self.metadata = {'vip': vip['id']}
+        self.metadata_set({'vip': vip['id']})
 
     def _show_resource(self):
         return self.neutron().show_pool(self.resource_id)['pool']
 
     def check_create_complete(self, data):
         attributes = self._show_resource()
-        if attributes['status'] == 'PENDING_CREATE':
+        status = attributes['status']
+        if status == 'PENDING_CREATE':
             return False
-        elif attributes['status'] == 'ACTIVE':
+        elif status == 'ACTIVE':
             vip_attributes = self.neutron().show_vip(
-                self.metadata['vip'])['vip']
-            if vip_attributes['status'] == 'PENDING_CREATE':
+                self.metadata_get()['vip'])['vip']
+            vip_status = vip_attributes['status']
+            if vip_status == 'PENDING_CREATE':
                 return False
-            elif vip_attributes['status'] == 'ACTIVE':
+            if vip_status == 'ACTIVE':
                 return True
-            raise exception.Error(
-                _('neutron reported unexpected vip resource[%(name)s] '
-                  'status[%(status)s]') %
-                {'name': vip_attributes['name'],
-                 'status': vip_attributes['status']})
-        raise exception.Error(
-            _('neutron reported unexpected pool resource[%(name)s] '
-              'status[%(status)s]') %
-            {'name': attributes['name'],
-             'status': attributes['status']})
+            if vip_status == 'ERROR':
+                raise resource.ResourceInError(
+                    resource_status=vip_status,
+                    status_reason=_('error in vip'))
+            raise resource.ResourceUnknownStatus(
+                resource_status=vip_status,
+                result=_('Pool creation failed due to vip'))
+        elif status == 'ERROR':
+            raise resource.ResourceInError(
+                resource_status=status,
+                status_reason=_('error in pool'))
+        else:
+            raise resource.ResourceUnknownStatus(
+                resource_status=status,
+                result=_('Pool creation failed'))
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
             client = self.neutron()
-            monitors = set(prop_diff.pop(self.MONITORS, []))
-            if monitors:
+            if self.MONITORS in prop_diff:
+                monitors = set(prop_diff.pop(self.MONITORS))
                 old_monitors = set(self.properties[self.MONITORS])
                 for monitor in old_monitors - monitors:
                     client.disassociate_health_monitor(self.resource_id,
@@ -396,8 +452,8 @@ class Pool(neutron.NeutronResource):
                 client.update_pool(self.resource_id, {'pool': prop_diff})
 
     def _resolve_attribute(self, name):
-        if name == 'vip':
-            return self.neutron().show_vip(self.metadata['vip'])['vip']
+        if name == self.VIP_ATTR:
+            return self.neutron().show_vip(self.metadata_get()['vip'])['vip']
         return super(Pool, self)._resolve_attribute(name)
 
     def _confirm_vip_delete(self):
@@ -405,24 +461,24 @@ class Pool(neutron.NeutronResource):
         while True:
             try:
                 yield
-                client.show_vip(self.metadata['vip'])
-            except NeutronClientException as ex:
-                self._handle_not_found_exception(ex)
+                client.show_vip(self.metadata_get()['vip'])
+            except Exception as ex:
+                self.client_plugin().ignore_not_found(ex)
                 break
 
     def handle_delete(self):
         checkers = []
-        if self.metadata:
+        if self.metadata_get():
             try:
-                self.neutron().delete_vip(self.metadata['vip'])
-            except NeutronClientException as ex:
-                self._handle_not_found_exception(ex)
+                self.neutron().delete_vip(self.metadata_get()['vip'])
+            except Exception as ex:
+                self.client_plugin().ignore_not_found(ex)
             else:
                 checkers.append(scheduler.TaskRunner(self._confirm_vip_delete))
         try:
             self.neutron().delete_pool(self.resource_id)
-        except NeutronClientException as ex:
-            self._handle_not_found_exception(ex)
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
         else:
             checkers.append(scheduler.TaskRunner(self._confirm_delete))
         return checkers
@@ -448,6 +504,14 @@ class PoolMember(neutron.NeutronResource):
         'pool_id', 'address', 'protocol_port', 'weight', 'admin_state_up',
     )
 
+    ATTRIBUTES = (
+        ADMIN_STATE_UP_ATTR, TENANT_ID, WEIGHT_ATTR, ADDRESS_ATTR,
+        POOL_ID_ATTR, PROTOCOL_PORT_ATTR, SHOW,
+    ) = (
+        'admin_state_up', 'tenant_id', 'weight', 'address',
+        'pool_id', 'protocol_port', 'show',
+    )
+
     properties_schema = {
         POOL_ID: properties.Schema(
             properties.Schema.STRING,
@@ -485,19 +549,30 @@ class PoolMember(neutron.NeutronResource):
     }
 
     attributes_schema = {
-        'admin_state_up': _('The administrative state of this pool '
-                            'member.'),
-        'tenant_id': _('Tenant owning the pool member.'),
-        'weight': _('Weight of the pool member in the pool.'),
-        'address': _('IP address of the pool member.'),
-        'pool_id': _('The ID of the load balancing pool.'),
-        'protocol_port': _('TCP port on which the pool member listens for'
-                           'requests or connections.'),
-        'show': _('All attributes.'),
+        ADMIN_STATE_UP_ATTR: attributes.Schema(
+            _('The administrative state of this pool member.')
+        ),
+        TENANT_ID: attributes.Schema(
+            _('Tenant owning the pool member.')
+        ),
+        WEIGHT_ATTR: attributes.Schema(
+            _('Weight of the pool member in the pool.')
+        ),
+        ADDRESS_ATTR: attributes.Schema(
+            _('IP address of the pool member.')
+        ),
+        POOL_ID_ATTR: attributes.Schema(
+            _('The ID of the load balancing pool.')
+        ),
+        PROTOCOL_PORT_ATTR: attributes.Schema(
+            _('TCP port on which the pool member listens for requests or '
+              'connections.')
+        ),
+        SHOW: attributes.Schema(
+            _('All attributes.')
+        ),
     }
 
-    update_allowed_keys = ('Properties',)
-
     def handle_create(self):
         pool = self.properties[self.POOL_ID]
         client = self.neutron()
@@ -531,8 +606,8 @@ class PoolMember(neutron.NeutronResource):
         client = self.neutron()
         try:
             client.delete_member(self.resource_id)
-        except NeutronClientException as ex:
-            self._handle_not_found_exception(ex)
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
         else:
             return self._delete_task()
 
@@ -563,72 +638,73 @@ class LoadBalancer(resource.Resource):
         MEMBERS: properties.Schema(
             properties.Schema.LIST,
             _('The list of Nova server IDs load balanced.'),
-            default=[],
             update_allowed=True
         ),
     }
 
-    update_allowed_keys = ('Properties',)
+    default_client_name = 'neutron'
 
     def handle_create(self):
         pool = self.properties[self.POOL_ID]
         client = self.neutron()
-        nova_client = self.nova()
         protocol_port = self.properties[self.PROTOCOL_PORT]
 
-        for member in self.properties.get(self.MEMBERS):
-            address = nova_utils.server_to_ipaddress(nova_client, member)
+        for member in self.properties[self.MEMBERS] or []:
+            address = self.client_plugin('nova').server_to_ipaddress(member)
             lb_member = client.create_member({
                 'member': {
                     'pool_id': pool,
                     'address': address,
                     'protocol_port': protocol_port}})['member']
-            db_api.resource_data_set(self, member, lb_member['id'])
+            self.data_set(member, lb_member['id'])
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        if self.MEMBERS in prop_diff:
-            members = set(prop_diff[self.MEMBERS])
-            rd_members = db_api.resource_data_get_all(self)
+        new_props = json_snippet.properties(self.properties_schema,
+                                            self.context)
+
+        # Valid use cases are:
+        # - Membership controlled by members property in template
+        # - Empty members property in template; membership controlled by
+        #   "updates" triggered from autoscaling group.
+        # Mixing the two will lead to undefined behaviour.
+        if (self.MEMBERS in prop_diff and
+                (self.properties[self.MEMBERS] is not None or
+                 new_props[self.MEMBERS] is not None)):
+            members = set(new_props[self.MEMBERS] or [])
+            rd_members = self.data()
             old_members = set(rd_members.keys())
             client = self.neutron()
             for member in old_members - members:
                 member_id = rd_members[member]
                 try:
                     client.delete_member(member_id)
-                except NeutronClientException as ex:
-                    if ex.status_code != 404:
-                        raise ex
-                db_api.resource_data_delete(self, member)
+                except Exception as ex:
+                    self.client_plugin().ignore_not_found(ex)
+                self.data_delete(member)
             pool = self.properties[self.POOL_ID]
-            nova_client = self.nova()
             protocol_port = self.properties[self.PROTOCOL_PORT]
             for member in members - old_members:
-                address = nova_utils.server_to_ipaddress(nova_client, member)
+                address = self.client_plugin('nova').server_to_ipaddress(
+                    member)
                 lb_member = client.create_member({
                     'member': {
                         'pool_id': pool,
                         'address': address,
                         'protocol_port': protocol_port}})['member']
-                db_api.resource_data_set(self, member, lb_member['id'])
+                self.data_set(member, lb_member['id'])
 
     def handle_delete(self):
         client = self.neutron()
-        for member in self.properties.get(self.MEMBERS):
+        for member in self.properties[self.MEMBERS] or []:
+            member_id = self.data().get(member)
             try:
-                member_id = db_api.resource_data_get(self, member)
                 client.delete_member(member_id)
-                db_api.resource_data_delete(self, member)
-            except NeutronClientException as ex:
-                if ex.status_code != 404:
-                    raise ex
-            except exception.NotFound:
-                pass
+            except Exception as ex:
+                self.client_plugin().ignore_not_found(ex)
+            self.data_delete(member)
 
 
 def resource_mapping():
-    if clients.neutronclient is None:
-        return {}
-
     return {
         'OS::Neutron::HealthMonitor': HealthMonitor,
         'OS::Neutron::Pool': Pool,
