@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,29 +12,27 @@
 #    under the License.
 
 import copy
-
-from oslo.config import cfg
 import uuid
 
+from oslo.config import cfg
+
 from heat.common import exception
-from heat.db import api as db_api
-from heat.engine import clients
+from heat.common.i18n import _
+from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import properties
 from heat.engine import resource
 from heat.engine.resources.neutron import subnet
-from heat.engine.resources import nova_utils
-from heat.engine.resources.software_config import software_config as sc
 from heat.engine import scheduler
 from heat.engine import stack_user
 from heat.engine import support
-from heat.openstack.common.gettextutils import _
+from heat.openstack.common import jsonutils
 from heat.openstack.common import log as logging
 from heat.openstack.common import uuidutils
 
 cfg.CONF.import_opt('instance_user', 'heat.common.config')
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class Server(stack_user.StackUser):
@@ -81,9 +78,17 @@ class Server(stack_user.StackUser):
     )
 
     _SOFTWARE_CONFIG_TRANSPORTS = (
-        POLL_SERVER_CFN, POLL_SERVER_HEAT
+        POLL_SERVER_CFN, POLL_SERVER_HEAT, POLL_TEMP_URL
     ) = (
-        'POLL_SERVER_CFN', 'POLL_SERVER_HEAT'
+        'POLL_SERVER_CFN', 'POLL_SERVER_HEAT', 'POLL_TEMP_URL'
+    )
+
+    ATTRIBUTES = (
+        NAME_ATTR, SHOW, ADDRESSES, NETWORKS_ATTR, FIRST_ADDRESS,
+        INSTANCE_NAME, ACCESSIPV4, ACCESSIPV6,
+    ) = (
+        'name', 'show', 'addresses', 'networks', 'first_address',
+        'instance_name', 'accessIPv4', 'accessIPv6',
     )
 
     properties_schema = {
@@ -258,7 +263,8 @@ class Server(stack_user.StackUser):
               'the cfn API action DescribeStackResource authenticated with '
               'the provided keypair. POLL_SERVER_HEAT will allow calls to '
               'the Heat API resource-show using the provided keystone '
-              'credentials.'),
+              'credentials. POLL_TEMP_URL will create and populate a '
+              'Swift TempURL with metadata for polling.'),
             default=POLL_SERVER_CFN,
             constraints=[
                 constraints.AllowedValues(_SOFTWARE_CONFIG_TRANSPORTS),
@@ -274,8 +280,8 @@ class Server(stack_user.StackUser):
             _('A UUID for the set of servers being requested.')
         ),
         CONFIG_DRIVE: properties.Schema(
-            properties.Schema.STRING,
-            _('value for config drive either boolean, or volume-id.')
+            properties.Schema.BOOLEAN,
+            _('If True, enable config drive on the server.')
         ),
         DISK_CONFIG: properties.Schema(
             properties.Schema.STRING,
@@ -300,57 +306,68 @@ class Server(stack_user.StackUser):
     }
 
     attributes_schema = {
-        'show': _('A dict of all server details as returned by the API.'),
-        'addresses': _('A dict of all network addresses with corresponding'
-                       'port_id.'),
-        'networks': _('A dict of assigned network addresses of the form: '
-                      '{"public": [ip1, ip2...], "private": [ip3, ip4]}.'),
-        'first_address': _('Convenience attribute to fetch the first '
-                           'assigned network address, or an '
-                           'empty string if nothing has been assigned '
-                           'at this time. Result may not be predictable '
-                           'if the server has addresses from more than one '
-                           'network.'),
-        'instance_name': _('AWS compatible instance name.'),
-        'accessIPv4': _('The manually assigned alternative public IPv4 '
-                        'address of the server.'),
-        'accessIPv6': _('The manually assigned alternative public IPv6 '
-                        'address of the server.'),
+        NAME_ATTR: attributes.Schema(
+            _('Name of the server.')
+        ),
+        SHOW: attributes.Schema(
+            _('A dict of all server details as returned by the API.')
+        ),
+        ADDRESSES: attributes.Schema(
+            _('A dict of all network addresses with corresponding port_id.')
+        ),
+        NETWORKS_ATTR: attributes.Schema(
+            _('A dict of assigned network addresses of the form: '
+              '{"public": [ip1, ip2...], "private": [ip3, ip4]}.')
+        ),
+        FIRST_ADDRESS: attributes.Schema(
+            _('Convenience attribute to fetch the first assigned network '
+              'address, or an empty string if nothing has been assigned at '
+              'this time. Result may not be predictable if the server has '
+              'addresses from more than one network.'),
+            support_status=support.SupportStatus(
+                status=support.DEPRECATED,
+                message=_('Use the networks attribute instead of '
+                          'first_address. For example: "{get_attr: '
+                          '[<server name>, networks, <network name>, 0]}"')
+            )
+        ),
+        INSTANCE_NAME: attributes.Schema(
+            _('AWS compatible instance name.')
+        ),
+        ACCESSIPV4: attributes.Schema(
+            _('The manually assigned alternative public IPv4 address '
+              'of the server.')
+        ),
+        ACCESSIPV6: attributes.Schema(
+            _('The manually assigned alternative public IPv6 address '
+              'of the server.')
+        ),
     }
 
-    update_allowed_keys = ('Metadata', 'Properties')
-
     # Server host name limit to 53 characters by due to typical default
     # linux HOST_NAME_MAX of 64, minus the .novalocal appended to the name
     physical_resource_name_limit = 53
 
+    default_client_name = 'nova'
+
     def __init__(self, name, json_snippet, stack):
         super(Server, self).__init__(name, json_snippet, stack)
         if self.user_data_software_config():
             self._register_access_key()
 
-    def physical_resource_name(self):
+    def _server_name(self):
         name = self.properties.get(self.NAME)
         if name:
             return name
 
-        return super(Server, self).physical_resource_name()
+        return self.physical_resource_name()
 
-    def _personality(self):
+    def _config_drive(self):
         # This method is overridden by the derived CloudServer resource
-        return self.properties.get(self.PERSONALITY)
+        return self.properties.get(self.CONFIG_DRIVE)
 
-    def _key_name(self):
-        # This method is overridden by the derived CloudServer resource
-        return self.properties.get(self.KEY_NAME)
-
-    @staticmethod
-    def _get_deployments_metadata(heatclient, server_id):
-        return heatclient.software_deployments.metadata(
-            server_id=server_id)
-
-    def _build_deployments_metadata(self):
-        meta = {}
+    def _populate_deployments_metadata(self, meta):
+        meta['deployments'] = meta.get('deployments', [])
         if self.transport_poll_server_heat():
             meta['os-collect-config'] = {'heat': {
                 'user_id': self._get_user_id(),
@@ -368,15 +385,25 @@ class Server(stack_user.StackUser):
                 'stack_name': self.stack.name,
                 'path': '%s.Metadata' % self.name}
             }
+        elif self.transport_poll_temp_url():
+            container = self.physical_resource_name()
+            object_name = str(uuid.uuid4())
 
-        deployments = []
-        # cannot query the deployments if the nova server does
-        # not exist yet
-        if self.resource_id:
-            deployments = self._get_deployments_metadata(
-                self.heat(), self.resource_id)
-        meta['deployments'] = deployments
-        return meta
+            self.client('swift').put_container(container)
+
+            url = self.client_plugin('swift').get_temp_url(
+                container, object_name, method='GET')
+            put_url = self.client_plugin('swift').get_temp_url(
+                container, object_name)
+            self.data_set('metadata_put_url', put_url)
+            self.data_set('metadata_object_name', object_name)
+
+            meta['os-collect-config'] = {'request': {
+                'metadata_url': url}
+            }
+            self.client('swift').put_object(
+                container, object_name, jsonutils.dumps(meta))
+        self.metadata_set(meta)
 
     def _register_access_key(self):
         '''
@@ -405,46 +432,22 @@ class Server(stack_user.StackUser):
 
     @property
     def access_key(self):
-        try:
-            return db_api.resource_data_get(self, 'access_key')
-        except exception.NotFound:
-            pass
+        return self.data().get('access_key')
 
     @property
     def secret_key(self):
-        try:
-            return db_api.resource_data_get(self, 'secret_key')
-        except exception.NotFound:
-            pass
+        return self.data().get('secret_key')
 
     @property
     def password(self):
-        try:
-            return db_api.resource_data_get(self, 'password')
-        except exception.NotFound:
-            pass
+        return self.data().get('password')
 
     @password.setter
     def password(self, password):
-        try:
-            if password is None:
-                db_api.resource_data_delete(self, 'password')
-            else:
-                db_api.resource_data_set(self, 'password', password, True)
-        except exception.NotFound:
-            pass
-
-    @property
-    def metadata(self):
-        if self.user_data_software_config():
-            return self._build_deployments_metadata()
+        if password is None:
+            self.data_delete('password')
         else:
-            return self._metadata
-
-    @metadata.setter
-    def metadata(self, metadata):
-        if not self.user_data_software_config():
-            self._metadata = metadata
+            self.data_set('password', password, True)
 
     def user_data_raw(self):
         return self.properties.get(self.USER_DATA_FORMAT) == self.RAW
@@ -461,6 +464,10 @@ class Server(stack_user.StackUser):
         return self.properties.get(
             self.SOFTWARE_CONFIG_TRANSPORT) == self.POLL_SERVER_HEAT
 
+    def transport_poll_temp_url(self):
+        return self.properties.get(
+            self.SOFTWARE_CONFIG_TRANSPORT) == self.POLL_TEMP_URL
+
     def handle_create(self):
         security_groups = self.properties.get(self.SECURITY_GROUPS)
 
@@ -470,14 +477,16 @@ class Server(stack_user.StackUser):
             if uuidutils.is_uuid_like(ud_content):
                 # attempt to load the userdata from software config
                 try:
-                    ud_content = sc.SoftwareConfig.get_software_config(
-                        self.heat(), ud_content)
-                except exception.SoftwareConfigMissing:
-                    # no config was found, so do not modify the user_data
-                    pass
+                    ud_content = self.heat().software_configs.get(
+                        ud_content).config
+                except Exception as ex:
+                    self.client_plugin('heat').ignore_not_found(ex)
+
+        metadata = self.metadata_get(True) or {}
 
         if self.user_data_software_config():
             self._create_transport_credentials()
+            self._populate_deployments_metadata(metadata)
 
         if self.properties[self.ADMIN_USER]:
             instance_user = self.properties[self.ADMIN_USER]
@@ -486,8 +495,8 @@ class Server(stack_user.StackUser):
         else:
             instance_user = None
 
-        userdata = nova_utils.build_userdata(
-            self,
+        userdata = self.client_plugin().build_userdata(
+            metadata,
             ud_content,
             instance_user=instance_user,
             user_data_format=user_data_format)
@@ -497,30 +506,32 @@ class Server(stack_user.StackUser):
 
         image = self.properties.get(self.IMAGE)
         if image:
-            image = nova_utils.get_image_id(self.nova(), image)
+            image = self.client_plugin('glance').get_image_id(image)
 
-        flavor_id = nova_utils.get_flavor_id(self.nova(), flavor)
+        flavor_id = self.client_plugin().get_flavor_id(flavor)
 
         instance_meta = self.properties.get(self.METADATA)
         if instance_meta is not None:
-            instance_meta = nova_utils.meta_serialize(instance_meta)
+            instance_meta = self.client_plugin().meta_serialize(
+                instance_meta)
 
         scheduler_hints = self.properties.get(self.SCHEDULER_HINTS)
         nics = self._build_nics(self.properties.get(self.NETWORKS))
         block_device_mapping = self._build_block_device_mapping(
             self.properties.get(self.BLOCK_DEVICE_MAPPING))
         reservation_id = self.properties.get(self.RESERVATION_ID)
-        config_drive = self.properties.get(self.CONFIG_DRIVE)
         disk_config = self.properties.get(self.DISK_CONFIG)
         admin_pass = self.properties.get(self.ADMIN_PASS) or None
+        personality_files = self.properties.get(self.PERSONALITY)
+        key_name = self.properties.get(self.KEY_NAME)
 
         server = None
         try:
             server = self.nova().servers.create(
-                name=self.physical_resource_name(),
+                name=self._server_name(),
                 image=image,
                 flavor=flavor_id,
-                key_name=self._key_name(),
+                key_name=key_name,
                 security_groups=security_groups,
                 userdata=userdata,
                 meta=instance_meta,
@@ -529,12 +540,12 @@ class Server(stack_user.StackUser):
                 availability_zone=availability_zone,
                 block_device_mapping=block_device_mapping,
                 reservation_id=reservation_id,
-                config_drive=config_drive,
+                config_drive=self._config_drive(),
                 disk_config=disk_config,
-                files=self._personality(),
+                files=personality_files,
                 admin_pass=admin_pass)
         finally:
-            # Avoid a race condition where the thread could be cancelled
+            # Avoid a race condition where the thread could be canceled
             # before the ID is stored
             if server is not None:
                 self.resource_id_set(server.id)
@@ -545,26 +556,28 @@ class Server(stack_user.StackUser):
         return self._check_active(server)
 
     def _check_active(self, server):
+        cp = self.client_plugin()
+        status = cp.get_status(server)
+        if status != 'ACTIVE':
+            cp.refresh_server(server)
+            status = cp.get_status(server)
 
-        if server.status != 'ACTIVE':
-            nova_utils.refresh_server(server)
-
-        # Some clouds append extra (STATUS) strings to the status
-        short_server_status = server.status.split('(')[0]
-        if short_server_status in nova_utils.deferred_server_statuses:
+        if status in cp.deferred_server_statuses:
             return False
-        elif server.status == 'ACTIVE':
+        elif status == 'ACTIVE':
             return True
-        elif server.status == 'ERROR':
-            exc = exception.Error(_('Creation of server %s failed.') %
-                                  server.name)
-            raise exc
+        elif status == 'ERROR':
+            fault = getattr(server, 'fault', {})
+            raise resource.ResourceInError(
+                resource_status=status,
+                status_reason=_("Message: %(message)s, Code: %(code)s") % {
+                    'message': fault.get('message', _('Unknown')),
+                    'code': fault.get('code', _('Unknown'))
+                })
         else:
-            exc = exception.Error(_('Creation of server %(server)s failed '
-                                    'with unknown status: %(status)s') %
-                                  dict(server=server.name,
-                                       status=server.status))
-            raise exc
+            raise resource.ResourceUnknownStatus(
+                resource_status=server.status,
+                result=_('Server is not active'))
 
     @classmethod
     def _build_block_device_mapping(cls, bdm):
@@ -584,8 +597,10 @@ class Server(stack_user.StackUser):
 
             volume_size = mapping.get(cls.BLOCK_DEVICE_MAPPING_VOLUME_SIZE)
             delete = mapping.get(cls.BLOCK_DEVICE_MAPPING_DELETE_ON_TERM)
-            if volume_size or delete:
-                mapping_parts.append(str(volume_size or 0))
+            if volume_size:
+                mapping_parts.append(str(volume_size))
+            else:
+                mapping_parts.append('')
             if delete:
                 mapping_parts.append(str(delete))
 
@@ -631,26 +646,27 @@ class Server(stack_user.StackUser):
         return nets
 
     def _resolve_attribute(self, name):
-        if name == 'first_address':
-            return nova_utils.server_to_ipaddress(
-                self.nova(), self.resource_id) or ''
+        if name == self.FIRST_ADDRESS:
+            return self.client_plugin().server_to_ipaddress(
+                self.resource_id) or ''
+        if name == self.NAME_ATTR:
+            return self._server_name()
         try:
             server = self.nova().servers.get(self.resource_id)
-        except clients.novaclient.exceptions.NotFound as ex:
-            logger.warn(_('Instance (%(server)s) not found: %(ex)s') % {
-                        'server': self.resource_id, 'ex': str(ex)})
+        except Exception as e:
+            self.client_plugin().ignore_not_found(e)
             return ''
-        if name == 'addresses':
+        if name == self.ADDRESSES:
             return self._add_port_for_address(server)
-        if name == 'networks':
+        if name == self.NETWORKS_ATTR:
             return server.networks
-        if name == 'instance_name':
+        if name == self.INSTANCE_NAME:
             return server._info.get('OS-EXT-SRV-ATTR:instance_name')
-        if name == 'accessIPv4':
+        if name == self.ACCESSIPV4:
             return server.accessIPv4
-        if name == 'accessIPv6':
+        if name == self.ACCESSIPV6:
             return server.accessIPv6
-        if name == 'show':
+        if name == self.SHOW:
             return server._info
 
     def add_dependencies(self, deps):
@@ -676,12 +692,11 @@ class Server(stack_user.StackUser):
     def _get_network_matches(self, old_networks, new_networks):
         # make new_networks similar on old_networks
         for net in new_networks:
-            for key in ('port', 'network', 'fixed_ip'):
+            for key in ('port', 'network', 'fixed_ip', 'uuid'):
                 net.setdefault(key)
         # find matches and remove them from old and new networks
         not_updated_networks = []
         for net in old_networks:
-            net.pop('uuid', None)
             if net in new_networks:
                 new_networks.remove(net)
                 not_updated_networks.append(net)
@@ -721,16 +736,15 @@ class Server(stack_user.StackUser):
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if 'Metadata' in tmpl_diff:
-            self.metadata = tmpl_diff['Metadata']
+            self.metadata_set(tmpl_diff['Metadata'])
 
         checkers = []
         server = None
 
         if self.METADATA in prop_diff:
             server = self.nova().servers.get(self.resource_id)
-            nova_utils.meta_update(self.nova(),
-                                   server,
-                                   prop_diff[self.METADATA])
+            self.client_plugin().meta_update(server,
+                                             prop_diff[self.METADATA])
 
         if self.FLAVOR in prop_diff:
 
@@ -742,11 +756,11 @@ class Server(stack_user.StackUser):
                 raise resource.UpdateReplace(self.name)
 
             flavor = prop_diff[self.FLAVOR]
-            flavor_id = nova_utils.get_flavor_id(self.nova(), flavor)
+            flavor_id = self.client_plugin().get_flavor_id(flavor)
             if not server:
                 server = self.nova().servers.get(self.resource_id)
-            checker = scheduler.TaskRunner(nova_utils.resize, server, flavor,
-                                           flavor_id)
+            checker = scheduler.TaskRunner(self.client_plugin().resize,
+                                           server, flavor, flavor_id)
             checkers.append(checker)
 
         if self.IMAGE in prop_diff:
@@ -756,20 +770,20 @@ class Server(stack_user.StackUser):
             if image_update_policy == 'REPLACE':
                 raise resource.UpdateReplace(self.name)
             image = prop_diff[self.IMAGE]
-            image_id = nova_utils.get_image_id(self.nova(), image)
+            image_id = self.client_plugin('glance').get_image_id(image)
             if not server:
                 server = self.nova().servers.get(self.resource_id)
             preserve_ephemeral = (
                 image_update_policy == 'REBUILD_PRESERVE_EPHEMERAL')
             checker = scheduler.TaskRunner(
-                nova_utils.rebuild, server, image_id,
+                self.client_plugin().rebuild, server, image_id,
                 preserve_ephemeral=preserve_ephemeral)
             checkers.append(checker)
 
         if self.NAME in prop_diff:
             if not server:
                 server = self.nova().servers.get(self.resource_id)
-            nova_utils.rename(server, prop_diff[self.NAME])
+            self.client_plugin().rename(server, prop_diff[self.NAME])
 
         if self.NETWORKS in prop_diff:
             new_networks = prop_diff.get(self.NETWORKS)
@@ -854,7 +868,15 @@ class Server(stack_user.StackUser):
         Refresh the metadata if new_metadata is None
         '''
         if new_metadata is None:
-            self.metadata = self.parsed_template('Metadata')
+            # Re-resolve the template metadata and merge it with the
+            # current resource metadata.  This is necessary because the
+            # attributes referenced in the template metadata may change
+            # and the resource itself adds keys to the metadata which
+            # are not specified in the template (e.g the deployments data)
+            meta = self.metadata_get(refresh=True) or {}
+            tmpl_meta = self.t.metadata()
+            meta.update(tmpl_meta)
+            self.metadata_set(meta)
 
     @staticmethod
     def _check_maximum(count, maximum, msg):
@@ -917,20 +939,20 @@ class Server(stack_user.StackUser):
                                    server=self.name)
                 raise exception.StackValidationFailed(message=msg)
             elif network.get(self.NETWORK_UUID):
-                logger.info(_('For the server "%(server)s" the "%(uuid)s" '
-                              'property is set to network "%(network)s". '
-                              '"%(uuid)s" property is deprecated. Use '
-                              '"%(id)s"  property instead.'
-                              '') % dict(uuid=self.NETWORK_UUID,
-                                         id=self.NETWORK_ID,
-                                         network=network[self.NETWORK_ID],
-                                         server=self.name))
+                LOG.info(_('For the server "%(server)s" the "%(uuid)s" '
+                           'property is set to network "%(network)s". '
+                           '"%(uuid)s" property is deprecated. Use '
+                           '"%(id)s"  property instead.')
+                         % dict(uuid=self.NETWORK_UUID,
+                                id=self.NETWORK_ID,
+                                network=network[self.NETWORK_ID],
+                                server=self.name))
 
         # retrieve provider's absolute limits if it will be needed
         metadata = self.properties.get(self.METADATA)
-        personality = self._personality()
+        personality = self.properties.get(self.PERSONALITY)
         if metadata is not None or personality:
-            limits = nova_utils.absolute_limits(self.nova())
+            limits = self.client_plugin().absolute_limits()
 
         # if 'security_groups' present for the server and explict 'port'
         # in one or more entries in 'networks', raise validation error
@@ -966,25 +988,43 @@ class Server(stack_user.StackUser):
                 self._check_maximum(len(bytes(contents)),
                                     limits['maxPersonalitySize'], msg)
 
+    def _delete_temp_url(self):
+        object_name = self.data().get('metadata_object_name')
+        if not object_name:
+            return
+        try:
+            container = self.physical_resource_name()
+            swift = self.client('swift')
+            swift.delete_object(container, object_name)
+            headers = swift.head_container(container)
+            if int(headers['x-container-object-count']) == 0:
+                swift.delete_container(container)
+        except Exception as ex:
+            self.client_plugin('swift').ignore_not_found(ex)
+
     def handle_delete(self):
-        '''
-        Delete a server, blocking until it is disposed by OpenStack
-        '''
+
         if self.resource_id is None:
             return
 
         if self.user_data_software_config():
             self._delete_user()
+            self._delete_temp_url()
 
         try:
             server = self.nova().servers.get(self.resource_id)
-        except clients.novaclient.exceptions.NotFound:
-            pass
+        except Exception as e:
+            self.client_plugin().ignore_not_found(e)
         else:
-            delete = scheduler.TaskRunner(nova_utils.delete_server, server)
-            delete(wait_time=0.2)
+            deleter = scheduler.TaskRunner(self.client_plugin().delete_server,
+                                           server)
+            deleter.start()
+            return deleter
 
-        self.resource_id_set(None)
+    def check_delete_complete(self, deleter):
+        if deleter is None or deleter.step():
+            return True
+        return False
 
     def handle_suspend(self):
         '''
@@ -998,11 +1038,14 @@ class Server(stack_user.StackUser):
 
         try:
             server = self.nova().servers.get(self.resource_id)
-        except clients.novaclient.exceptions.NotFound:
-            raise exception.NotFound(_('Failed to find server %s') %
-                                     self.resource_id)
+        except Exception as e:
+            if self.client_plugin().is_not_found(e):
+                raise exception.NotFound(_('Failed to find server %s') %
+                                         self.resource_id)
+            else:
+                raise
         else:
-            logger.debug(_('suspending server %s') % self.resource_id)
+            LOG.debug('suspending server %s' % self.resource_id)
             # We want the server.suspend to happen after the volume
             # detachement has finished, so pass both tasks and the server
             suspend_runner = scheduler.TaskRunner(server.suspend)
@@ -1018,11 +1061,11 @@ class Server(stack_user.StackUser):
             if server.status == 'SUSPENDED':
                 return True
 
-            nova_utils.refresh_server(server)
-            logger.debug(_('%(name)s check_suspend_complete status '
-                         '= %(status)s') % {
-                             'name': self.name, 'status': server.status})
-            if server.status in list(nova_utils.deferred_server_statuses +
+            cp = self.client_plugin()
+            cp.refresh_server(server)
+            LOG.debug('%(name)s check_suspend_complete status = %(status)s'
+                      % {'name': self.name, 'status': server.status})
+            if server.status in list(cp.deferred_server_statuses +
                                      ['ACTIVE']):
                 return server.status == 'SUSPENDED'
             else:
@@ -1044,32 +1087,48 @@ class Server(stack_user.StackUser):
 
         try:
             server = self.nova().servers.get(self.resource_id)
-        except clients.novaclient.exceptions.NotFound:
-            raise exception.NotFound(_('Failed to find server %s') %
-                                     self.resource_id)
+        except Exception as e:
+            if self.client_plugin().is_not_found(e):
+                raise exception.NotFound(_('Failed to find server %s') %
+                                         self.resource_id)
+            else:
+                raise
         else:
-            logger.debug(_('resuming server %s') % self.resource_id)
+            LOG.debug('resuming server %s' % self.resource_id)
             server.resume()
             return server
 
     def check_resume_complete(self, server):
         return self._check_active(server)
 
+    def handle_snapshot(self):
+        image_id = self.nova().servers.create_image(
+            self.resource_id, self.physical_resource_name())
+        return image_id
 
-class FlavorConstraint(object):
-
-    def validate(self, value, context):
-        nova_client = clients.Clients(context).nova()
-        try:
-            nova_utils.get_flavor_id(nova_client, value)
-        except exception.FlavorMissing:
-            return False
-        else:
+    def check_snapshot_complete(self, image_id):
+        image = self.nova().images.get(image_id)
+        if image.status == 'ACTIVE':
+            self.data_set('snapshot_image_id', image.id)
             return True
+        elif image.status == 'ERROR':
+            raise exception.Error(image.status)
+        return False
+
+    def handle_delete_snapshot(self, snapshot):
+        image_id = snapshot['resource_data']['snapshot_image_id']
+        try:
+            self.nova().images.delete(image_id)
+        except Exception as e:
+            self.client_plugin().ignore_not_found(e)
 
 
-def constraint_mapping():
-    return {'nova.flavor': FlavorConstraint}
+class FlavorConstraint(constraints.BaseCustomConstraint):
+
+    expected_exceptions = (exception.FlavorMissing,)
+
+    def validate_with_client(self, client, value):
+        client.client_plugin('nova').get_flavor_id(value)
 
 
 def resource_mapping():
