@@ -14,20 +14,20 @@
 #    under the License.
 
 import copy
+import six
 import uuid
 
-import heatclient.exc as heat_exp
-
 from heat.common import exception
-from heat.db import api as db_api
+from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import properties
 from heat.engine import resource
+from heat.engine.resources import resource_group
 from heat.engine.resources.software_config import software_config as sc
 from heat.engine import signal_responder
 from heat.openstack.common import log as logging
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class SoftwareDeployment(signal_responder.SignalResponder):
@@ -76,7 +76,7 @@ class SoftwareDeployment(signal_responder.SignalResponder):
         resource.Resource.RESUME,
     )
 
-    DEPLOY_ATTRIBUTES = (
+    ATTRIBUTES = (
         STDOUT, STDERR, STATUS_CODE
     ) = (
         'deploy_stdout', 'deploy_stderr', 'deploy_status_code'
@@ -149,13 +149,18 @@ class SoftwareDeployment(signal_responder.SignalResponder):
     }
 
     attributes_schema = {
-        STDOUT: _("Captured stdout from the configuration execution."),
-        STDERR: _("Captured stderr from the configuration execution."),
-        STATUS_CODE: _("Returned status code from the configuration "
-                       "execution"),
+        STDOUT: attributes.Schema(
+            _("Captured stdout from the configuration execution.")
+        ),
+        STDERR: attributes.Schema(
+            _("Captured stderr from the configuration execution.")
+        ),
+        STATUS_CODE: attributes.Schema(
+            _("Returned status code from the configuration execution")
+        ),
     }
 
-    update_allowed_keys = ('Properties',)
+    default_client_name = 'heat'
 
     def _signal_transport_cfn(self):
         return self.properties.get(
@@ -188,25 +193,27 @@ class SoftwareDeployment(signal_responder.SignalResponder):
     def _delete_derived_config(self, derived_config_id):
         try:
             self.heat().software_configs.delete(derived_config_id)
-        except heat_exp.HTTPNotFound:
-            pass
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
 
-    def _get_derived_config(self, action):
+    def _get_derived_config(self, action, source_config):
 
-        source_config_id = self.properties.get(self.CONFIG)
-        source_config = self.heat().software_configs.get(source_config_id)
         derived_params = self._build_derived_config_params(
             action, source_config.to_dict())
         derived_config = self.heat().software_configs.create(**derived_params)
         return derived_config.id
 
     def _handle_action(self, action):
-        if action not in self.properties[self.DEPLOY_ACTIONS]:
+        config_id = self.properties.get(self.CONFIG)
+        config = self.heat().software_configs.get(config_id)
+
+        if action not in self.properties[self.DEPLOY_ACTIONS]\
+                and not config.group == 'component':
             return
 
         props = self._build_properties(
             self.properties,
-            self._get_derived_config(action),
+            self._get_derived_config(action, config),
             action)
 
         if action == self.CREATE:
@@ -236,7 +243,7 @@ class SoftwareDeployment(signal_responder.SignalResponder):
         elif sd.status == SoftwareDeployment.FAILED:
             message = _("Deployment to server "
                         "failed: %s") % sd.status_reason
-            logger.error(message)
+            LOG.error(message)
             exc = exception.Error(message)
             raise exc
 
@@ -353,32 +360,22 @@ class SoftwareDeployment(signal_responder.SignalResponder):
 
     @property
     def password(self):
-        try:
-            return db_api.resource_data_get(self, 'password')
-        except exception.NotFound:
-            pass
+        return self.data().get('password')
 
     @password.setter
     def password(self, password):
-        try:
-            if password is None:
-                db_api.resource_data_delete(self, 'password')
-            else:
-                db_api.resource_data_set(self, 'password', password, True)
-        except exception.NotFound:
-            pass
+        if password is None:
+            self.data_delete('password')
+        else:
+            self.data_set('password', password, True)
 
     def check_create_complete(self, sd):
         return self._check_complete(sd)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
-            self.properties = properties.Properties(
-                self.properties_schema,
-                json_snippet.get('Properties', {}),
-                self.stack.resolve_runtime_data,
-                self.name,
-                self.context)
+            self.properties = json_snippet.properties(self.properties_schema,
+                                                      self.context)
 
         return self._handle_action(self.UPDATE)
 
@@ -411,8 +408,8 @@ class SoftwareDeployment(signal_responder.SignalResponder):
                 sd = self.heat().software_deployments.get(self.resource_id)
                 derived_config_id = sd.config_id
                 sd.delete()
-            except heat_exp.HTTPNotFound:
-                pass
+            except Exception as ex:
+                self.client_plugin().ignore_not_found(ex)
 
         if derived_config_id:
             self._delete_derived_config(derived_config_id)
@@ -441,11 +438,15 @@ class SoftwareDeployment(signal_responder.SignalResponder):
         ov = sd.output_values or {}
         status = None
         status_reasons = {}
-        if details.get(self.STATUS_CODE):
+        status_code = details.get(self.STATUS_CODE)
+        if status_code and str(status_code) != '0':
             status = self.FAILED
             status_reasons[self.STATUS_CODE] = _(
                 'Deployment exited with non-zero status code: %s'
             ) % details.get(self.STATUS_CODE)
+            event_reason = 'deployment failed (%s)' % status_code
+        else:
+            event_reason = 'deployment succeeded'
 
         for output in sc.outputs or []:
             out_key = output['name']
@@ -454,38 +455,153 @@ class SoftwareDeployment(signal_responder.SignalResponder):
                 if output.get('error_output', False):
                     status = self.FAILED
                     status_reasons[out_key] = details[out_key]
+                    event_reason = 'deployment failed'
 
-        for out_key in self.DEPLOY_ATTRIBUTES:
+        for out_key in self.ATTRIBUTES:
             ov[out_key] = details.get(out_key)
 
         if status == self.FAILED:
             # build a status reason out of all of the values of outputs
             # flagged as error_output
-            status_reason = ', '.join([' : '.join((k, str(status_reasons[k])))
-                                       for k in status_reasons])
+            status_reasons = [' : '.join((k, six.text_type(status_reasons[k])))
+                              for k in status_reasons]
+            status_reason = ', '.join(status_reasons)
         else:
             status = self.COMPLETE
             status_reason = _('Outputs received')
         sd.update(output_values=ov, status=status, status_reason=status_reason)
+        # Return a string describing the outcome of handling the signal data
+        return event_reason
 
-    def FnGetAtt(self, key):
+    def FnGetAtt(self, key, *path):
         '''
         Resource attributes map to deployment outputs values
         '''
         sd = self.heat().software_deployments.get(self.resource_id)
         if key in sd.output_values:
-            return sd.output_values.get(key)
+            attribute = sd.output_values.get(key)
+            return attributes.select_from_attribute(attribute, path)
 
         # Since there is no value for this key yet, check the output schemas
         # to find out if the key is valid
         sc = self.heat().software_configs.get(self.properties[self.CONFIG])
         output_keys = [output['name'] for output in sc.outputs]
-        if key not in output_keys and key not in self.DEPLOY_ATTRIBUTES:
+        if key not in output_keys and key not in self.ATTRIBUTES:
             raise exception.InvalidTemplateAttribute(resource=self.name,
                                                      key=key)
+        return None
+
+    def validate(self):
+        '''
+        Validate any of the provided params
+
+        :raises StackValidationFailed: if any property failed validation.
+        '''
+        super(SoftwareDeployment, self).validate()
+        server = self.properties.get(self.SERVER)
+        if server:
+            res = self.stack.resource_by_refid(server)
+            if res:
+                if not res.user_data_software_config():
+                    raise exception.StackValidationFailed(message=_(
+                        "Resource %s's property user_data_format should be "
+                        "set to SOFTWARE_CONFIG since there are software "
+                        "deployments on it.") % server)
+
+
+class SoftwareDeployments(resource_group.ResourceGroup):
+
+    PROPERTIES = (
+        SERVERS,
+        CONFIG,
+        INPUT_VALUES,
+        DEPLOY_ACTIONS,
+        NAME,
+        SIGNAL_TRANSPORT,
+    ) = (
+        'servers',
+        SoftwareDeployment.CONFIG,
+        SoftwareDeployment.INPUT_VALUES,
+        SoftwareDeployment.DEPLOY_ACTIONS,
+        SoftwareDeployment.NAME,
+        SoftwareDeployment.SIGNAL_TRANSPORT,
+    )
+
+    ATTRIBUTES = (
+        STDOUTS, STDERRS, STATUS_CODES
+    ) = (
+        'deploy_stdouts', 'deploy_stderrs', 'deploy_status_codes'
+    )
+
+    _sd_ps = SoftwareDeployment.properties_schema
+    _rg_ps = resource_group.ResourceGroup.properties_schema
+
+    properties_schema = {
+        SERVERS: properties.Schema(
+            properties.Schema.MAP,
+            _('A map of Nova names and IDs to apply configuration to.'),
+        ),
+        CONFIG: _sd_ps[CONFIG],
+        INPUT_VALUES: _sd_ps[INPUT_VALUES],
+        DEPLOY_ACTIONS: _sd_ps[DEPLOY_ACTIONS],
+        NAME: _sd_ps[NAME],
+        SIGNAL_TRANSPORT: _sd_ps[SIGNAL_TRANSPORT]
+    }
+
+    attributes_schema = {
+        STDOUTS: attributes.Schema(
+            _("A map of Nova names and captured stdouts from the "
+              "configuration execution to each server.")
+        ),
+        STDERRS: attributes.Schema(
+            _("A map of Nova names and captured stderrs from the "
+              "configuration execution to each server.")
+        ),
+        STATUS_CODES: attributes.Schema(
+            _("A map of Nova names and returned status code from the "
+              "configuration execution")
+        ),
+    }
+
+    def _resource_names(self, properties=None):
+        p = properties or self.properties
+        return p.get(self.SERVERS, {}).keys()
+
+    def _do_prop_replace(self, res_name, res_def_template):
+        res_def = copy.deepcopy(res_def_template)
+        props = res_def[self.RESOURCE_DEF_PROPERTIES]
+        servers = self.properties.get(self.SERVERS, {})
+        props[SoftwareDeployment.SERVER] = servers.get(res_name)
+        return res_def
+
+    def _build_resource_definition(self, include_all=False):
+        p = self.properties
+        return {
+            self.RESOURCE_DEF_TYPE: 'OS::Heat::SoftwareDeployment',
+            self.RESOURCE_DEF_PROPERTIES: {
+                self.CONFIG: p[self.CONFIG],
+                self.INPUT_VALUES: p[self.INPUT_VALUES],
+                self.DEPLOY_ACTIONS: p[self.DEPLOY_ACTIONS],
+                self.SIGNAL_TRANSPORT: p[self.SIGNAL_TRANSPORT],
+                self.NAME: p[self.NAME],
+            }
+        }
+
+    def FnGetAtt(self, key, *path):
+        rg = super(SoftwareDeployments, self)
+        if key == self.STDOUTS:
+            return rg.FnGetAtt(
+                rg.ATTR_ATTRIBUTES, SoftwareDeployment.STDOUT)
+        if key == self.STDERRS:
+            return rg.FnGetAtt(
+                rg.ATTR_ATTRIBUTES, SoftwareDeployment.STDERR)
+        if key == self.STATUS_CODES:
+            return rg.FnGetAtt(
+                rg.ATTR_ATTRIBUTES, SoftwareDeployment.STATUS_CODE)
 
 
 def resource_mapping():
     return {
         'OS::Heat::SoftwareDeployment': SoftwareDeployment,
+        'OS::Heat::SoftwareDeployments': SoftwareDeployments,
     }
