@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -16,6 +15,7 @@ import hashlib
 import json
 
 from requests import exceptions
+import six
 
 from heat.common import exception
 from heat.common import template_format
@@ -25,9 +25,6 @@ from heat.engine import environment
 from heat.engine import properties
 from heat.engine import stack_resource
 from heat.engine import template
-from heat.openstack.common import log as logging
-
-logger = logging.getLogger(__name__)
 
 
 def generate_class(name, template_name):
@@ -59,7 +56,6 @@ class TemplateResource(stack_resource.StackResource):
         self._parsed_nested = None
         self.stack = stack
         self.validation_exception = None
-        self.update_allowed_keys = ('Properties', 'Metadata')
 
         tri = stack.env.get_resource_info(
             json_snippet['Type'],
@@ -80,15 +76,16 @@ class TemplateResource(stack_resource.StackResource):
         self.attributes_schema = {}
         super(TemplateResource, self).__init__(name, json_snippet, stack)
         if self.validation_exception is None:
-            self._generate_schema(self.t.get('Properties', {}))
+            self._generate_schema(self.t)
 
-    def _generate_schema(self, props):
+    def _generate_schema(self, definition):
         self._parsed_nested = None
         try:
             tmpl = template.Template(self.child_template())
         except ValueError as download_error:
             self.validation_exception = download_error
-            tmpl = template.Template({})
+            tmpl = template.Template(
+                {"HeatTemplateFormatVersion": "2012-12-12"})
 
         # re-generate the properties and attributes from the template.
         self.properties_schema = (properties.Properties
@@ -96,10 +93,7 @@ class TemplateResource(stack_resource.StackResource):
         self.attributes_schema = (attributes.Attributes
                                   .schema_from_outputs(tmpl[tmpl.OUTPUTS]))
 
-        self.properties = properties.Properties(self.properties_schema,
-                                                props,
-                                                self._resolve_runtime_data,
-                                                self.name,
+        self.properties = definition.properties(self.properties_schema,
                                                 self.context)
         self.attributes = attributes.Attributes(self.name,
                                                 self.attributes_schema,
@@ -114,7 +108,15 @@ class TemplateResource(stack_resource.StackResource):
             if not pval.implemented():
                 continue
 
-            val = self.properties[pname]
+            try:
+                val = self.properties[pname]
+            except ValueError:
+                if self.action == self.INIT:
+                    prop = self.properties.props[pname]
+                    val = prop.get_value(None)
+                else:
+                    raise
+
             if val is not None:
                 # take a list and create a CommaDelimitedList
                 if pval.type() == properties.Schema.LIST:
@@ -143,7 +145,7 @@ class TemplateResource(stack_resource.StackResource):
         return self._parsed_nested
 
     def implementation_signature(self):
-        self._generate_schema(self.t.get('Properties', {}))
+        self._generate_schema(self.t)
         schema_names = ([prop for prop in self.properties_schema] +
                         [at for at in self.attributes_schema])
         schema_hash = hashlib.sha1(';'.join(schema_names))
@@ -190,7 +192,7 @@ class TemplateResource(stack_resource.StackResource):
 
             ps = self.properties_schema.get(n)
             if (n in self.properties_schema and
-                    (fs.type != ps.type)):
+                    (fs.allowed_param_prop_type() != ps.type)):
                 # Type mismatch
                 msg = (_("Property %(n)s type mismatch between facade %(type)s"
                        " (%(fs_type)s) and provider (%(ps_type)s)") % {
@@ -215,13 +217,13 @@ class TemplateResource(stack_resource.StackResource):
 
     def validate(self):
         if self.validation_exception is not None:
-            msg = str(self.validation_exception)
+            msg = six.text_type(self.validation_exception)
             raise exception.StackValidationFailed(message=msg)
 
         try:
             self.template_data()
         except ValueError as ex:
-            msg = _("Failed to retrieve template data: %s") % str(ex)
+            msg = _("Failed to retrieve template data: %s") % ex
             raise exception.StackValidationFailed(message=msg)
         cri = self.stack.env.get_resource_info(
             self.type(),
@@ -236,16 +238,25 @@ class TemplateResource(stack_resource.StackResource):
         return super(TemplateResource, self).validate()
 
     def handle_adopt(self, resource_data=None):
-        return self.create_with_template(self.parsed_nested(),
-                                         self._to_parameters(),
+        return self.create_with_template(self.child_template(),
+                                         self.child_params(),
                                          adopt_data=resource_data)
 
+    def check_adopt_complete(self, stack_creator):
+        done = stack_creator.step()
+        if done:
+            if self._nested.state != (self._nested.ADOPT,
+                                      self._nested.COMPLETE):
+                raise exception.Error(self._nested.status_reason)
+
+        return done
+
     def handle_create(self):
         return self.create_with_template(self.child_template(),
                                          self.child_params())
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        self._generate_schema(json_snippet.get('Properties', {}))
+        self._generate_schema(json_snippet)
         return self.update_with_template(self.child_template(),
                                          self.child_params())
 
