@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -14,17 +13,18 @@
 
 from oslo.config import cfg
 
+from heat.common import environment_format
 from heat.common import exception
+from heat.common.i18n import _
 from heat.engine import attributes
 from heat.engine import environment
 from heat.engine import parser
 from heat.engine import resource
 from heat.engine import scheduler
-
+from heat.engine import template
 from heat.openstack.common import log as logging
-from heat.openstack.common.gettextutils import _
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class StackResource(resource.Resource):
@@ -48,10 +48,10 @@ class StackResource(resource.Resource):
             self.recursion_depth = 0
 
     def _outputs_to_attribs(self, json_snippet):
-        if not self.attributes and 'Outputs' in json_snippet:
+        outputs = json_snippet.get('Outputs')
+        if not self.attributes and outputs:
             self.attributes_schema = (
-                attributes.Attributes
-                .schema_from_outputs(json_snippet.get('Outputs')))
+                attributes.Attributes.schema_from_outputs(outputs))
             self.attributes = attributes.Attributes(self.name,
                                                     self.attributes_schema,
                                                     self._resolve_attribute)
@@ -101,28 +101,69 @@ class StackResource(resource.Resource):
         StackResource will be returned as if it were a regular Resource.
         '''
         try:
-            template = parser.Template(self.child_template())
+            child_template = self.child_template()
             params = self.child_params()
         except NotImplementedError:
             not_implemented_msg = _("Preview of '%s' not yet implemented")
-            logger.warning(not_implemented_msg % self.__class__.__name__)
+            LOG.warning(not_implemented_msg % self.__class__.__name__)
             return self
 
-        self._validate_nested_resources(template)
         name = "%s-%s" % (self.stack.name, self.name)
+        self._nested = self._parse_nested_stack(name, child_template, params)
+
+        return self.nested().preview_resources()
+
+    def _parse_child_template(self, child_template):
+        parsed_child_template = child_template
+        if isinstance(parsed_child_template, template.Template):
+            parsed_child_template = parsed_child_template.t
+        return parser.Template(parsed_child_template, files=self.stack.t.files)
+
+    def _parse_nested_stack(self, stack_name, child_template, child_params,
+                            timeout_mins=None, adopt_data=None):
+        if self.recursion_depth >= cfg.CONF.max_nested_stack_depth:
+            msg = _("Recursion depth exceeds %d.") % \
+                cfg.CONF.max_nested_stack_depth
+            raise exception.RequestLimitExceeded(message=msg)
+
+        parsed_template = self._parse_child_template(child_template)
+        self._validate_nested_resources(parsed_template)
+
+        # Don't overwrite the attributes_schema for subclasses that
+        # define their own attributes_schema.
+        if not hasattr(type(self), 'attributes_schema'):
+            self.attributes = None
+            self._outputs_to_attribs(parsed_template)
+
+        if timeout_mins is None:
+            timeout_mins = self.stack.timeout_mins
+
+        stack_user_project_id = self.stack.stack_user_project_id
+
+        # Note we disable rollback for nested stacks, since they
+        # should be rolled back by the parent stack on failure
         nested = parser.Stack(self.context,
-                              name,
-                              template,
-                              self._nested_environment(params),
+                              stack_name,
+                              parsed_template,
+                              self._nested_environment(child_params),
+                              timeout_mins=timeout_mins,
                               disable_rollback=True,
                               parent_resource=self,
-                              owner_id=self.stack.id)
+                              owner_id=self.stack.id,
+                              user_creds_id=self.stack.user_creds_id,
+                              stack_user_project_id=stack_user_project_id,
+                              adopt_stack_data=adopt_data)
+        nested.validate()
+        return nested
 
-        return nested.preview_resources()
-
-    def _validate_nested_resources(self, template):
-        total_resources = (len(template[template.RESOURCES]) +
+    def _validate_nested_resources(self, templ):
+        total_resources = (len(templ[templ.RESOURCES]) +
                            self.stack.root_stack.total_resources())
+
+        if self.nested():
+            # It's an update and these resources will be deleted
+            total_resources -= len(self.nested().resources)
+
         if (total_resources > cfg.CONF.max_resources_per_stack):
             message = exception.StackResourceLimitExceeded.msg_fmt
             raise exception.RequestLimitExceeded(message=message)
@@ -135,10 +176,10 @@ class StackResource(resource.Resource):
         """
         nested_env = environment.Environment()
         nested_env.registry = self.stack.env.registry
-        user_env = {environment.PARAMETERS: {}}
+        user_env = {environment_format.PARAMETERS: {}}
         if user_params is not None:
-            if environment.PARAMETERS not in user_params:
-                user_env[environment.PARAMETERS] = user_params
+            if environment_format.PARAMETERS not in user_params:
+                user_env[environment_format.PARAMETERS] = user_params
             else:
                 user_env.update(user_params)
         nested_env.load(user_env)
@@ -146,33 +187,11 @@ class StackResource(resource.Resource):
 
     def create_with_template(self, child_template, user_params,
                              timeout_mins=None, adopt_data=None):
-        '''
-        Handle the creation of the nested stack from a given JSON template.
-        '''
-        if self.recursion_depth >= cfg.CONF.max_nested_stack_depth:
-            msg = _("Recursion depth exceeds %d.") % \
-                cfg.CONF.max_nested_stack_depth
-            raise exception.RequestLimitExceeded(message=msg)
-        template = parser.Template(child_template, files=self.stack.t.files)
-        self._validate_nested_resources(template)
-        self._outputs_to_attribs(child_template)
-
-        if timeout_mins is None:
-            timeout_mins = self.stack.timeout_mins
-
-        # Note we disable rollback for nested stacks, since they
-        # should be rolled back by the parent stack on failure
-        nested = parser.Stack(self.context,
-                              self.physical_resource_name(),
-                              template,
-                              self._nested_environment(user_params),
-                              timeout_mins=timeout_mins,
-                              disable_rollback=True,
-                              parent_resource=self,
-                              owner_id=self.stack.id,
-                              adopt_stack_data=adopt_data)
-        nested.validate()
-        self._nested = nested
+        """Create the nested stack with the given template."""
+        name = self.physical_resource_name()
+        self._nested = self._parse_nested_stack(name, child_template,
+                                                user_params, timeout_mins,
+                                                adopt_data)
         nested_id = self._nested.store()
         self.resource_id_set(nested_id)
 
@@ -186,6 +205,8 @@ class StackResource(resource.Resource):
         return stack_creator
 
     def check_create_complete(self, stack_creator):
+        if stack_creator is None:
+            return True
         done = stack_creator.step()
         if done:
             if self._nested.state != (self._nested.CREATE,
@@ -197,42 +218,15 @@ class StackResource(resource.Resource):
     def update_with_template(self, child_template, user_params,
                              timeout_mins=None):
         """Update the nested stack with the new template."""
-        template = parser.Template(child_template, files=self.stack.t.files)
-        # Note that there is no call to self._outputs_to_attribs here.
-        # If we have a use case for updating attributes of the resource based
-        # on updated templates we should make sure it's optional because not
-        # all subclasses want that behavior, since they may offer custom
-        # attributes.
         nested_stack = self.nested()
         if nested_stack is None:
             raise exception.Error(_('Cannot update %s, stack not created')
                                   % self.name)
-        res_diff = (
-            len(template[template.RESOURCES]) - len(nested_stack.resources))
-        new_size = nested_stack.root_stack.total_resources() + res_diff
-        if new_size > cfg.CONF.max_resources_per_stack:
-            raise exception.RequestLimitExceeded(
-                message=exception.StackResourceLimitExceeded.msg_fmt)
 
-        if timeout_mins is None:
-            timeout_mins = self.stack.timeout_mins
-
-        # Note we disable rollback for nested stacks, since they
-        # should be rolled back by the parent stack on failure
-        stack = parser.Stack(self.context,
-                             self.physical_resource_name(),
-                             template,
-                             self._nested_environment(user_params),
-                             timeout_mins=timeout_mins,
-                             disable_rollback=True,
-                             parent_resource=self,
-                             owner_id=self.stack.id)
+        name = self.physical_resource_name()
+        stack = self._parse_nested_stack(name, child_template, user_params,
+                                         timeout_mins)
         stack.parameters.set_stack_id(nested_stack.identifier())
-        stack.validate()
-
-        if not hasattr(type(self), 'attributes_schema'):
-            self.attributes = None
-            self._outputs_to_attribs(child_template)
 
         updater = scheduler.TaskRunner(nested_stack.update_task, stack)
         updater.start()
@@ -259,7 +253,7 @@ class StackResource(resource.Resource):
         try:
             stack = self.nested()
         except exception.NotFound:
-            logger.info(_("Stack not found to delete"))
+            LOG.info(_("Stack not found to delete"))
         else:
             if stack is not None:
                 delete_task = scheduler.TaskRunner(stack.delete)
@@ -323,11 +317,24 @@ class StackResource(resource.Resource):
 
         return done
 
-    def set_deletion_policy(self, policy):
-        self.nested().set_deletion_policy(policy)
+    def handle_check(self):
+        stack = self.nested()
+        if stack is None:
+            raise exception.Error(_('Cannot check %s, stack not created')
+                                  % self.name)
 
-    def get_abandon_data(self):
-        return self.nested().get_abandon_data()
+        check_task = scheduler.TaskRunner(self._nested.stack_task,
+                                          action=self._nested.CHECK,
+                                          aggregate_exceptions=True)
+
+        check_task.start(timeout=self._nested.timeout_secs())
+        return check_task
+
+    def check_check_complete(self, check_task):
+        return check_task.step()
+
+    def prepare_abandon(self):
+        return self.nested().prepare_abandon()
 
     def get_output(self, op):
         '''
@@ -345,4 +352,4 @@ class StackResource(resource.Resource):
         return stack.output(op)
 
     def _resolve_attribute(self, name):
-        return unicode(self.get_output(name))
+        return self.get_output(name)
