@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -14,15 +13,13 @@
 
 import keystoneclient.exceptions as kc_exception
 
-from heat.db import api as db_api
 from heat.common import exception
+from heat.common.i18n import _
 from heat.engine import resource
-
 from heat.openstack.common import log as logging
-from heat.openstack.common.gettextutils import _
 
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class StackUser(resource.Resource):
@@ -32,7 +29,6 @@ class StackUser(resource.Resource):
     # (in a project specific to the stack)
     def __init__(self, name, json_snippet, stack):
         super(StackUser, self).__init__(name, json_snippet, stack)
-        self.password = None
 
     def handle_create(self):
         self._create_user()
@@ -47,23 +43,39 @@ class StackUser(resource.Resource):
         # Create a keystone user in the stack domain project
         user_id = self.keystone().create_stack_domain_user(
             username=self.physical_resource_name(),
-            password=self.password,
+            password=getattr(self, 'password', None),
             project_id=self.stack.stack_user_project_id)
 
         # Store the ID in resource data, for compatibility with SignalResponder
-        db_api.resource_data_set(self, 'user_id', user_id)
+        self.data_set('user_id', user_id)
+
+    def _user_token(self):
+        project_id = self.stack.stack_user_project_id
+        if not project_id:
+            raise ValueError(_("Can't get user token, user not yet created"))
+        password = getattr(self, 'password', None)
+        # FIXME(shardy): the create and getattr here could allow insane
+        # passwords, e.g a zero length string, if these happen it almost
+        # certainly means a bug elsewhere in heat, so add assertion to catch
+        if password is None:
+            raise ValueError(_("Can't get user token without password"))
+
+        return self.keystone().stack_domain_user_token(
+            username=self.physical_resource_name(),
+            project_id=project_id, password=password)
 
     def _get_user_id(self):
-        try:
-            return db_api.resource_data_get(self, 'user_id')
-        except exception.NotFound:
+        user_id = self.data().get('user_id')
+        if user_id:
+            return user_id
+        else:
             # FIXME(shardy): This is a legacy hack for backwards compatibility
             # remove after an appropriate transitional period...
             # Assume this is a resource that was created with
             # a previous version of heat and that the resource_id
             # is the user_id
             if self.resource_id:
-                db_api.resource_data_set(self, 'user_id', self.resource_id)
+                self.data_set('user_id', self.resource_id)
                 return self.resource_id
 
     def handle_delete(self):
@@ -83,16 +95,13 @@ class StackUser(resource.Resource):
             # compatibility with resources created before the migration
             # to stack_user.StackUser domain users.  After an appropriate
             # transitional period, this should be removed.
-            logger.warning(_('Reverting to legacy user delete path'))
+            LOG.warning(_('Reverting to legacy user delete path'))
             try:
                 self.keystone().delete_stack_user(user_id)
             except kc_exception.NotFound:
                 pass
         for data_key in ('credential_id', 'access_key', 'secret_key'):
-            try:
-                db_api.resource_data_delete(self, data_key)
-            except exception.NotFound:
-                pass
+            self.data_delete(data_key)
 
     def handle_suspend(self):
         user_id = self._get_user_id()
@@ -123,21 +132,23 @@ class StackUser(resource.Resource):
             raise exception.Error(_("Error creating ec2 keypair for user %s") %
                                   user_id)
         else:
-            db_api.resource_data_set(self, 'credential_id', kp.id,
-                                     redact=True)
-            db_api.resource_data_set(self, 'access_key', kp.access,
-                                     redact=True)
-            db_api.resource_data_set(self, 'secret_key', kp.secret,
-                                     redact=True)
+            try:
+                credential_id = kp.id
+            except AttributeError:
+                # keystone v2 keypairs do not have an id attribute. Use the
+                # access key instead.
+                credential_id = kp.access
+            self.data_set('credential_id', credential_id, redact=True)
+            self.data_set('access_key', kp.access, redact=True)
+            self.data_set('secret_key', kp.secret, redact=True)
         return kp
 
     def _delete_keypair(self):
         # Subclasses may optionally call this to delete a keypair created
         # via _create_keypair
         user_id = self._get_user_id()
-        try:
-            credential_id = db_api.resource_data_get(self, 'credential_id')
-        except exception.NotFound:
+        credential_id = self.data().get('credential_id')
+        if not credential_id:
             return
 
         try:
@@ -149,7 +160,4 @@ class StackUser(resource.Resource):
                 user_id=user_id, credential_id=credential_id)
 
         for data_key in ('access_key', 'secret_key', 'credential_id'):
-            try:
-                db_api.resource_data_delete(self, data_key)
-            except exception.NotFound:
-                pass
+            self.data_delete(data_key)
