@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -13,6 +13,7 @@
 
 import copy
 import json
+import six
 
 import mox
 from oslo.config import cfg
@@ -20,15 +21,14 @@ from testtools.matchers import MatchesRegex
 
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
+from heat.engine.clients.os import nova
+from heat.engine import function
 from heat.engine.notification import stack as notification
 from heat.engine import parser
 from heat.engine.resources import instance
 from heat.engine.resources import loadbalancer as lb
 from heat.engine.resources import wait_condition as wc
-from heat.engine import stack_user
 from heat.tests.common import HeatTestCase
-from heat.tests import fakes
 from heat.tests import utils
 from heat.tests.v1_1 import fakes as fakes11
 
@@ -206,18 +206,15 @@ class AutoScalingGroupTest(HeatTestCase):
     def setUp(self):
         super(AutoScalingGroupTest, self).setUp()
         self.fc = fakes11.FakeClient()
-        self.fkc = fakes.FakeKeystoneClient(username='test_stack.CfnLBUser')
+        self.stub_keystoneclient(username='test_stack.CfnLBUser')
         cfg.CONF.set_default('heat_waitcondition_server_url',
                              'http://127.0.0.1:8000/v1/waitcondition')
-        utils.setup_dummy_db()
 
     def _stub_validate(self):
         self.m.StubOutWithMock(parser.Stack, 'validate')
         parser.Stack.validate().MultipleTimes()
 
     def _stub_lb_create(self):
-        self.m.StubOutWithMock(stack_user.StackUser, 'keystone')
-        stack_user.StackUser.keystone().MultipleTimes().AndReturn(self.fkc)
         self.m.StubOutWithMock(wc.WaitConditionHandle, 'get_status')
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS'])
 
@@ -236,7 +233,6 @@ class AutoScalingGroupTest(HeatTestCase):
         """
         self._stub_validate()
 
-        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
         self.m.StubOutWithMock(instance.Instance, 'handle_create')
         self.m.StubOutWithMock(instance.Instance, 'check_create_complete')
 
@@ -245,7 +241,8 @@ class AutoScalingGroupTest(HeatTestCase):
 
         cookie = object()
 
-        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
+        nova.NovaClientPlugin._create().AndReturn(self.fc)
         # for load balancer setup
         if setup_lb:
             self._stub_lb_create()
@@ -272,14 +269,10 @@ class AutoScalingGroupTest(HeatTestCase):
         notification.send(mox.IgnoreArg()).MultipleTimes().AndReturn(None)
 
         # for instances in the group
-        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
         self.m.StubOutWithMock(instance.Instance, 'handle_create')
         self.m.StubOutWithMock(instance.Instance, 'check_create_complete')
         self.m.StubOutWithMock(instance.Instance, 'destroy')
 
-        if num_reloads_expected_on_updt > 1:
-            clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
-
         cookie = object()
         for i in range(num_creates_expected_on_updt):
             instance.Instance.handle_create().AndReturn(cookie)
@@ -294,14 +287,12 @@ class AutoScalingGroupTest(HeatTestCase):
         """
         Expect update of the instances
         """
-        self.m.StubOutWithMock(instance.Instance, 'nova')
-        instance.Instance.nova().MultipleTimes().AndReturn(self.fc)
 
         def activate_status(server):
             server.status = 'VERIFY_RESIZE'
 
         return_server = self.fc.servers.list()[1]
-        return_server.id = 1234
+        return_server.id = '1234'
         return_server.get = activate_status.__get__(return_server)
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
@@ -326,13 +317,22 @@ class AutoScalingGroupTest(HeatTestCase):
     def test_parse_without_update_policy(self):
         tmpl = template_format.parse(asg_tmpl_without_updt_policy)
         stack = utils.parse_stack(tmpl)
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
+        self.m.ReplayAll()
+
         stack.validate()
         grp = stack['WebServerGroup']
         self.assertFalse(grp.update_policy['AutoScalingRollingUpdate'])
+        self.m.VerifyAll()
 
     def test_parse_with_update_policy(self):
         tmpl = template_format.parse(asg_tmpl_with_updt_policy)
         stack = utils.parse_stack(tmpl)
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
+        self.m.ReplayAll()
+
         stack.validate()
         tmpl_grp = tmpl['Resources']['WebServerGroup']
         tmpl_policy = tmpl_grp['UpdatePolicy']['AutoScalingRollingUpdate']
@@ -346,10 +346,15 @@ class AutoScalingGroupTest(HeatTestCase):
         self.assertEqual(1, int(policy['MinInstancesInService']))
         self.assertEqual(tmpl_batch_sz, int(policy['MaxBatchSize']))
         self.assertEqual('PT1S', policy['PauseTime'])
+        self.m.VerifyAll()
 
     def test_parse_with_default_update_policy(self):
         tmpl = template_format.parse(asg_tmpl_with_default_updt_policy)
         stack = utils.parse_stack(tmpl)
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
+        self.m.ReplayAll()
+
         stack.validate()
         grp = stack['WebServerGroup']
         self.assertTrue(grp.update_policy)
@@ -360,19 +365,30 @@ class AutoScalingGroupTest(HeatTestCase):
         self.assertEqual(0, int(policy['MinInstancesInService']))
         self.assertEqual(1, int(policy['MaxBatchSize']))
         self.assertEqual('PT0S', policy['PauseTime'])
+        self.m.VerifyAll()
 
     def test_parse_with_bad_update_policy(self):
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
+        self.m.ReplayAll()
         tmpl = template_format.parse(asg_tmpl_with_bad_updt_policy)
         stack = utils.parse_stack(tmpl)
-        self.assertRaises(exception.StackValidationFailed, stack.validate)
+        error = self.assertRaises(
+            exception.StackValidationFailed, stack.validate)
+        self.assertIn("foo", six.text_type(error))
 
     def test_parse_with_bad_pausetime_in_update_policy(self):
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
+        self.m.ReplayAll()
         tmpl = template_format.parse(asg_tmpl_with_default_updt_policy)
         group = tmpl['Resources']['WebServerGroup']
         policy = group['UpdatePolicy']['AutoScalingRollingUpdate']
         policy['PauseTime'] = 'P1YT1H'
         stack = utils.parse_stack(tmpl)
-        self.assertRaises(exception.StackValidationFailed, stack.validate)
+        error = self.assertRaises(
+            exception.StackValidationFailed, stack.validate)
+        self.assertIn("Only ISO 8601 duration format", six.text_type(error))
 
     def validate_update_policy_diff(self, current, updated):
 
@@ -393,7 +409,7 @@ class AutoScalingGroupTest(HeatTestCase):
         # get the updated json snippet for the InstanceGroup resource in the
         # context of the current stack
         updated_grp = updated_stack['WebServerGroup']
-        updated_grp_json = current_stack.resolve_runtime_data(updated_grp.t)
+        updated_grp_json = function.resolve(updated_grp.t)
 
         # identify the template difference
         tmpl_diff = updated_grp.update_template_diff(
@@ -426,16 +442,25 @@ class AutoScalingGroupTest(HeatTestCase):
                                  num_creates_expected_on_updt,
                                  num_deletes_expected_on_updt,
                                  num_reloads_expected_on_updt,
-                                 update_replace):
+                                 update_replace,
+                                 update_image_id=None):
 
         # setup stack from the initial template
         tmpl = template_format.parse(init_template)
         stack = utils.parse_stack(tmpl)
+
+        self.stub_KeypairConstraint_validate()
+        self.stub_ImageConstraint_validate()
+        self.m.ReplayAll()
+
         stack.validate()
+        self.m.VerifyAll()
+        self.m.UnsetStubs()
 
         # test stack create
         size = int(stack['WebServerGroup'].properties['MinSize'])
         self._stub_grp_create(size)
+        self.stub_ImageConstraint_validate()
         self.m.ReplayAll()
         stack.create()
         self.m.VerifyAll()
@@ -483,7 +508,6 @@ class AutoScalingGroupTest(HeatTestCase):
         new_updt_pol = new_grp_tmpl['UpdatePolicy']['AutoScalingRollingUpdate']
         new_batch_sz = int(new_updt_pol['MaxBatchSize'])
         self.assertNotEqual(new_batch_sz, init_batch_sz)
-        self._stub_validate()
         if update_replace:
             self._stub_grp_replace(size, size, num_reloads_expected_on_updt)
         else:
@@ -491,7 +515,12 @@ class AutoScalingGroupTest(HeatTestCase):
                                   num_deletes_expected_on_updt,
                                   num_reloads_expected_on_updt)
         self.stub_wallclock()
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
         self.m.ReplayAll()
+
+        stack.validate()
+
         stack.update(updated_stack)
         self.m.VerifyAll()
         self.assertEqual(('UPDATE', 'COMPLETE'), stack.state)
@@ -555,7 +584,8 @@ class AutoScalingGroupTest(HeatTestCase):
         policy['MinInstancesInService'] = '1'
         policy['MaxBatchSize'] = '3'
         config = updt_template['Resources']['LaunchConfig']
-        config['Properties']['ImageId'] = 'F17-x86_64-cfntools'
+        update_image = 'F17-x86_64-cfntools'
+        config['Properties']['ImageId'] = update_image
 
         self.update_autoscaling_group(asg_tmpl_with_updt_policy,
                                       json.dumps(updt_template),
@@ -563,7 +593,8 @@ class AutoScalingGroupTest(HeatTestCase):
                                       num_creates_expected_on_updt=0,
                                       num_deletes_expected_on_updt=0,
                                       num_reloads_expected_on_updt=9,
-                                      update_replace=True)
+                                      update_replace=True,
+                                      update_image_id=update_image)
 
     def test_autoscaling_group_update_replace_with_adjusted_capacity(self):
         """
@@ -576,7 +607,8 @@ class AutoScalingGroupTest(HeatTestCase):
         policy['MinInstancesInService'] = '8'
         policy['MaxBatchSize'] = '4'
         config = updt_template['Resources']['LaunchConfig']
-        config['Properties']['ImageId'] = 'F17-x86_64-cfntools'
+        update_image = 'F17-x86_64-cfntools'
+        config['Properties']['ImageId'] = update_image
 
         self.update_autoscaling_group(asg_tmpl_with_updt_policy,
                                       json.dumps(updt_template),
@@ -584,7 +616,8 @@ class AutoScalingGroupTest(HeatTestCase):
                                       num_creates_expected_on_updt=2,
                                       num_deletes_expected_on_updt=2,
                                       num_reloads_expected_on_updt=7,
-                                      update_replace=True)
+                                      update_replace=True,
+                                      update_image_id=update_image)
 
     def test_autoscaling_group_update_replace_huge_batch_size(self):
         """
@@ -596,7 +629,8 @@ class AutoScalingGroupTest(HeatTestCase):
         policy['MinInstancesInService'] = '0'
         policy['MaxBatchSize'] = '20'
         config = updt_template['Resources']['LaunchConfig']
-        config['Properties']['ImageId'] = 'F17-x86_64-cfntools'
+        update_image = 'F17-x86_64-cfntools'
+        config['Properties']['ImageId'] = update_image
 
         self.update_autoscaling_group(asg_tmpl_with_updt_policy,
                                       json.dumps(updt_template),
@@ -604,7 +638,8 @@ class AutoScalingGroupTest(HeatTestCase):
                                       num_creates_expected_on_updt=0,
                                       num_deletes_expected_on_updt=0,
                                       num_reloads_expected_on_updt=3,
-                                      update_replace=True)
+                                      update_replace=True,
+                                      update_image_id=update_image)
 
     def test_autoscaling_group_update_replace_huge_min_in_service(self):
         """
@@ -617,6 +652,7 @@ class AutoScalingGroupTest(HeatTestCase):
         policy['MaxBatchSize'] = '1'
         policy['PauseTime'] = 'PT0S'
         config = updt_template['Resources']['LaunchConfig']
+        update_image = 'F17-x86_64-cfntools'
         config['Properties']['ImageId'] = 'F17-x86_64-cfntools'
 
         self.update_autoscaling_group(asg_tmpl_with_updt_policy,
@@ -625,7 +661,8 @@ class AutoScalingGroupTest(HeatTestCase):
                                       num_creates_expected_on_updt=1,
                                       num_deletes_expected_on_updt=1,
                                       num_reloads_expected_on_updt=12,
-                                      update_replace=True)
+                                      update_replace=True,
+                                      update_image_id=update_image)
 
     def test_autoscaling_group_update_no_replace(self):
         """
@@ -678,11 +715,18 @@ class AutoScalingGroupTest(HeatTestCase):
         # setup stack from the initial template
         tmpl = template_format.parse(asg_tmpl_with_updt_policy)
         stack = utils.parse_stack(tmpl)
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
+        self.m.ReplayAll()
+
         stack.validate()
+        self.m.VerifyAll()
+        self.m.UnsetStubs()
 
         # test stack create
         size = int(stack['WebServerGroup'].properties['MinSize'])
         self._stub_grp_create(size)
+        self.stub_ImageConstraint_validate()
         self.m.ReplayAll()
         stack.create()
         self.m.VerifyAll()
@@ -735,6 +779,7 @@ class AutoScalingGroupTest(HeatTestCase):
         # test stack create
         size = int(stack['WebServerGroup'].properties['MinSize'])
         self._stub_grp_create(size)
+        self.stub_ImageConstraint_validate()
         self.m.ReplayAll()
         stack.create()
         self.m.VerifyAll()
@@ -771,6 +816,8 @@ class AutoScalingGroupTest(HeatTestCase):
         self._stub_grp_replace(num_creates_expected_on_updt=0,
                                num_deletes_expected_on_updt=0,
                                num_reloads_expected_on_updt=1)
+        self.stub_KeypairConstraint_validate()
+        self.stub_ImageConstraint_validate()
         self.m.ReplayAll()
         stack.update(updated_stack)
         self.m.VerifyAll()
