@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -14,26 +14,24 @@
 import copy
 import json
 
+from ceilometerclient import exc as ceilometerclient_exc
 import mox
 from oslo.config import cfg
-import testtools
+import six
 
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
+from heat.engine.clients.os import ceilometer
 from heat.engine import parser
 from heat.engine.properties import schemata
 from heat.engine import resource
 from heat.engine.resources.ceilometer import alarm
+from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.engine import stack_user
-from heat.openstack.common.importutils import try_import
 from heat.tests.common import HeatTestCase
-from heat.tests import fakes
 from heat.tests import generic_resource
 from heat.tests import utils
 
-ceilometerclient = try_import('ceilometerclient.v2')
 
 alarm_template = '''
 {
@@ -127,11 +125,9 @@ class FakeCeilometerClient(object):
     alarms = FakeCeilometerAlarms()
 
 
-@testtools.skipIf(ceilometerclient is None, 'ceilometerclient unavailable')
 class CeilometerAlarmTest(HeatTestCase):
     def setUp(self):
         super(CeilometerAlarmTest, self).setUp()
-        utils.setup_dummy_db()
 
         resource._register_class('SignalResourceType',
                                  generic_resource.SignalResource)
@@ -139,11 +135,9 @@ class CeilometerAlarmTest(HeatTestCase):
         cfg.CONF.set_default('heat_waitcondition_server_url',
                              'http://server.test:8000/v1/waitcondition')
 
-        self.fc = fakes.FakeKeystoneClient()
+        self.stub_keystoneclient()
         self.fa = FakeCeilometerClient()
 
-    # Note tests creating a stack should be decorated with @stack_delete_after
-    # to ensure the stack is properly cleaned up
     def create_stack(self, template=None):
         if template is None:
             template = alarm_template
@@ -155,10 +149,6 @@ class CeilometerAlarmTest(HeatTestCase):
                              disable_rollback=True)
         stack.store()
 
-        self.m.StubOutWithMock(stack_user.StackUser, 'keystone')
-        stack_user.StackUser.keystone().MultipleTimes().AndReturn(
-            self.fc)
-
         self.m.StubOutWithMock(alarm.CeilometerAlarm, 'ceilometer')
         alarm.CeilometerAlarm.ceilometer().MultipleTimes().AndReturn(
             self.fa)
@@ -167,11 +157,13 @@ class CeilometerAlarmTest(HeatTestCase):
         al['description'] = mox.IgnoreArg()
         al['name'] = mox.IgnoreArg()
         al['alarm_actions'] = mox.IgnoreArg()
+        al['matching_metadata'] = dict(
+            ('metadata.metering.%s' % k, v)
+            for k, v in al['matching_metadata'].items())
         self.m.StubOutWithMock(self.fa.alarms, 'create')
         self.fa.alarms.create(**al).AndReturn(FakeCeilometerAlarm())
         return stack
 
-    @utils.stack_delete_after
     def test_mem_alarm_high_update_no_replace(self):
         '''
         Make sure that we can change the update-able properties
@@ -195,24 +187,28 @@ class CeilometerAlarmTest(HeatTestCase):
         self.stack.create()
         rsrc = self.stack['MEMAlarmHigh']
 
-        snippet = copy.deepcopy(rsrc.parsed_template())
-        snippet['Properties']['comparison_operator'] = 'lt'
-        snippet['Properties']['description'] = 'fruity'
-        snippet['Properties']['evaluation_periods'] = '2'
-        snippet['Properties']['period'] = '90'
-        snippet['Properties']['enabled'] = 'true'
-        snippet['Properties']['repeat_actions'] = True
-        snippet['Properties']['statistic'] = 'max'
-        snippet['Properties']['threshold'] = '39'
-        snippet['Properties']['insufficient_data_actions'] = []
-        snippet['Properties']['alarm_actions'] = []
-        snippet['Properties']['ok_actions'] = ['signal_handler']
+        props = copy.copy(rsrc.properties.data)
+        props.update({
+            'comparison_operator': 'lt',
+            'description': 'fruity',
+            'evaluation_periods': '2',
+            'period': '90',
+            'enabled': 'true',
+            'repeat_actions': True,
+            'statistic': 'max',
+            'threshold': '39',
+            'insufficient_data_actions': [],
+            'alarm_actions': [],
+            'ok_actions': ['signal_handler'],
+        })
+        snippet = rsrc_defn.ResourceDefinition(rsrc.name,
+                                               rsrc.type(),
+                                               props)
 
         scheduler.TaskRunner(rsrc.update, snippet)()
 
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_mem_alarm_high_update_replace(self):
         '''
         Make sure that the Alarm resource IS replaced when non-update-able
@@ -229,15 +225,17 @@ class CeilometerAlarmTest(HeatTestCase):
         self.stack.create()
         rsrc = self.stack['MEMAlarmHigh']
 
-        snippet = copy.deepcopy(rsrc.parsed_template())
-        snippet['Properties']['meter_name'] = 'temp'
+        props = copy.copy(rsrc.properties.data)
+        props['meter_name'] = 'temp'
+        snippet = rsrc_defn.ResourceDefinition(rsrc.name,
+                                               rsrc.type(),
+                                               props)
 
         updater = scheduler.TaskRunner(rsrc.update, snippet)
         self.assertRaises(resource.UpdateReplace, updater)
 
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_mem_alarm_suspend_resume(self):
         """
         Make sure that the Alarm resource gets disabled on suspend
@@ -263,7 +261,6 @@ class CeilometerAlarmTest(HeatTestCase):
 
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_mem_alarm_high_correct_int_parameters(self):
         self.stack = self.create_stack(not_string_alarm_template)
 
@@ -285,13 +282,14 @@ class CeilometerAlarmTest(HeatTestCase):
             snippet['Resources']['MEMAlarmHigh']['Properties'][p] = '60a'
             stack = utils.parse_stack(snippet)
 
+            resource_defns = stack.t.resource_definitions(stack)
             rsrc = alarm.CeilometerAlarm(
-                'MEMAlarmHigh', snippet['Resources']['MEMAlarmHigh'], stack)
+                'MEMAlarmHigh', resource_defns['MEMAlarmHigh'], stack)
             error = self.assertRaises(exception.StackValidationFailed,
                                       rsrc.validate)
             self.assertEqual(
                 "Property error : MEMAlarmHigh: %s Value '60a' is not an "
-                "integer" % p, str(error))
+                "integer" % p, six.text_type(error))
 
     def test_mem_alarm_high_not_integer_parameters(self):
         snippet = template_format.parse(not_string_alarm_template)
@@ -299,26 +297,28 @@ class CeilometerAlarmTest(HeatTestCase):
             snippet['Resources']['MEMAlarmHigh']['Properties'][p] = [60]
             stack = utils.parse_stack(snippet)
 
+            resource_defns = stack.t.resource_definitions(stack)
             rsrc = alarm.CeilometerAlarm(
-                'MEMAlarmHigh', snippet['Resources']['MEMAlarmHigh'], stack)
+                'MEMAlarmHigh', resource_defns['MEMAlarmHigh'], stack)
             error = self.assertRaises(exception.StackValidationFailed,
                                       rsrc.validate)
             self.assertEqual(
                 "Property error : MEMAlarmHigh: %s int() argument must be "
-                "a string or a number, not 'list'" % p, str(error))
+                "a string or a number, not 'list'" % p, six.text_type(error))
 
     def test_mem_alarm_high_check_not_required_parameters(self):
         snippet = template_format.parse(not_string_alarm_template)
         snippet['Resources']['MEMAlarmHigh']['Properties'].pop('meter_name')
         stack = utils.parse_stack(snippet)
 
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = alarm.CeilometerAlarm(
-            'MEMAlarmHigh', snippet['Resources']['MEMAlarmHigh'], stack)
+            'MEMAlarmHigh', resource_defns['MEMAlarmHigh'], stack)
         error = self.assertRaises(exception.StackValidationFailed,
                                   rsrc.validate)
         self.assertEqual(
             "Property error : MEMAlarmHigh: Property meter_name not assigned",
-            str(error))
+            six.text_type(error))
 
         for p in ('period', 'evaluation_periods', 'statistic',
                   'comparison_operator'):
@@ -326,8 +326,9 @@ class CeilometerAlarmTest(HeatTestCase):
             snippet['Resources']['MEMAlarmHigh']['Properties'].pop(p)
             stack = utils.parse_stack(snippet)
 
+            resource_defns = stack.t.resource_definitions(stack)
             rsrc = alarm.CeilometerAlarm(
-                'MEMAlarmHigh', snippet['Resources']['MEMAlarmHigh'], stack)
+                'MEMAlarmHigh', resource_defns['MEMAlarmHigh'], stack)
             self.assertIsNone(rsrc.validate())
 
     def test_delete_alarm_not_found(self):
@@ -336,7 +337,7 @@ class CeilometerAlarmTest(HeatTestCase):
         self.stack = self.create_stack(template=json.dumps(t))
         self.m.StubOutWithMock(self.fa.alarms, 'delete')
         self.fa.alarms.delete('foo').AndRaise(
-            alarm.ceilometerclient_exc.HTTPNotFound())
+            ceilometerclient_exc.HTTPNotFound())
 
         self.m.ReplayAll()
         self.stack.create()
@@ -348,17 +349,15 @@ class CeilometerAlarmTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-@testtools.skipIf(ceilometerclient is None, 'ceilometerclient unavailable')
 class CombinationAlarmTest(HeatTestCase):
 
     def setUp(self):
         super(CombinationAlarmTest, self).setUp()
         self.fc = FakeCeilometerClient()
-        self.m.StubOutWithMock(clients.OpenStackClients, 'ceilometer')
-        utils.setup_dummy_db()
+        self.m.StubOutWithMock(ceilometer.CeilometerClientPlugin, '_create')
 
     def create_alarm(self):
-        clients.OpenStackClients.ceilometer().MultipleTimes().AndReturn(
+        ceilometer.CeilometerClientPlugin._create().AndReturn(
             self.fc)
         self.m.StubOutWithMock(self.fc.alarms, 'create')
         self.fc.alarms.create(
@@ -370,8 +369,9 @@ class CombinationAlarmTest(HeatTestCase):
         ).AndReturn(FakeCeilometerAlarm())
         snippet = template_format.parse(combination_alarm_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return alarm.CombinationAlarm(
-            'CombinAlarm', snippet['Resources']['CombinAlarm'], stack)
+            'CombinAlarm', resource_defns['CombinAlarm'], stack)
 
     def test_create(self):
         rsrc = self.create_alarm()
@@ -386,13 +386,14 @@ class CombinationAlarmTest(HeatTestCase):
         snippet = template_format.parse(combination_alarm_template)
         snippet['Resources']['CombinAlarm']['Properties']['alarm_ids'] = []
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = alarm.CombinationAlarm(
-            'CombinAlarm', snippet['Resources']['CombinAlarm'], stack)
+            'CombinAlarm', resource_defns['CombinAlarm'], stack)
         error = self.assertRaises(exception.StackValidationFailed,
                                   rsrc.validate)
         self.assertEqual(
             "Property error : CombinAlarm: alarm_ids length (0) is out of "
-            "range (min: 1, max: None)", str(error))
+            "range (min: 1, max: None)", six.text_type(error))
 
     def test_update(self):
         rsrc = self.create_alarm()
@@ -453,7 +454,7 @@ class CombinationAlarmTest(HeatTestCase):
         rsrc = self.create_alarm()
         self.m.StubOutWithMock(self.fc.alarms, 'delete')
         self.fc.alarms.delete('foo').AndRaise(
-            alarm.ceilometerclient_exc.HTTPNotFound())
+            ceilometerclient_exc.HTTPNotFound())
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         scheduler.TaskRunner(rsrc.delete)()
