@@ -12,20 +12,18 @@
 #    under the License.
 
 import copy
+import six
 
-from testtools import skipIf
+from neutronclient.common import exceptions
+from neutronclient.v2_0 import client as neutronclient
 
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
 from heat.engine.resources.neutron import firewall
 from heat.engine import scheduler
-from heat.openstack.common.importutils import try_import
 from heat.tests.common import HeatTestCase
-from heat.tests import fakes
 from heat.tests import utils
 
-neutronclient = try_import('neutronclient.v2_0.client')
 
 firewall_template = '''
 {
@@ -86,7 +84,6 @@ firewall_rule_template = '''
 '''
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class FirewallTest(HeatTestCase):
 
     def setUp(self):
@@ -95,12 +92,9 @@ class FirewallTest(HeatTestCase):
         self.m.StubOutWithMock(neutronclient.Client, 'delete_firewall')
         self.m.StubOutWithMock(neutronclient.Client, 'show_firewall')
         self.m.StubOutWithMock(neutronclient.Client, 'update_firewall')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
     def create_firewall(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_firewall({
             'firewall': {
                 'name': 'test-firewall', 'admin_state_up': True,
@@ -109,8 +103,9 @@ class FirewallTest(HeatTestCase):
 
         snippet = template_format.parse(firewall_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return firewall.Firewall(
-            'firewall', snippet['Resources']['firewall'], stack)
+            'firewall', resource_defns['firewall'], stack)
 
     def test_create(self):
         rsrc = self.create_firewall()
@@ -120,32 +115,31 @@ class FirewallTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_failed(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_firewall({
             'firewall': {
                 'name': 'test-firewall', 'admin_state_up': True,
                 'firewall_policy_id': 'policy-id'}}
-        ).AndRaise(firewall.NeutronClientException())
+        ).AndRaise(exceptions.NeutronClientException())
         self.m.ReplayAll()
 
         snippet = template_format.parse(firewall_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = firewall.Firewall(
-            'firewall', snippet['Resources']['firewall'], stack)
+            'firewall', resource_defns['firewall'], stack)
 
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_delete(self):
         neutronclient.Client.delete_firewall('5678')
         neutronclient.Client.show_firewall('5678').AndRaise(
-            firewall.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_firewall()
         self.m.ReplayAll()
@@ -156,7 +150,7 @@ class FirewallTest(HeatTestCase):
 
     def test_delete_already_gone(self):
         neutronclient.Client.delete_firewall('5678').AndRaise(
-            firewall.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_firewall()
         self.m.ReplayAll()
@@ -167,7 +161,7 @@ class FirewallTest(HeatTestCase):
 
     def test_delete_failed(self):
         neutronclient.Client.delete_firewall('5678').AndRaise(
-            firewall.NeutronClientException(status_code=400))
+            exceptions.NeutronClientException(status_code=400))
 
         rsrc = self.create_firewall()
         self.m.ReplayAll()
@@ -176,7 +170,7 @@ class FirewallTest(HeatTestCase):
                                   scheduler.TaskRunner(rsrc.delete))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
@@ -200,7 +194,7 @@ class FirewallTest(HeatTestCase):
                                   rsrc.FnGetAtt, 'subnet_id')
         self.assertEqual(
             'The Referenced Attribute (firewall subnet_id) is '
-            'incorrect.', str(error))
+            'incorrect.', six.text_type(error))
         self.m.VerifyAll()
 
     def test_update(self):
@@ -217,7 +211,6 @@ class FirewallTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class FirewallPolicyTest(HeatTestCase):
 
     def setUp(self):
@@ -226,12 +219,9 @@ class FirewallPolicyTest(HeatTestCase):
         self.m.StubOutWithMock(neutronclient.Client, 'delete_firewall_policy')
         self.m.StubOutWithMock(neutronclient.Client, 'show_firewall_policy')
         self.m.StubOutWithMock(neutronclient.Client, 'update_firewall_policy')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
     def create_firewall_policy(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_firewall_policy({
             'firewall_policy': {
                 'name': 'test-firewall-policy', 'shared': True,
@@ -240,8 +230,9 @@ class FirewallPolicyTest(HeatTestCase):
 
         snippet = template_format.parse(firewall_policy_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return firewall.FirewallPolicy(
-            'firewall_policy', snippet['Resources']['firewall_policy'], stack)
+            'firewall_policy', resource_defns['firewall_policy'], stack)
 
     def test_create(self):
         rsrc = self.create_firewall_policy()
@@ -251,32 +242,31 @@ class FirewallPolicyTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_failed(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_firewall_policy({
             'firewall_policy': {
                 'name': 'test-firewall-policy', 'shared': True,
                 'audited': True, 'firewall_rules': ['rule-id-1', 'rule-id-2']}}
-        ).AndRaise(firewall.NeutronClientException())
+        ).AndRaise(exceptions.NeutronClientException())
         self.m.ReplayAll()
 
         snippet = template_format.parse(firewall_policy_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = firewall.FirewallPolicy(
-            'firewall_policy', snippet['Resources']['firewall_policy'], stack)
+            'firewall_policy', resource_defns['firewall_policy'], stack)
 
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_delete(self):
         neutronclient.Client.delete_firewall_policy('5678')
         neutronclient.Client.show_firewall_policy('5678').AndRaise(
-            firewall.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_firewall_policy()
         self.m.ReplayAll()
@@ -287,7 +277,7 @@ class FirewallPolicyTest(HeatTestCase):
 
     def test_delete_already_gone(self):
         neutronclient.Client.delete_firewall_policy('5678').AndRaise(
-            firewall.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_firewall_policy()
         self.m.ReplayAll()
@@ -298,7 +288,7 @@ class FirewallPolicyTest(HeatTestCase):
 
     def test_delete_failed(self):
         neutronclient.Client.delete_firewall_policy('5678').AndRaise(
-            firewall.NeutronClientException(status_code=400))
+            exceptions.NeutronClientException(status_code=400))
 
         rsrc = self.create_firewall_policy()
         self.m.ReplayAll()
@@ -307,7 +297,7 @@ class FirewallPolicyTest(HeatTestCase):
                                   scheduler.TaskRunner(rsrc.delete))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
@@ -330,7 +320,7 @@ class FirewallPolicyTest(HeatTestCase):
                                   rsrc.FnGetAtt, 'subnet_id')
         self.assertEqual(
             'The Referenced Attribute (firewall_policy subnet_id) is '
-            'incorrect.', str(error))
+            'incorrect.', six.text_type(error))
         self.m.VerifyAll()
 
     def test_update(self):
@@ -347,7 +337,6 @@ class FirewallPolicyTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class FirewallRuleTest(HeatTestCase):
 
     def setUp(self):
@@ -356,12 +345,9 @@ class FirewallRuleTest(HeatTestCase):
         self.m.StubOutWithMock(neutronclient.Client, 'delete_firewall_rule')
         self.m.StubOutWithMock(neutronclient.Client, 'show_firewall_rule')
         self.m.StubOutWithMock(neutronclient.Client, 'update_firewall_rule')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
     def create_firewall_rule(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_firewall_rule({
             'firewall_rule': {
                 'name': 'test-firewall-rule', 'shared': True,
@@ -371,8 +357,9 @@ class FirewallRuleTest(HeatTestCase):
 
         snippet = template_format.parse(firewall_rule_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return firewall.FirewallRule(
-            'firewall_rule', snippet['Resources']['firewall_rule'], stack)
+            'firewall_rule', resource_defns['firewall_rule'], stack)
 
     def test_create(self):
         rsrc = self.create_firewall_rule()
@@ -382,33 +369,32 @@ class FirewallRuleTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_failed(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_firewall_rule({
             'firewall_rule': {
                 'name': 'test-firewall-rule', 'shared': True,
                 'action': 'allow', 'protocol': 'tcp', 'enabled': True,
                 'ip_version': "4"}}
-        ).AndRaise(firewall.NeutronClientException())
+        ).AndRaise(exceptions.NeutronClientException())
         self.m.ReplayAll()
 
         snippet = template_format.parse(firewall_rule_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = firewall.FirewallRule(
-            'firewall_rule', snippet['Resources']['firewall_rule'], stack)
+            'firewall_rule', resource_defns['firewall_rule'], stack)
 
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_delete(self):
         neutronclient.Client.delete_firewall_rule('5678')
         neutronclient.Client.show_firewall_rule('5678').AndRaise(
-            firewall.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_firewall_rule()
         self.m.ReplayAll()
@@ -419,7 +405,7 @@ class FirewallRuleTest(HeatTestCase):
 
     def test_delete_already_gone(self):
         neutronclient.Client.delete_firewall_rule('5678').AndRaise(
-            firewall.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_firewall_rule()
         self.m.ReplayAll()
@@ -430,7 +416,7 @@ class FirewallRuleTest(HeatTestCase):
 
     def test_delete_failed(self):
         neutronclient.Client.delete_firewall_rule('5678').AndRaise(
-            firewall.NeutronClientException(status_code=400))
+            exceptions.NeutronClientException(status_code=400))
 
         rsrc = self.create_firewall_rule()
         self.m.ReplayAll()
@@ -439,7 +425,7 @@ class FirewallRuleTest(HeatTestCase):
                                   scheduler.TaskRunner(rsrc.delete))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
@@ -462,7 +448,7 @@ class FirewallRuleTest(HeatTestCase):
                                   rsrc.FnGetAtt, 'subnet_id')
         self.assertEqual(
             'The Referenced Attribute (firewall_rule subnet_id) is '
-            'incorrect.', str(error))
+            'incorrect.', six.text_type(error))
         self.m.VerifyAll()
 
     def test_update(self):
