@@ -1,3 +1,4 @@
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -10,19 +11,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from testtools import skipIf
+from neutronclient.common import exceptions
+from neutronclient.v2_0 import client as neutronclient
+import six
 
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
 from heat.engine.resources.neutron import metering
 from heat.engine import scheduler
-from heat.openstack.common.importutils import try_import
 from heat.tests.common import HeatTestCase
-from heat.tests import fakes
 from heat.tests import utils
 
-neutronclient = try_import('neutronclient.v2_0.client')
 
 metering_template = '''
 {
@@ -51,7 +50,6 @@ metering_template = '''
 '''
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class MeteringLabelTest(HeatTestCase):
 
     def setUp(self):
@@ -65,12 +63,9 @@ class MeteringLabelTest(HeatTestCase):
                                'delete_metering_label_rule')
         self.m.StubOutWithMock(neutronclient.Client,
                                'show_metering_label_rule')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
     def create_metering_label(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_metering_label({
             'metering_label': {
                 'name': 'TestLabel',
@@ -79,8 +74,9 @@ class MeteringLabelTest(HeatTestCase):
 
         snippet = template_format.parse(metering_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return metering.MeteringLabel(
-            'label', snippet['Resources']['label'], stack)
+            'label', resource_defns['label'], stack)
 
     def test_create(self):
         rsrc = self.create_metering_label()
@@ -90,31 +86,30 @@ class MeteringLabelTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_failed(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_metering_label({
             'metering_label': {
                 'name': 'TestLabel',
                 'description': 'Description of TestLabel'}
-        }).AndRaise(metering.NeutronClientException())
+        }).AndRaise(exceptions.NeutronClientException())
         self.m.ReplayAll()
 
         snippet = template_format.parse(metering_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = metering.MeteringLabel(
-            'label', snippet['Resources']['label'], stack)
+            'label', resource_defns['label'], stack)
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_delete(self):
         neutronclient.Client.delete_metering_label('1234')
         neutronclient.Client.show_metering_label('1234').AndRaise(
-            metering.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_metering_label()
         self.m.ReplayAll()
@@ -125,7 +120,7 @@ class MeteringLabelTest(HeatTestCase):
 
     def test_delete_already_gone(self):
         neutronclient.Client.delete_metering_label('1234').AndRaise(
-            metering.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_metering_label()
         self.m.ReplayAll()
@@ -136,7 +131,7 @@ class MeteringLabelTest(HeatTestCase):
 
     def test_delete_failed(self):
         neutronclient.Client.delete_metering_label('1234').AndRaise(
-            metering.NeutronClientException(status_code=400))
+            exceptions.NeutronClientException(status_code=400))
 
         rsrc = self.create_metering_label()
         self.m.ReplayAll()
@@ -145,7 +140,7 @@ class MeteringLabelTest(HeatTestCase):
                                   scheduler.TaskRunner(rsrc.delete))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
@@ -164,7 +159,6 @@ class MeteringLabelTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class MeteringRuleTest(HeatTestCase):
 
     def setUp(self):
@@ -178,12 +172,9 @@ class MeteringRuleTest(HeatTestCase):
                                'delete_metering_label_rule')
         self.m.StubOutWithMock(neutronclient.Client,
                                'show_metering_label_rule')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
     def create_metering_label_rule(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_metering_label_rule({
             'metering_label_rule': {
                 'metering_label_id': 'None',
@@ -194,8 +185,9 @@ class MeteringRuleTest(HeatTestCase):
 
         snippet = template_format.parse(metering_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return metering.MeteringRule(
-            'rule', snippet['Resources']['rule'], stack)
+            'rule', resource_defns['rule'], stack)
 
     def test_create(self):
         rsrc = self.create_metering_label_rule()
@@ -205,33 +197,32 @@ class MeteringRuleTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_failed(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_metering_label_rule({
             'metering_label_rule': {
                 'metering_label_id': 'None',
                 'remote_ip_prefix': '10.0.3.0/24',
                 'direction': 'ingress',
                 'excluded': False}
-        }).AndRaise(metering.NeutronClientException())
+        }).AndRaise(exceptions.NeutronClientException())
         self.m.ReplayAll()
 
         snippet = template_format.parse(metering_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = metering.MeteringRule(
-            'rule', snippet['Resources']['rule'], stack)
+            'rule', resource_defns['rule'], stack)
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_delete(self):
         neutronclient.Client.delete_metering_label_rule('5678')
         neutronclient.Client.show_metering_label_rule('5678').AndRaise(
-            metering.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_metering_label_rule()
         self.m.ReplayAll()
@@ -242,7 +233,7 @@ class MeteringRuleTest(HeatTestCase):
 
     def test_delete_already_gone(self):
         neutronclient.Client.delete_metering_label_rule('5678').AndRaise(
-            metering.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_metering_label_rule()
         self.m.ReplayAll()
@@ -253,7 +244,7 @@ class MeteringRuleTest(HeatTestCase):
 
     def test_delete_failed(self):
         neutronclient.Client.delete_metering_label_rule('5678').AndRaise(
-            metering.NeutronClientException(status_code=400))
+            exceptions.NeutronClientException(status_code=400))
 
         rsrc = self.create_metering_label_rule()
         self.m.ReplayAll()
@@ -262,7 +253,7 @@ class MeteringRuleTest(HeatTestCase):
                                   scheduler.TaskRunner(rsrc.delete))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
