@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -11,14 +10,18 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
 """Tests for :module:'heat.engine.resources.nova_utls'."""
 
 import mock
+import six
 import uuid
 
+from novaclient import exceptions as nova_exceptions
+
 from heat.common import exception
-from heat.engine import clients
 from heat.engine.resources import nova_utils
+from heat.engine import scheduler
 from heat.tests.common import HeatTestCase
 from heat.tests.v1_1 import fakes
 
@@ -32,25 +35,10 @@ class NovaUtilsTests(HeatTestCase):
     def setUp(self):
         super(NovaUtilsTests, self).setUp()
         self.nova_client = self.m.CreateMockAnything()
-
-    def test_get_image_id(self):
-        """Tests the get_image_id function."""
-        my_image = self.m.CreateMockAnything()
-        img_id = str(uuid.uuid4())
-        img_name = 'myfakeimage'
-        my_image.id = img_id
-        my_image.name = img_name
-        self.nova_client.images = self.m.CreateMockAnything()
-        self.nova_client.images.get(img_id).AndReturn(my_image)
-        self.nova_client.images.list().MultipleTimes().AndReturn([my_image])
-        self.m.ReplayAll()
-        self.assertEqual(img_id, nova_utils.get_image_id(self.nova_client,
-                                                         img_id))
-        self.assertEqual(img_id, nova_utils.get_image_id(self.nova_client,
-                                                         'myfakeimage'))
-        self.assertRaises(exception.ImageNotFound, nova_utils.get_image_id,
-                          self.nova_client, 'noimage')
-        self.m.VerifyAll()
+        self.mock_warnings = mock.patch(
+            'heat.engine.resources.nova_utils.warnings')
+        self.mock_warnings.start()
+        self.addCleanup(self.mock_warnings.stop)
 
     def test_get_ip(self):
         my_image = self.m.CreateMockAnything()
@@ -100,7 +88,10 @@ class NovaUtilsTests(HeatTestCase):
         my_key.public_key = my_pub_key
         my_key.name = my_key_name
         self.nova_client.keypairs = self.m.CreateMockAnything()
-        self.nova_client.keypairs.list().MultipleTimes().AndReturn([my_key])
+        self.nova_client.keypairs.get(
+            my_key_name).AndReturn(my_key)
+        self.nova_client.keypairs.get(
+            'notakey').AndRaise(fakes.fake_exception())
         self.m.ReplayAll()
         self.assertEqual(my_key, nova_utils.get_keypair(self.nova_client,
                                                         my_key_name))
@@ -108,9 +99,44 @@ class NovaUtilsTests(HeatTestCase):
                           self.nova_client, 'notakey')
         self.m.VerifyAll()
 
+    def test_delete_server(self):
+        server = mock.Mock()
+        server.status = "DELETED"
+        task = scheduler.TaskRunner(nova_utils.delete_server, server)
+        self.assertIsNone(task())
+
+    def test_delete_server_notfound(self):
+        server = mock.Mock()
+        server.delete.side_effect = nova_exceptions.NotFound(404)
+        task = scheduler.TaskRunner(nova_utils.delete_server, server)
+        self.assertIsNone(task())
+
+    def test_delete_noserver(self):
+        task = scheduler.TaskRunner(nova_utils.delete_server, None)
+        self.assertIsNone(task())
+
+    def test_delete_servererror(self):
+        server = mock.Mock()
+        server.name = "myserver"
+        server.status = "ERROR"
+        server.fault = {
+            "message": "test error",
+        }
+        task = scheduler.TaskRunner(nova_utils.delete_server, server)
+        err = self.assertRaises(exception.Error, task)
+        self.assertIn("myserver delete failed: (None) test error",
+                      six.text_type(err))
+
 
 class NovaUtilsRefreshServerTests(HeatTestCase):
 
+    def setUp(self):
+        super(NovaUtilsRefreshServerTests, self).setUp()
+        self.mock_warnings = mock.patch(
+            'heat.engine.resources.nova_utils.warnings')
+        self.mock_warnings.start()
+        self.addCleanup(self.mock_warnings.stop)
+
     def test_successful_refresh(self):
         server = self.m.CreateMockAnything()
         server.get().AndReturn(None)
@@ -121,8 +147,7 @@ class NovaUtilsRefreshServerTests(HeatTestCase):
 
     def test_overlimit_error(self):
         server = mock.Mock()
-        server.get.side_effect = clients.novaclient.exceptions.OverLimit(
-            413, "limit reached")
+        server.get.side_effect = fakes.fake_exception(413)
         self.assertIsNone(nova_utils.refresh_server(server))
 
     def test_500_error(self):
@@ -143,13 +168,10 @@ class NovaUtilsRefreshServerTests(HeatTestCase):
 
     def test_unhandled_exception(self):
         server = self.m.CreateMockAnything()
-        msg = ("ClientException: The server has either erred or is "
-               "incapable of performing the requested operation.")
-        server.get().AndRaise(
-            clients.novaclient.exceptions.ClientException(501, msg))
+        server.get().AndRaise(fakes.fake_exception(501))
         self.m.ReplayAll()
 
-        self.assertRaises(clients.novaclient.exceptions.ClientException,
+        self.assertRaises(nova_exceptions.ClientException,
                           nova_utils.refresh_server, server)
         self.m.VerifyAll()
 
@@ -159,11 +181,15 @@ class NovaUtilsUserdataTests(HeatTestCase):
     def setUp(self):
         super(NovaUtilsUserdataTests, self).setUp()
         self.nova_client = self.m.CreateMockAnything()
+        self.mock_warnings = mock.patch(
+            'heat.engine.resources.nova_utils.warnings')
+        self.mock_warnings.start()
+        self.addCleanup(self.mock_warnings.stop)
 
     def test_build_userdata(self):
         """Tests the build_userdata function."""
         resource = self.m.CreateMockAnything()
-        resource.metadata = {}
+        resource.metadata_get().AndReturn({})
         self.m.StubOutWithMock(nova_utils.cfg, 'CONF')
         cnf = nova_utils.cfg.CONF
         cnf.heat_metadata_server_url = 'http://server.test:123'
@@ -185,7 +211,7 @@ class NovaUtilsUserdataTests(HeatTestCase):
     def test_build_userdata_without_instance_user(self):
         """Don't add a custom instance user when not requested."""
         resource = self.m.CreateMockAnything()
-        resource.metadata = {}
+        resource.metadata_get().AndReturn({})
         self.m.StubOutWithMock(nova_utils.cfg, 'CONF')
         cnf = nova_utils.cfg.CONF
         cnf.instance_user = 'config_instance_user'
@@ -201,7 +227,7 @@ class NovaUtilsUserdataTests(HeatTestCase):
     def test_build_userdata_with_instance_user(self):
         """Add the custom instance user when requested."""
         resource = self.m.CreateMockAnything()
-        resource.metadata = {}
+        resource.metadata_get().AndReturn(None)
         self.m.StubOutWithMock(nova_utils.cfg, 'CONF')
         cnf = nova_utils.cfg.CONF
         cnf.instance_user = 'config_instance_user'
@@ -217,6 +243,13 @@ class NovaUtilsUserdataTests(HeatTestCase):
 
 class NovaUtilsMetadataTests(HeatTestCase):
 
+    def setUp(self):
+        super(NovaUtilsMetadataTests, self).setUp()
+        self.mock_warnings = mock.patch(
+            'heat.engine.resources.nova_utils.warnings')
+        self.mock_warnings.start()
+        self.addCleanup(self.mock_warnings.stop)
+
     def test_serialize_string(self):
         original = {'test_key': 'simple string value'}
         self.assertEqual(original, nova_utils.meta_serialize(original))
