@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -12,12 +12,12 @@
 #    under the License.
 
 import json
+import six
 
 import testtools
 
 from heat.common import exception
 from heat.common import identifier
-from heat.engine import constraints as constr
 from heat.engine import parameters
 from heat.engine import template
 
@@ -26,7 +26,8 @@ class ParameterTest(testtools.TestCase):
 
     def new_parameter(self, name, schema, value=None,
                       validate_value=True):
-        tmpl = template.Template({'Parameters': {name: schema}})
+        tmpl = template.Template({'HeatTemplateFormatVersion': '2012-12-12',
+                                  'Parameters': {name: schema}})
         schema = tmpl.param_schemata()[name]
         param = parameters.Parameter(name, schema, value)
         param.validate(validate_value)
@@ -49,9 +50,24 @@ class ParameterTest(testtools.TestCase):
         p = self.new_parameter('p', {'Type': 'Json'}, validate_value=False)
         self.assertIsInstance(p, parameters.JsonParam)
 
+    def test_json_return(self):
+        p = self.new_parameter('p', {'Type': 'Json'}, {"a": 1, "b": "a"})
+        self.assertEqual('{"a": 1, "b": "a"}', str(p))
+
+    def test_json_return_no_echo_true(self):
+        p = self.new_parameter(
+            'p', {'Type': 'Json', 'NoEcho': 'true'}, {"a": 1})
+        self.assertTrue(p.hidden())
+        self.assertEqual(str(p), '******')
+
     def test_new_bad_type(self):
-        self.assertRaises(constr.InvalidSchemaError, self.new_parameter, 'p',
-                          {'Type': 'List'}, validate_value=False)
+        self.assertRaises(exception.InvalidSchemaError, self.new_parameter,
+                          'p', {'Type': 'List'}, validate_value=False)
+
+    def test_list_as_str(self):
+        p = self.new_parameter('p', {'Type': 'CommaDelimitedList'}, 'a,b,c')
+        self.assertEqual(['a', 'b', 'c'], p.value())
+        self.assertEqual('a,b,c', str(p))
 
     def test_default_no_override(self):
         p = self.new_parameter('defaulted', {'Type': 'String',
@@ -74,9 +90,9 @@ class ParameterTest(testtools.TestCase):
                   'AllowedValues': ['foo'],
                   'ConstraintDescription': 'wibble',
                   'Default': 'bar'}
-        err = self.assertRaises(constr.InvalidSchemaError,
+        err = self.assertRaises(exception.InvalidSchemaError,
                                 self.new_parameter, 'p', schema, 'foo')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
 
     def test_no_echo_true(self):
         p = self.new_parameter('anechoic',
@@ -124,17 +140,17 @@ class ParameterTest(testtools.TestCase):
         schema = {'Type': 'String',
                   'ConstraintDescription': 'wibble',
                   'MinLength': '4'}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, 'foo')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
 
     def test_string_overflow(self):
         schema = {'Type': 'String',
                   'ConstraintDescription': 'wibble',
                   'MaxLength': '2'}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, 'foo')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
 
     def test_string_pattern_good(self):
         schema = {'Type': 'String',
@@ -146,17 +162,17 @@ class ParameterTest(testtools.TestCase):
         schema = {'Type': 'String',
                   'ConstraintDescription': 'wibble',
                   'AllowedPattern': '[a-z]*'}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, '1foo')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
 
     def test_string_pattern_bad_suffix(self):
         schema = {'Type': 'String',
                   'ConstraintDescription': 'wibble',
                   'AllowedPattern': '[a-z]*'}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, 'foo1')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
 
     def test_string_value_list_good(self):
         schema = {'Type': 'String',
@@ -173,9 +189,9 @@ class ParameterTest(testtools.TestCase):
         schema = {'Type': 'String',
                   'ConstraintDescription': 'wibble',
                   'AllowedValues': ['foo', 'bar', 'baz']}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, 'blarg')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
 
     def test_number_int_good(self):
         schema = {'Type': 'Number',
@@ -184,28 +200,41 @@ class ParameterTest(testtools.TestCase):
         p = self.new_parameter('p', schema, '3')
         self.assertEqual(3, p.value())
 
-    def test_number_float_good(self):
+    def test_number_float_good_string(self):
         schema = {'Type': 'Number',
                   'MinValue': '3.0',
                   'MaxValue': '4.0'}
         p = self.new_parameter('p', schema, '3.5')
         self.assertEqual(3.5, p.value())
 
+    def test_number_float_good_number(self):
+        schema = {'Type': 'Number',
+                  'MinValue': '3.0',
+                  'MaxValue': '4.0'}
+        p = self.new_parameter('p', schema, 3.5)
+        self.assertEqual(3.5, p.value())
+
     def test_number_low(self):
         schema = {'Type': 'Number',
                   'ConstraintDescription': 'wibble',
                   'MinValue': '4'}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, '3')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
 
     def test_number_high(self):
         schema = {'Type': 'Number',
                   'ConstraintDescription': 'wibble',
                   'MaxValue': '2'}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, '3')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
+
+    def test_number_bad(self):
+        schema = {'Type': 'Number'}
+        err = self.assertRaises(exception.StackValidationFailed,
+                                self.new_parameter, 'p', schema, 'str')
+        self.assertIn('float', six.text_type(err))
 
     def test_number_value_list_good(self):
         schema = {'Type': 'Number',
@@ -217,9 +246,15 @@ class ParameterTest(testtools.TestCase):
         schema = {'Type': 'Number',
                   'ConstraintDescription': 'wibble',
                   'AllowedValues': ['1', '3', '5']}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, '2')
-        self.assertIn('wibble', str(err))
+        self.assertIn('wibble', six.text_type(err))
+
+    def test_list_value_list_default_empty(self):
+        schema = {'Type': 'CommaDelimitedList'}
+        schema['Default'] = ''
+        p = self.new_parameter('p', schema)
+        self.assertEqual([''], p.value())
 
     def test_list_value_list_good(self):
         schema = {'Type': 'CommaDelimitedList',
@@ -237,12 +272,13 @@ class ParameterTest(testtools.TestCase):
         schema = {'Type': 'CommaDelimitedList',
                   'ConstraintDescription': 'wibble',
                   'AllowedValues': ['foo', 'bar', 'baz']}
-        err = self.assertRaises(ValueError, self.new_parameter,
-                                'p', schema, 'foo,baz,blarg')
-        self.assertIn('wibble', str(err))
+        err = self.assertRaises(exception.StackValidationFailed,
+                                self.new_parameter, 'p', schema,
+                                'foo,baz,blarg')
+        self.assertIn('wibble', six.text_type(err))
 
     def test_map_value(self):
-        '''Happy path for value thats already a map.'''
+        '''Happy path for value that's already a map.'''
         schema = {'Type': 'Json'}
         val = {"foo": "bar", "items": [1, 2, 3]}
         p = self.new_parameter('p', schema, val)
@@ -256,7 +292,7 @@ class ParameterTest(testtools.TestCase):
         val = {"foo": "bar", "not_json": len}
         err = self.assertRaises(ValueError,
                                 self.new_parameter, 'p', schema, val)
-        self.assertIn('Value must be valid JSON', str(err))
+        self.assertIn('Value must be valid JSON', six.text_type(err))
 
     def test_map_value_parse(self):
         '''Happy path for value that's a string.'''
@@ -274,25 +310,59 @@ class ParameterTest(testtools.TestCase):
         val = "I am not a map"
         err = self.assertRaises(ValueError,
                                 self.new_parameter, 'p', schema, val)
-        self.assertIn('Value must be valid JSON', str(err))
+        self.assertIn('Value must be valid JSON', six.text_type(err))
 
     def test_map_underrun(self):
         '''Test map length under MIN_LEN.'''
         schema = {'Type': 'Json',
                   'MinLength': 3}
         val = {"foo": "bar", "items": [1, 2, 3]}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, val)
-        self.assertIn('out of range', str(err))
+        self.assertIn('out of range', six.text_type(err))
 
     def test_map_overrun(self):
         '''Test map length over MAX_LEN.'''
         schema = {'Type': 'Json',
                   'MaxLength': 1}
         val = {"foo": "bar", "items": [1, 2, 3]}
-        err = self.assertRaises(ValueError,
+        err = self.assertRaises(exception.StackValidationFailed,
                                 self.new_parameter, 'p', schema, val)
-        self.assertIn('out of range', str(err))
+        self.assertIn('out of range', six.text_type(err))
+
+    def test_json_list(self):
+        schema = {'Type': 'Json'}
+        val = ["fizz", "buzz"]
+        p = self.new_parameter('p', schema, val)
+        self.assertIsInstance(p.value(), list)
+        self.assertIn("fizz", p.value())
+        self.assertIn("buzz", p.value())
+
+    def test_json_string_list(self):
+        schema = {'Type': 'Json'}
+        val = '["fizz", "buzz"]'
+        p = self.new_parameter('p', schema, val)
+        self.assertIsInstance(p.value(), list)
+        self.assertIn("fizz", p.value())
+        self.assertIn("buzz", p.value())
+
+    def test_bool_value_true(self):
+        schema = {'Type': 'Boolean'}
+        for val in ('1', 't', 'true', 'on', 'y', 'yes', True, 1):
+            bo = self.new_parameter('bo', schema, val)
+            self.assertEqual(True, bo.value())
+
+    def test_bool_value_false(self):
+        schema = {'Type': 'Boolean'}
+        for val in ('0', 'f', 'false', 'off', 'n', 'no', False, 0):
+            bo = self.new_parameter('bo', schema, val)
+            self.assertEqual(False, bo.value())
+
+    def test_bool_value_invalid(self):
+        schema = {'Type': 'Boolean'}
+        err = self.assertRaises(exception.StackValidationFailed,
+                                self.new_parameter, 'bo', schema, 'foo')
+        self.assertIn("Unrecognized value 'foo'", unicode(err))
 
     def test_missing_param(self):
         '''Test missing user parameter.'''
@@ -300,6 +370,15 @@ class ParameterTest(testtools.TestCase):
                           self.new_parameter, 'p',
                           {'Type': 'String'})
 
+    def test_param_name_in_error_message(self):
+        schema = {'Type': 'String',
+                  'AllowedPattern': '[a-z]*'}
+        err = self.assertRaises(exception.StackValidationFailed,
+                                self.new_parameter, 'testparam', schema, '234')
+        expected = 'Parameter \'testparam\' is invalid: '\
+            '"234" does not match pattern "[a-z]*"'
+        self.assertEqual(expected, unicode(err))
+
 
 params_schema = json.loads('''{
   "Parameters" : {
@@ -313,8 +392,10 @@ params_schema = json.loads('''{
 
 
 class ParametersTest(testtools.TestCase):
-    def new_parameters(self, stack_name, tmpl, user_params={}, stack_id=None,
-                       validate_value=True):
+    def new_parameters(self, stack_name, tmpl, user_params=None,
+                       stack_id=None, validate_value=True):
+        user_params = user_params or {}
+        tmpl.update({'HeatTemplateFormatVersion': '2012-12-12'})
         tmpl = template.Template(tmpl)
         params = tmpl.parameters(
             identifier.HeatIdentifier('', stack_name, stack_id),
@@ -420,7 +501,7 @@ class ParametersTest(testtools.TestCase):
         params = {'Parameters': {'Foo': {'Type': 'String'},
                                  'NoAttr': 'No attribute.',
                                  'Bar': {'Type': 'Number', 'Default': '1'}}}
-        self.assertRaises(constr.InvalidSchemaError,
+        self.assertRaises(exception.InvalidSchemaError,
                           self.new_parameters,
                           'test',
                           params)
@@ -429,12 +510,16 @@ class ParametersTest(testtools.TestCase):
 class ParameterSchemaTest(testtools.TestCase):
 
     def test_validate_schema_wrong_key(self):
-        error = self.assertRaises(constr.InvalidSchemaError,
-                                  parameters.Schema.from_dict, {"foo": "bar"})
-        self.assertEqual("Invalid key 'foo' for parameter", str(error))
+        error = self.assertRaises(exception.InvalidSchemaError,
+                                  parameters.Schema.from_dict, 'param_name',
+                                  {"foo": "bar"})
+        self.assertEqual("Invalid key 'foo' for parameter (param_name)",
+                         six.text_type(error))
 
     def test_validate_schema_no_type(self):
-        error = self.assertRaises(constr.InvalidSchemaError,
+        error = self.assertRaises(exception.InvalidSchemaError,
                                   parameters.Schema.from_dict,
+                                  'broken',
                                   {"Description": "Hi!"})
-        self.assertEqual("Missing parameter type", str(error))
+        self.assertEqual("Missing parameter type for parameter: broken",
+                         six.text_type(error))
