@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -11,9 +11,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import six
 import testtools
 
 from heat.common import exception
+from heat.engine.cfn import functions as cfn_funcs
 from heat.engine import constraints
 from heat.engine.hot import parameters as hot_param
 from heat.engine import parameters
@@ -30,6 +32,7 @@ class PropertySchemaTest(testtools.TestCase):
             'default': 'wibble',
             'required': True,
             'update_allowed': False,
+            'immutable': False,
             'constraints': [
                 {'length': {'min': 4, 'max': 8}},
             ]
@@ -50,13 +53,15 @@ class PropertySchemaTest(testtools.TestCase):
                     'default': 'wibble',
                     'required': True,
                     'update_allowed': False,
+                    'immutable': False,
                     'constraints': [
                         {'length': {'min': 4, 'max': 8}},
                     ]
                 }
             },
             'required': False,
-            'update_allowed': False
+            'update_allowed': False,
+            'immutable': False,
         }
         s = properties.Schema(properties.Schema.STRING, 'A string',
                               default='wibble', required=True,
@@ -75,6 +80,7 @@ class PropertySchemaTest(testtools.TestCase):
                     'default': 'wibble',
                     'required': True,
                     'update_allowed': False,
+                    'immutable': False,
                     'constraints': [
                         {'length': {'min': 4, 'max': 8}},
                     ]
@@ -82,6 +88,7 @@ class PropertySchemaTest(testtools.TestCase):
             },
             'required': False,
             'update_allowed': False,
+            'immutable': False,
         }
         s = properties.Schema(properties.Schema.STRING, 'A string',
                               default='wibble', required=True,
@@ -105,6 +112,7 @@ class PropertySchemaTest(testtools.TestCase):
                             'default': 'wibble',
                             'required': True,
                             'update_allowed': False,
+                            'immutable': False,
                             'constraints': [
                                 {'length': {'min': 4, 'max': 8}},
                             ]
@@ -112,10 +120,12 @@ class PropertySchemaTest(testtools.TestCase):
                     },
                     'required': False,
                     'update_allowed': False,
+                    'immutable': False,
                 }
             },
             'required': False,
             'update_allowed': False,
+            'immutable': False,
         }
         s = properties.Schema(properties.Schema.STRING, 'A string',
                               default='wibble', required=True,
@@ -163,6 +173,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertEqual('wibble', s.default)
         self.assertTrue(s.required)
         self.assertEqual(3, len(s.constraints))
+        self.assertFalse(s.immutable)
 
     def test_from_legacy_min_length(self):
         s = properties.Schema.from_legacy({
@@ -310,7 +321,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertEqual('wibble', ss.default)
 
     def test_from_legacy_invalid_key(self):
-        self.assertRaises(constraints.InvalidSchemaError,
+        self.assertRaises(exception.InvalidSchemaError,
                           properties.Schema.from_legacy,
                           {'Type': 'String', 'Foo': 'Bar'})
 
@@ -320,7 +331,7 @@ class PropertySchemaTest(testtools.TestCase):
                           "m2.xlarge", "m2.2xlarge", "m2.4xlarge",
                           "c1.medium", "c1.xlarge", "cc1.4xlarge"]
         constraint_desc = "Must be a valid EC2 instance type."
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "String",
             "Description": description,
             "Default": "m1.large",
@@ -345,7 +356,7 @@ class PropertySchemaTest(testtools.TestCase):
         description = "WebServer EC2 instance type"
         allowed_pattern = "[A-Za-z0-9.]*"
         constraint_desc = "Must contain only alphanumeric characters."
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "String",
             "Description": description,
             "Default": "m1.large",
@@ -370,7 +381,7 @@ class PropertySchemaTest(testtools.TestCase):
         description = "WebServer EC2 instance type"
         allowed_pattern = "[A-Za-z0-9.]*"
         constraint_desc = "Must contain only alphanumeric characters."
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "String",
             "Description": description,
             "Default": "m1.large",
@@ -396,7 +407,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
     def test_from_param_string_min_len(self):
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
             "Type": "String",
             "Default": "m1.large",
@@ -413,7 +424,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertIsNone(len_constraint.max)
 
     def test_from_param_string_max_len(self):
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
             "Type": "String",
             "Default": "m1.large",
@@ -430,7 +441,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertEqual(11, len_constraint.max)
 
     def test_from_param_string_min_max_len(self):
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
             "Type": "String",
             "Default": "m1.large",
@@ -448,7 +459,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertEqual(11, len_constraint.max)
 
     def test_from_param_no_default(self):
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
             "Type": "String",
         })
@@ -460,7 +471,7 @@ class PropertySchemaTest(testtools.TestCase):
 
     def test_from_number_param_min(self):
         default = "42"
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "Number",
             "Default": default,
             "MinValue": "10",
@@ -480,7 +491,7 @@ class PropertySchemaTest(testtools.TestCase):
 
     def test_from_number_param_max(self):
         default = "42"
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "Number",
             "Default": default,
             "MaxValue": "100",
@@ -500,7 +511,7 @@ class PropertySchemaTest(testtools.TestCase):
 
     def test_from_number_param_min_max(self):
         default = "42"
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "Number",
             "Default": default,
             "MinValue": "10",
@@ -522,7 +533,7 @@ class PropertySchemaTest(testtools.TestCase):
     def test_from_number_param_allowed_vals(self):
         default = "42"
         constraint_desc = "The quick brown fox jumps over the lazy dog."
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "Number",
             "Default": default,
             "AllowedValues": ["10", "42", "100"],
@@ -542,7 +553,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
     def test_from_list_param(self):
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "CommaDelimitedList",
             "Default": "foo,bar,baz"
         })
@@ -554,7 +565,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertFalse(schema.required)
 
     def test_from_json_param(self):
-        param = parameters.Schema.from_dict({
+        param = parameters.Schema.from_dict('name', {
             "Type": "Json",
             "Default": {"foo": "bar", "blarg": "wibble"}
         })
@@ -604,11 +615,11 @@ class PropertyTest(testtools.TestCase):
         self.assertEqual('String', p.type())
 
     def test_bad_type(self):
-        self.assertRaises(constraints.InvalidSchemaError,
+        self.assertRaises(exception.InvalidSchemaError,
                           properties.Property, {'Type': 'Fish'})
 
     def test_bad_key(self):
-        self.assertRaises(constraints.InvalidSchemaError,
+        self.assertRaises(exception.InvalidSchemaError,
                           properties.Property,
                           {'Type': 'String', 'Foo': 'Bar'})
 
@@ -616,118 +627,126 @@ class PropertyTest(testtools.TestCase):
         schema = {'Type': 'String',
                   'AllowedPattern': '[a-z]*'}
         p = properties.Property(schema)
-        self.assertEqual('foo', p.validate_data('foo'))
+        self.assertEqual('foo', p.get_value('foo', True))
 
     def test_string_pattern_bad_prefix(self):
         schema = {'Type': 'String',
                   'AllowedPattern': '[a-z]*'}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, '1foo')
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, '1foo', True)
 
     def test_string_pattern_bad_suffix(self):
         schema = {'Type': 'String',
                   'AllowedPattern': '[a-z]*'}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, 'foo1')
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, 'foo1', True)
 
     def test_string_value_list_good(self):
         schema = {'Type': 'String',
                   'AllowedValues': ['foo', 'bar', 'baz']}
         p = properties.Property(schema)
-        self.assertEqual('bar', p.validate_data('bar'))
+        self.assertEqual('bar', p.get_value('bar', True))
 
     def test_string_value_list_bad(self):
         schema = {'Type': 'String',
                   'AllowedValues': ['foo', 'bar', 'baz']}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, 'blarg')
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, 'blarg', True)
 
     def test_string_maxlength_good(self):
         schema = {'Type': 'String',
                   'MaxLength': '5'}
         p = properties.Property(schema)
-        self.assertEqual('abcd', p.validate_data('abcd'))
+        self.assertEqual('abcd', p.get_value('abcd', True))
 
     def test_string_exceeded_maxlength(self):
         schema = {'Type': 'String',
                   'MaxLength': '5'}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, 'abcdef')
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, 'abcdef', True)
 
     def test_string_length_in_range(self):
         schema = {'Type': 'String',
                   'MinLength': '5',
                   'MaxLength': '10'}
         p = properties.Property(schema)
-        self.assertEqual('abcdef', p.validate_data('abcdef'))
+        self.assertEqual('abcdef', p.get_value('abcdef', True))
 
     def test_string_minlength_good(self):
         schema = {'Type': 'String',
                   'MinLength': '5'}
         p = properties.Property(schema)
-        self.assertEqual('abcde', p.validate_data('abcde'))
+        self.assertEqual('abcde', p.get_value('abcde', True))
 
     def test_string_smaller_than_minlength(self):
         schema = {'Type': 'String',
                   'MinLength': '5'}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, 'abcd')
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, 'abcd', True)
 
     def test_int_good(self):
         schema = {'Type': 'Integer',
                   'MinValue': 3,
                   'MaxValue': 3}
         p = properties.Property(schema)
-        self.assertEqual(3, p.validate_data(3))
+        self.assertEqual(3, p.get_value(3, True))
 
     def test_int_bad(self):
         schema = {'Type': 'Integer'}
         p = properties.Property(schema)
-        ex = self.assertRaises(TypeError, p.validate_data, [1])
+        ex = self.assertRaises(TypeError, p.get_value, [1])
         self.assertEqual("int() argument must be a string or a number, "
-                         "not 'list'", str(ex))
+                         "not 'list'", six.text_type(ex))
 
     def test_int_from_str_good(self):
         schema = {'Type': 'Integer'}
         p = properties.Property(schema)
-        self.assertEqual(3, p.validate_data('3'))
+        self.assertEqual(3, p.get_value('3'))
 
     def test_int_from_str_bad(self):
         schema = {'Type': 'Integer'}
         p = properties.Property(schema)
-        ex = self.assertRaises(TypeError, p.validate_data, '3a')
-        self.assertEqual("Value '3a' is not an integer", str(ex))
+        ex = self.assertRaises(TypeError, p.get_value, '3a')
+        self.assertEqual("Value '3a' is not an integer", six.text_type(ex))
 
     def test_integer_low(self):
         schema = {'Type': 'Integer',
                   'MinValue': 4}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, 3)
+        self.assertRaises(exception.StackValidationFailed, p.get_value, 3,
+                          True)
 
     def test_integer_high(self):
         schema = {'Type': 'Integer',
                   'MaxValue': 2}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, 3)
+        self.assertRaises(exception.StackValidationFailed, p.get_value, 3,
+                          True)
 
     def test_integer_value_list_good(self):
         schema = {'Type': 'Integer',
                   'AllowedValues': [1, 3, 5]}
         p = properties.Property(schema)
-        self.assertEqual(5, p.validate_data(5))
+        self.assertEqual(5, p.get_value(5), True)
 
     def test_integer_value_list_bad(self):
         schema = {'Type': 'Integer',
                   'AllowedValues': [1, 3, 5]}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, 2)
+        self.assertRaises(exception.StackValidationFailed, p.get_value, 2,
+                          True)
 
     def test_number_good(self):
         schema = {'Type': 'Number',
                   'MinValue': '3',
                   'MaxValue': '3'}
         p = properties.Property(schema)
-        self.assertEqual(3, p.validate_data(3))
+        self.assertEqual(3, p.get_value(3, True))
 
     def test_numbers_from_strings(self):
         """Numbers can be converted from strings."""
@@ -735,124 +754,129 @@ class PropertyTest(testtools.TestCase):
                   'MinValue': '3',
                   'MaxValue': '3'}
         p = properties.Property(schema)
-        self.assertEqual(3, p.validate_data('3'))
+        self.assertEqual(3, p.get_value('3'))
 
     def test_number_value_list_good(self):
         schema = {'Type': 'Number',
                   'AllowedValues': [1, 3, 5]}
         p = properties.Property(schema)
-        self.assertEqual(5, p.validate_data('5'))
+        self.assertEqual(5, p.get_value('5', True))
 
     def test_number_value_list_bad(self):
         schema = {'Type': 'Number',
                   'AllowedValues': ['1', '3', '5']}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, '2')
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, '2', True)
 
     def test_number_low(self):
         schema = {'Type': 'Number',
                   'MinValue': '4'}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, '3')
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, '3', True)
 
     def test_number_high(self):
         schema = {'Type': 'Number',
                   'MaxValue': '2'}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, '3')
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, '3', True)
 
     def test_boolean_true(self):
         p = properties.Property({'Type': 'Boolean'})
-        self.assertIs(True, p.validate_data('True'))
-        self.assertIs(True, p.validate_data('true'))
-        self.assertIs(True, p.validate_data(True))
+        self.assertIs(True, p.get_value('True'))
+        self.assertIs(True, p.get_value('true'))
+        self.assertIs(True, p.get_value(True))
 
     def test_boolean_false(self):
         p = properties.Property({'Type': 'Boolean'})
-        self.assertIs(False, p.validate_data('False'))
-        self.assertIs(False, p.validate_data('false'))
-        self.assertIs(False, p.validate_data(False))
+        self.assertIs(False, p.get_value('False'))
+        self.assertIs(False, p.get_value('false'))
+        self.assertIs(False, p.get_value(False))
 
     def test_boolean_invalid(self):
         p = properties.Property({'Type': 'Boolean'})
-        self.assertRaises(ValueError, p.validate_data, 'fish')
+        self.assertRaises(ValueError, p.get_value, 'fish')
 
     def test_list_string(self):
         p = properties.Property({'Type': 'List'})
-        self.assertRaises(TypeError, p.validate_data, 'foo')
+        self.assertRaises(TypeError, p.get_value, 'foo')
 
     def test_list_good(self):
         p = properties.Property({'Type': 'List'})
-        self.assertEqual(['foo', 'bar'], p.validate_data(['foo', 'bar']))
+        self.assertEqual(['foo', 'bar'], p.get_value(['foo', 'bar']))
 
     def test_list_dict(self):
         p = properties.Property({'Type': 'List'})
-        self.assertRaises(TypeError, p.validate_data, {'foo': 'bar'})
+        self.assertRaises(TypeError, p.get_value, {'foo': 'bar'})
 
     def test_list_maxlength_good(self):
         schema = {'Type': 'List',
                   'MaxLength': '3'}
         p = properties.Property(schema)
-        self.assertEqual(['1', '2'], p.validate_data(['1', '2']))
+        self.assertEqual(['1', '2'], p.get_value(['1', '2'], True))
 
     def test_list_exceeded_maxlength(self):
         schema = {'Type': 'List',
                   'MaxLength': '2'}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, ['1', '2', '3'])
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, ['1', '2', '3'], True)
 
     def test_list_length_in_range(self):
         schema = {'Type': 'List',
                   'MinLength': '2',
                   'MaxLength': '4'}
         p = properties.Property(schema)
-        self.assertEqual(['1', '2', '3'], p.validate_data(['1', '2', '3']))
+        self.assertEqual(['1', '2', '3'], p.get_value(['1', '2', '3'], True))
 
     def test_list_minlength_good(self):
         schema = {'Type': 'List',
                   'MinLength': '3'}
         p = properties.Property(schema)
-        self.assertEqual(['1', '2', '3'], p.validate_data(['1', '2', '3']))
+        self.assertEqual(['1', '2', '3'], p.get_value(['1', '2', '3'], True))
 
     def test_list_smaller_than_minlength(self):
         schema = {'Type': 'List',
                   'MinLength': '4'}
         p = properties.Property(schema)
-        self.assertRaises(ValueError, p.validate_data, ['1', '2', '3'])
+        self.assertRaises(exception.StackValidationFailed,
+                          p.get_value, ['1', '2', '3'], True)
 
     def test_map_string(self):
         p = properties.Property({'Type': 'Map'})
-        self.assertRaises(TypeError, p.validate_data, 'foo')
+        self.assertRaises(TypeError, p.get_value, 'foo')
 
     def test_map_list(self):
         p = properties.Property({'Type': 'Map'})
-        self.assertRaises(TypeError, p.validate_data, ['foo'])
+        self.assertRaises(TypeError, p.get_value, ['foo'])
 
     def test_map_schema_good(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
         p = properties.Property({'Type': 'Map', 'Schema': map_schema})
-        self.assertEqual({'valid': True}, p.validate_data({'valid': 'TRUE'}))
+        self.assertEqual({'valid': True}, p.get_value({'valid': 'TRUE'}))
 
     def test_map_schema_bad_data(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
         p = properties.Property({'Type': 'Map', 'Schema': map_schema})
         ex = self.assertRaises(exception.StackValidationFailed,
-                               p.validate_data, {'valid': 'fish'})
+                               p.get_value, {'valid': 'fish'}, True)
         self.assertEqual('Property error : valid "fish" is not '
-                         'a valid boolean', str(ex))
+                         'a valid boolean', six.text_type(ex))
 
     def test_map_schema_missing_data(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
         p = properties.Property({'Type': 'Map', 'Schema': map_schema})
-        self.assertEqual({'valid': None}, p.validate_data({}))
+        self.assertEqual({'valid': None}, p.get_value({}))
 
     def test_map_schema_missing_required_data(self):
         map_schema = {'valid': {'Type': 'Boolean', 'Required': True}}
         p = properties.Property({'Type': 'Map', 'Schema': map_schema})
         ex = self.assertRaises(exception.StackValidationFailed,
-                               p.validate_data, {})
+                               p.get_value, {}, True)
         self.assertEqual('Property error : Property valid not assigned',
-                         str(ex))
+                         six.text_type(ex))
 
     def test_list_schema_good(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
@@ -860,31 +884,31 @@ class PropertyTest(testtools.TestCase):
         p = properties.Property({'Type': 'List', 'Schema': list_schema})
         self.assertEqual([{'valid': True},
                           {'valid': False}],
-                         p.validate_data([{'valid': 'TRUE'},
-                                          {'valid': 'False'}]))
+                         p.get_value([{'valid': 'TRUE'},
+                                      {'valid': 'False'}]))
 
     def test_list_schema_bad_data(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
         list_schema = {'Type': 'Map', 'Schema': map_schema}
         p = properties.Property({'Type': 'List', 'Schema': list_schema})
         ex = self.assertRaises(exception.StackValidationFailed,
-                               p.validate_data,
-                               [{'valid': 'True'}, {'valid': 'fish'}])
+                               p.get_value,
+                               [{'valid': 'True'}, {'valid': 'fish'}], True)
         self.assertEqual('Property error : 1 Property error : 1: valid '
-                         '"fish" is not a valid boolean', str(ex))
+                         '"fish" is not a valid boolean', six.text_type(ex))
 
     def test_list_schema_int_good(self):
         list_schema = {'Type': 'Integer'}
         p = properties.Property({'Type': 'List', 'Schema': list_schema})
-        self.assertEqual([1, 2, 3], p.validate_data([1, 2, 3]))
+        self.assertEqual([1, 2, 3], p.get_value([1, 2, 3]))
 
     def test_list_schema_int_bad_data(self):
         list_schema = {'Type': 'Integer'}
         p = properties.Property({'Type': 'List', 'Schema': list_schema})
         ex = self.assertRaises(exception.StackValidationFailed,
-                               p.validate_data, [42, 'fish'])
+                               p.get_value, [42, 'fish'], True)
         self.assertEqual('Property error : 1 Value \'fish\' is not '
-                         'an integer', str(ex))
+                         'an integer', six.text_type(ex))
 
 
 class PropertiesTest(testtools.TestCase):
@@ -1016,7 +1040,64 @@ class PropertiesTest(testtools.TestCase):
 
         props = properties.Properties(schema, {'foo': 'baz'}, bad_resolver)
         err = self.assertRaises(ValueError, props.get, 'foo')
-        self.assertEqual('foo resolution failed!', str(err))
+        self.assertEqual('foo resolution failed!', six.text_type(err))
+
+    def test_resolve_returns_none(self):
+        schema = {'foo': {'Type': 'String', "MinLength": "5"}}
+
+        def test_resolver(prop):
+            return None
+
+        props = properties.Properties(schema,
+                                      {'foo': 'get_attr: [db, value]'},
+                                      test_resolver)
+        try:
+            self.assertIsNone(props.validate())
+        except exception.StackValidationFailed:
+            self.fail("Constraints should not have been evaluated.")
+
+    def test_resolve_ref_with_constraints(self):
+        # create test custom constraint
+        class IncorrectConstraint(constraints.BaseCustomConstraint):
+
+            expected_exceptions = (Exception,)
+
+            def validate_with_client(self, client, value):
+                raise Exception("Test exception")
+
+        class TestCustomConstraint(constraints.CustomConstraint):
+            @property
+            def custom_constraint(self):
+                return IncorrectConstraint()
+
+        # create schema with test constraint
+        schema = {
+            'foo': properties.Schema(
+                properties.Schema.STRING,
+                constraints=[TestCustomConstraint('test_constraint')]
+            )
+        }
+
+        # define parameters for function
+        def test_resolver(prop):
+            return 'None'
+
+        class rsrc(object):
+            action = INIT = "INIT"
+
+        stack = {'another_res': rsrc()}
+
+        # define properties with function and constraint
+        props = properties.Properties(
+            schema,
+            {'foo': cfn_funcs.ResourceRef(
+                stack, 'get_resource', 'another_res')},
+            test_resolver)
+
+        try:
+            self.assertIsNone(props.validate())
+        except exception.StackValidationFailed:
+            self.fail("Constraints should not have been evaluated.")
 
     def test_schema_from_params(self):
         params_snippet = {
@@ -1108,6 +1189,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "The WordPress database admin account username",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"length": {"min": 1, "max": 16},
                      "description": "must begin with a letter and contain "
@@ -1122,6 +1204,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "Distribution of choice",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"allowed_values": ["F18", "F17", "U10",
                                         "RHEL-6.1", "RHEL-6.2", "RHEL-6.3"]}
@@ -1132,6 +1215,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "WebServer EC2 instance type",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"allowed_values": ["t1.micro",
                                         "m1.small",
@@ -1151,6 +1235,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "Root password for MySQL",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"length": {"min": 1, "max": 41},
                      "description": "must contain only alphanumeric "
@@ -1166,12 +1251,14 @@ class PropertiesTest(testtools.TestCase):
                                 "SSH access to the instances"),
                 "required": True,
                 'update_allowed': True,
+                'immutable': False,
             },
             "DBPassword": {
                 "type": "string",
                 "description": "The WordPress database admin account password",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"length": {"min": 1, "max": 41},
                      "description": "must contain only alphanumeric "
@@ -1186,6 +1273,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "The WordPress database name",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"length": {"min": 1, "max": 64},
                      "description": "must begin with a letter and contain "
@@ -1196,7 +1284,7 @@ class PropertiesTest(testtools.TestCase):
                 ]
             },
         }
-        params = dict((n, parameters.Schema.from_dict(s)) for n, s
+        params = dict((n, parameters.Schema.from_dict(n, s)) for n, s
                       in params_snippet.items())
         props_schemata = properties.Properties.schema_from_params(params)
 
@@ -1291,12 +1379,14 @@ class PropertiesTest(testtools.TestCase):
                                 "SSH access to the instances"),
                 "required": True,
                 'update_allowed': True,
+                'immutable': False,
             },
             "InstanceType": {
                 "type": "string",
                 "description": "WebServer EC2 instance type",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"allowed_values": ["t1.micro", "m1.small", "m1.large",
                                         "m1.xlarge", "m2.xlarge", "m2.2xlarge",
@@ -1310,6 +1400,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "Distribution of choice",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"allowed_values": ["F18", "F17", "U10",
                                         "RHEL-6.1", "RHEL-6.2", "RHEL-6.3"],
@@ -1321,6 +1412,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "The WordPress database name",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"length": {"min": 1, "max": 64},
                      "description": "Length must be between 1 and 64"},
@@ -1334,6 +1426,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "The WordPress database admin account username",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"length": {"min": 1, "max": 16},
                      "description": "Length must be between 1 and 16"},
@@ -1347,6 +1440,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "The WordPress database admin account password",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"length": {"min": 1, "max": 41},
                      "description": "Length must be between 1 and 41"},
@@ -1360,6 +1454,7 @@ class PropertiesTest(testtools.TestCase):
                 "description": "Root password for MySQL",
                 "required": False,
                 'update_allowed': True,
+                'immutable': False,
                 "constraints": [
                     {"length": {"min": 1, "max": 41},
                      "description": "Length must be between 1 and 41"},
@@ -1369,13 +1464,25 @@ class PropertiesTest(testtools.TestCase):
                 ]
             }
         }
-        params = dict((n, hot_param.HOTParamSchema.from_dict(s)) for n, s
+        params = dict((n, hot_param.HOTParamSchema.from_dict(n, s)) for n, s
                       in params_snippet.items())
         props_schemata = properties.Properties.schema_from_params(params)
 
         self.assertEqual(expected,
                          dict((n, dict(s)) for n, s in props_schemata.items()))
 
+    def test_compare_same(self):
+        schema = {'foo': {'Type': 'Integer'}}
+        props_a = properties.Properties(schema, {'foo': 1})
+        props_b = properties.Properties(schema, {'foo': 1})
+        self.assertFalse(props_a != props_b)
+
+    def test_compare_different(self):
+        schema = {'foo': {'Type': 'Integer'}}
+        props_a = properties.Properties(schema, {'foo': 0})
+        props_b = properties.Properties(schema, {'foo': 1})
+        self.assertTrue(props_a != props_b)
+
 
 class PropertiesValidationTest(testtools.TestCase):
     def test_required(self):
@@ -1576,7 +1683,7 @@ class PropertiesValidationTest(testtools.TestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
         self.assertEqual('Property error : foo Property error : foo: 0 '
-                         'Unknown Property bar', str(ex))
+                         'Unknown Property bar', six.text_type(ex))
 
     def test_nested_properties_schema_invalid_property_in_map(self):
         child_schema = {'Key': {'Type': 'String',
@@ -1596,7 +1703,7 @@ class PropertiesValidationTest(testtools.TestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
         self.assertEqual('Property error : foo Property error : foo: boo '
-                         'Unknown Property bar', str(ex))
+                         'Unknown Property bar', six.text_type(ex))
 
     def test_more_nested_properties_schema_invalid_property_in_list(self):
         nested_child_schema = {'Key': {'Type': 'String',
@@ -1615,7 +1722,7 @@ class PropertiesValidationTest(testtools.TestCase):
                                props.validate)
         self.assertEqual('Property error : foo Property error : foo: 0 '
                          'Property error : 0: doo Unknown Property bar',
-                         str(ex))
+                         six.text_type(ex))
 
     def test_more_nested_properties_schema_invalid_property_in_map(self):
         nested_child_schema = {'Key': {'Type': 'String',
@@ -1634,4 +1741,20 @@ class PropertiesValidationTest(testtools.TestCase):
                                props.validate)
         self.assertEqual('Property error : foo Property error : foo: boo '
                          'Property error : boo: doo Unknown Property bar',
-                         str(ex))
+                         six.text_type(ex))
+
+    def test_schema_to_template_empty_schema(self):
+        schema = {}
+
+        (parameters, props) = \
+            properties.Properties.schema_to_parameters_and_properties(schema)
+        self.assertEqual({}, parameters)
+        self.assertEqual({}, props)
+
+    def test_update_allowed_and_immutable_contradict(self):
+        schema = {'foo': properties.Schema(
+            properties.Schema.STRING,
+            update_allowed=True,
+            immutable=True)}
+        props = properties.Properties(schema, {})
+        self.assertRaises(exception.InvalidSchemaError, props.validate)
