@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -13,28 +13,30 @@
 
 import json
 import os
+import six
 import uuid
 import yaml
 
-import testscenarios
+import mock
 
 from heat.common import exception
 from heat.common import template_format
 from heat.common import urlfetch
-
+from heat.engine import attributes
 from heat.engine import environment
 from heat.engine import parser
 from heat.engine import properties
 from heat.engine import resource
 from heat.engine import resources
 from heat.engine.resources import template_resource
-
+from heat.engine import rsrc_defn
+from heat.engine import support
 from heat.tests.common import HeatTestCase
 from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
 
 
-load_tests = testscenarios.load_tests_apply_scenarios
+empty_template = {"HeatTemplateFormatVersion": "2012-12-12"}
 
 
 class MyCloudResource(generic_rsrc.GenericResource):
@@ -44,7 +46,6 @@ class MyCloudResource(generic_rsrc.GenericResource):
 class ProviderTemplateTest(HeatTestCase):
     def setUp(self):
         super(ProviderTemplateTest, self).setUp()
-        utils.setup_dummy_db()
         resource._register_class('OS::ResourceType',
                                  generic_rsrc.GenericResource)
         resource._register_class('myCloud::ResourceType',
@@ -108,7 +109,9 @@ class ProviderTemplateTest(HeatTestCase):
         files = {'test_resource.template': json.dumps(provider)}
 
         class DummyResource(object):
-            attributes_schema = {"Foo": "A test attribute"}
+            support_status = support.SupportStatus()
+
+            attributes_schema = {"Foo": attributes.Schema("A test attribute")}
             properties_schema = {
                 "Foo": {"Type": "String"},
                 "AList": {"Type": "List"},
@@ -122,7 +125,8 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}, files=files), env=env,
+                             parser.Template(empty_template, files=files),
+                             env=env,
                              stack_id=str(uuid.uuid4()))
 
         map_prop_val = {
@@ -133,18 +137,18 @@ class ProviderTemplateTest(HeatTestCase):
                 "key5": False
             }
         }
-        json_snippet = {
-            "Type": "DummyResource",
-            "Properties": {
-                "Foo": "Bar",
-                "AList": ["one", "two", "three"],
-                "ListEmpty": [],
-                "ANum": 5,
-                "AMap": map_prop_val
-            }
+        prop_vals = {
+            "Foo": "Bar",
+            "AList": ["one", "two", "three"],
+            "ListEmpty": [],
+            "ANum": 5,
+            "AMap": map_prop_val,
         }
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  'DummyResource',
+                                                  prop_vals)
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      json_snippet, stack)
+                                                      definition, stack)
         temp_res.validate()
         converted_params = temp_res.child_params()
         self.assertTrue(converted_params)
@@ -153,15 +157,27 @@ class ProviderTemplateTest(HeatTestCase):
         # verify String conversion
         self.assertEqual("Bar", converted_params.get("Foo"))
         # verify List conversion
-        self.assertEqual(",".join(json_snippet.get("Properties",
-                                                   {}).get("AList",
-                                                           [])),
-                         converted_params.get("AList"))
+        self.assertEqual("one,two,three", converted_params.get("AList"))
         # verify Number conversion
         self.assertEqual(5, converted_params.get("ANum"))
         # verify Map conversion
         self.assertEqual(map_prop_val, converted_params.get("AMap"))
 
+        with mock.patch.object(properties.Properties, '__getitem__') as m_get:
+            m_get.side_effect = ValueError('boom')
+
+            # If the property doesn't exist on INIT, return default value
+            temp_res.action = temp_res.INIT
+            converted_params = temp_res.child_params()
+            for key in DummyResource.properties_schema:
+                self.assertIn(key, converted_params)
+            self.assertEqual({}, converted_params['AMap'])
+            self.assertEqual(0, converted_params['ANum'])
+
+            # If the property doesn't exist past INIT, then error out
+            temp_res.action = temp_res.CREATE
+            self.assertRaises(ValueError, temp_res.child_params)
+
     def test_attributes_extra(self):
         provider = {
             'HeatTemplateFormatVersion': '2012-12-12',
@@ -173,27 +189,28 @@ class ProviderTemplateTest(HeatTestCase):
         files = {'test_resource.template': json.dumps(provider)}
 
         class DummyResource(object):
+            support_status = support.SupportStatus()
             properties_schema = {}
-            attributes_schema = {"Foo": "A test attribute"}
+            attributes_schema = {"Foo": attributes.Schema("A test attribute")}
 
         env = environment.Environment()
         resource._register_class('DummyResource', DummyResource)
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}, files=files), env=env,
+                             parser.Template(empty_template, files=files),
+                             env=env,
                              stack_id=str(uuid.uuid4()))
 
-        json_snippet = {
-            "Type": "DummyResource",
-        }
-
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource")
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      json_snippet, stack)
+                                                      definition, stack)
         self.assertIsNone(temp_res.validate())
 
     def test_attributes_missing(self):
         provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
             'Outputs': {
                 'Blarg': {'Value': 'wibble'},
             },
@@ -201,23 +218,23 @@ class ProviderTemplateTest(HeatTestCase):
         files = {'test_resource.template': json.dumps(provider)}
 
         class DummyResource(object):
+            support_status = support.SupportStatus()
             properties_schema = {}
-            attributes_schema = {"Foo": "A test attribute"}
-
-        json_snippet = {
-            "Type": "DummyResource",
-        }
+            attributes_schema = {"Foo": attributes.Schema("A test attribute")}
 
         env = environment.Environment()
         resource._register_class('DummyResource', DummyResource)
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}, files=files), env=env,
+                             parser.Template(empty_template, files=files),
+                             env=env,
                              stack_id=str(uuid.uuid4()))
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource")
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      json_snippet, stack)
+                                                      definition, stack)
         self.assertRaises(exception.StackValidationFailed,
                           temp_res.validate)
 
@@ -232,32 +249,31 @@ class ProviderTemplateTest(HeatTestCase):
         files = {'test_resource.template': json.dumps(provider)}
 
         class DummyResource(object):
+            support_status = support.SupportStatus()
             properties_schema = {"Foo":
                                  properties.Schema(properties.Schema.STRING,
                                                    required=True)}
             attributes_schema = {}
 
-        json_snippet = {
-            "Type": "DummyResource",
-            "Properties": {
-                "Foo": "bar",
-            },
-        }
-
         env = environment.Environment()
         resource._register_class('DummyResource', DummyResource)
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}, files=files), env=env,
+                             parser.Template(empty_template, files=files),
+                             env=env,
                              stack_id=str(uuid.uuid4()))
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Foo": "bar"})
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      json_snippet, stack)
+                                                      definition, stack)
         self.assertIsNone(temp_res.validate())
 
     def test_properties_missing(self):
         provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
             'Parameters': {
                 'Blarg': {'Type': 'String', 'Default': 'wibble'},
             },
@@ -265,30 +281,31 @@ class ProviderTemplateTest(HeatTestCase):
         files = {'test_resource.template': json.dumps(provider)}
 
         class DummyResource(object):
+            support_status = support.SupportStatus()
             properties_schema = {"Foo":
                                  properties.Schema(properties.Schema.STRING,
                                                    required=True)}
             attributes_schema = {}
 
-        json_snippet = {
-            "Type": "DummyResource",
-        }
-
         env = environment.Environment()
         resource._register_class('DummyResource', DummyResource)
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}, files=files), env=env,
+                             parser.Template(empty_template, files=files),
+                             env=env,
                              stack_id=str(uuid.uuid4()))
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource")
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      json_snippet, stack)
+                                                      definition, stack)
         self.assertRaises(exception.StackValidationFailed,
                           temp_res.validate)
 
     def test_properties_extra_required(self):
         provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
             'Parameters': {
                 'Blarg': {'Type': 'String'},
             },
@@ -296,31 +313,30 @@ class ProviderTemplateTest(HeatTestCase):
         files = {'test_resource.template': json.dumps(provider)}
 
         class DummyResource(object):
+            support_status = support.SupportStatus()
             properties_schema = {}
             attributes_schema = {}
 
-        json_snippet = {
-            "Type": "DummyResource",
-            "Properties": {
-                "Blarg": "wibble",
-            },
-        }
-
         env = environment.Environment()
         resource._register_class('DummyResource', DummyResource)
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}, files=files), env=env,
+                             parser.Template(empty_template, files=files),
+                             env=env,
                              stack_id=str(uuid.uuid4()))
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Blarg": "wibble"})
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      json_snippet, stack)
+                                                      definition, stack)
         self.assertRaises(exception.StackValidationFailed,
                           temp_res.validate)
 
     def test_properties_type_mismatch(self):
         provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
             'Parameters': {
                 'Foo': {'Type': 'String'},
             },
@@ -328,29 +344,95 @@ class ProviderTemplateTest(HeatTestCase):
         files = {'test_resource.template': json.dumps(provider)}
 
         class DummyResource(object):
+            support_status = support.SupportStatus()
             properties_schema = {"Foo":
                                  properties.Schema(properties.Schema.MAP)}
             attributes_schema = {}
 
-        json_snippet = {
-            "Type": "DummyResource",
-            "Properties": {
-                "Foo": "bar",
-            },
-        }
-
         env = environment.Environment()
         resource._register_class('DummyResource', DummyResource)
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}, files=files), env=env,
+                             parser.Template(
+                                 {'HeatTemplateFormatVersion': '2012-12-12'},
+                                 files=files), env=env,
                              stack_id=str(uuid.uuid4()))
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Foo": "bar"})
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      json_snippet, stack)
-        self.assertRaises(exception.StackValidationFailed,
-                          temp_res.validate)
+                                                      definition, stack)
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               temp_res.validate)
+        self.assertEqual("Property Foo type mismatch between facade "
+                         "DummyResource (Map) and provider (String)",
+                         six.text_type(ex))
+
+    def test_properties_type_match(self):
+        provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'Length': {'Type': 'Number'},
+            },
+        }
+        files = {'test_resource.template': json.dumps(provider)}
+
+        class DummyResource(object):
+            support_status = support.SupportStatus()
+            properties_schema = {"Length":
+                                 properties.Schema(properties.Schema.INTEGER)}
+            attributes_schema = {}
+
+        env = environment.Environment()
+        resource._register_class('DummyResource', DummyResource)
+        env.load({'resource_registry':
+                  {'DummyResource': 'test_resource.template'}})
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             parser.Template(
+                                 {'HeatTemplateFormatVersion': '2012-12-12'},
+                                 files=files), env=env,
+                             stack_id=str(uuid.uuid4()))
+
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Length": 10})
+        temp_res = template_resource.TemplateResource('test_t_res',
+                                                      definition, stack)
+        self.assertIsNone(temp_res.validate())
+
+    def test_boolean_type_provider(self):
+        provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'Foo': {'Type': 'Boolean'},
+            },
+        }
+        files = {'test_resource.template': json.dumps(provider)}
+
+        class DummyResource(object):
+            support_status = support.SupportStatus()
+            properties_schema = {"Foo":
+                                 properties.Schema(properties.Schema.BOOLEAN)}
+            attributes_schema = {}
+
+        env = environment.Environment()
+        resource._register_class('DummyResource', DummyResource)
+        env.load({'resource_registry':
+                  {'DummyResource': 'test_resource.template'}})
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             parser.Template(
+                                 {'HeatTemplateFormatVersion': '2012-12-12'},
+                                 files=files), env=env,
+                             stack_id=str(uuid.uuid4()))
+
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Foo": "False"})
+        temp_res = template_resource.TemplateResource('test_t_res',
+                                                      definition, stack)
+        self.assertIsNone(temp_res.validate())
 
     def test_get_template_resource(self):
         # assertion: if the name matches {.yaml|.template} we get the
@@ -400,25 +482,29 @@ class ProviderTemplateTest(HeatTestCase):
         self.assertTrue(test_templ, "Empty test template")
         self.m.StubOutWithMock(urlfetch, "get")
         urlfetch.get(test_templ_name,
-                     allowed_schemes=('file',)).AndRaise(IOError)
+                     allowed_schemes=('file',))\
+            .AndRaise(urlfetch.URLFetchError(_('Failed to retrieve template')))
         urlfetch.get(test_templ_name,
                      allowed_schemes=('http', 'https')).AndReturn(test_templ)
         parsed_test_templ = template_format.parse(test_templ)
         self.m.ReplayAll()
-        json_snippet = {
-            "Type": test_templ_name,
-            "Properties": {
-                "KeyName": "mykeyname",
-                "DBName": "wordpress1",
-                "DBUsername": "wpdbuser",
-                "DBPassword": "wpdbpass",
-                "DBRootPassword": "wpdbrootpass",
-                "LinuxDistribution": "U10"
-            }
-        }
+
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}), stack_id=str(uuid.uuid4()))
-        templ_resource = resource.Resource("test_templ_resource", json_snippet,
+                             parser.Template(empty_template),
+                             stack_id=str(uuid.uuid4()))
+
+        properties = {
+            "KeyName": "mykeyname",
+            "DBName": "wordpress1",
+            "DBUsername": "wpdbuser",
+            "DBPassword": "wpdbpass",
+            "DBRootPassword": "wpdbrootpass",
+            "LinuxDistribution": "U10"
+        }
+        definition = rsrc_defn.ResourceDefinition("test_templ_resource",
+                                                  test_templ_name,
+                                                  properties)
+        templ_resource = resource.Resource("test_templ_resource", definition,
                                            stack)
         self.m.VerifyAll()
         self.assertIsInstance(templ_resource,
@@ -427,7 +513,7 @@ class ProviderTemplateTest(HeatTestCase):
             self.assertIn(prop, templ_resource.properties)
         for attrib in parsed_test_templ.get("Outputs", {}):
             self.assertIn(attrib, templ_resource.attributes)
-        for k, v in json_snippet.get("Properties").items():
+        for k, v in properties.items():
             self.assertEqual(v, templ_resource.properties[k])
         self.assertEqual(
             {'WordPress_Single_Instance.yaml':
@@ -455,7 +541,7 @@ class ProviderTemplateTest(HeatTestCase):
         g_env.load({'resource_registry':
                    {'Test::Frodo': test_templ_name}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}),
+                             parser.Template(empty_template),
                              stack_id=str(uuid.uuid4()))
 
         minimal_temp = json.dumps({'HeatTemplateFormatVersion': '2012-12-12',
@@ -467,8 +553,10 @@ class ProviderTemplateTest(HeatTestCase):
                                       'file')).AndReturn(minimal_temp)
         self.m.ReplayAll()
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  'Test::Frodo')
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      {"Type": 'Test::Frodo'},
+                                                      definition,
                                                       stack)
         self.assertIsNone(temp_res.validate())
         self.m.VerifyAll()
@@ -481,11 +569,13 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'Test::Flippy': test_templ_name}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}), env=env,
+                             parser.Template(empty_template), env=env,
                              stack_id=str(uuid.uuid4()))
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  'Test::Flippy')
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      {"Type": 'Test::Flippy'},
+                                                      definition,
                                                       stack)
 
         self.assertRaises(exception.StackValidationFailed, temp_res.validate)
@@ -501,17 +591,20 @@ class ProviderTemplateTest(HeatTestCase):
         g_env.load({'resource_registry':
                    {'Test::Frodo': test_templ_name}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}),
+                             parser.Template(empty_template),
                              stack_id=str(uuid.uuid4()))
 
         self.m.StubOutWithMock(urlfetch, "get")
         urlfetch.get(test_templ_name,
                      allowed_schemes=('http', 'https',
-                                      'file')).AndRaise(IOError)
+                                      'file'))\
+            .AndRaise(urlfetch.URLFetchError(_('Failed to retrieve template')))
         self.m.ReplayAll()
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  'Test::Frodo')
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      {"Type": 'Test::Frodo'},
+                                                      definition,
                                                       stack)
         self.assertRaises(exception.StackValidationFailed, temp_res.validate)
         self.m.VerifyAll()
@@ -527,16 +620,19 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'Test::Flippy': test_templ_name}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}), env=env,
+                             parser.Template(empty_template), env=env,
                              stack_id=str(uuid.uuid4()))
 
         self.m.StubOutWithMock(urlfetch, "get")
         urlfetch.get(test_templ_name,
-                     allowed_schemes=('http', 'https')).AndRaise(IOError)
+                     allowed_schemes=('http', 'https'))\
+            .AndRaise(urlfetch.URLFetchError(_('Failed to retrieve template')))
         self.m.ReplayAll()
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  'Test::Flippy')
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      {"Type": 'Test::Flippy'},
+                                                      definition,
                                                       stack)
         self.assertRaises(exception.StackValidationFailed, temp_res.validate)
         self.m.VerifyAll()
@@ -552,13 +648,15 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'Test::Flippy': test_templ_name}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template({}), env=env,
+                             parser.Template(empty_template), env=env,
                              stack_id=str(uuid.uuid4()))
 
         self.m.ReplayAll()
 
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  'Test::Flippy')
         temp_res = template_resource.TemplateResource('test_t_res',
-                                                      {"Type": 'Test::Flippy'},
+                                                      definition,
                                                       stack)
         self.assertRaises(exception.StackValidationFailed, temp_res.validate)
         self.m.VerifyAll()
@@ -770,7 +868,6 @@ Outputs:
 
     def setUp(self):
         super(ProviderTemplateUpdateTest, self).setUp()
-        utils.setup_dummy_db()
         self.ctx = utils.dummy_context('test_username', 'aaaa', 'password')
 
     def create_stack(self):
@@ -782,7 +879,6 @@ Outputs:
         self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
         return stack
 
-    @utils.stack_delete_after
     def test_template_resource_update_template_schema(self):
         stack = self.create_stack()
         self.stack = stack
@@ -809,3 +905,79 @@ Outputs:
             self.assertNotEqual(initial_val,
                                 stack.output('value'))
         self.m.VerifyAll()
+
+
+class ProviderTemplateAdoptTest(HeatTestCase):
+    main_template = '''
+HeatTemplateFormatVersion: '2012-12-12'
+Resources:
+  the_nested:
+    Type: the.yaml
+    Properties:
+      one: my_name
+
+Outputs:
+  identifier:
+    Value: {Ref: the_nested}
+  value:
+    Value: {'Fn::GetAtt': [the_nested, the_str]}
+'''
+
+    nested_tmpl = '''
+HeatTemplateFormatVersion: '2012-12-12'
+Parameters:
+  one:
+    Default: foo
+    Type: String
+Resources:
+  RealRandom:
+    Type: OS::Heat::RandomString
+    Properties:
+      salt: {Ref: one}
+Outputs:
+  the_str:
+    Value: {'Fn::GetAtt': [RealRandom, value]}
+'''
+
+    def setUp(self):
+        super(ProviderTemplateAdoptTest, self).setUp()
+        self.ctx = utils.dummy_context('test_username', 'aaaa', 'password')
+
+    def _yaml_to_json(self, yaml_templ):
+        return yaml.load(yaml_templ)
+
+    def test_abandon(self):
+        t = template_format.parse(self.main_template)
+        tmpl = parser.Template(t, files={'the.yaml': self.nested_tmpl})
+        stack = parser.Stack(self.ctx, utils.random_name(), tmpl)
+        stack.store()
+        stack.create()
+        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
+        info = stack.prepare_abandon()
+        self.assertEqual(self._yaml_to_json(self.main_template),
+                         info['template'])
+        self.assertEqual(self._yaml_to_json(self.nested_tmpl),
+                         info['resources']['the_nested']['template'])
+        self.m.VerifyAll()
+
+    def test_adopt(self):
+        data = {'resources': {u'the_nested': {
+            'resources': {u'RealRandom': {
+                'resource_data': {
+                    u'value': u'N8hE5C7ijdGn4RwnuygbAokGHnTq4cFJ'},
+                'resource_id': 'N8hE5C7ijdGn4RwnuygbAokGHnTq4cFJ'}}}}}
+
+        t = template_format.parse(self.main_template)
+        tmpl = parser.Template(t, files={'the.yaml': self.nested_tmpl})
+        stack = parser.Stack(self.ctx, utils.random_name(), tmpl,
+                             adopt_stack_data=data)
+        self.stack = stack
+        stack.store()
+        stack.adopt()
+
+        self.assertEqual(('ADOPT', 'COMPLETE'), stack.state)
+        nested_res = data['resources']['the_nested']['resources']
+        self.assertEqual(nested_res['RealRandom']['resource_data']['value'],
+                         stack.output('value'))
+
+        self.m.VerifyAll()
