@@ -14,7 +14,7 @@
 # limitations under the License.
 
 import mock
-from oslo_config import cfg
+from oslo.config import cfg
 import six
 
 from heat.common import exception
@@ -22,9 +22,9 @@ from heat.common import template_format
 from heat.engine.clients.os import glance
 from heat.engine.clients.os import neutron
 from heat.engine.clients.os import sahara
-from heat.engine.resources.openstack.sahara import sahara_cluster as sc
+from heat.engine.resources import sahara_cluster as sc
 from heat.engine import scheduler
-from heat.tests import common
+from heat.tests.common import HeatTestCase
 from heat.tests import utils
 
 
@@ -39,7 +39,7 @@ resources:
       plugin_name: vanilla
       hadoop_version: 2.3.0
       cluster_template_id: some_cluster_template_id
-      default_image_id: some_image
+      image: some_image
       key_name: admin
       neutron_management_network: some_network
 """
@@ -52,10 +52,9 @@ class FakeCluster(object):
         self.name = "super-cluster"
         self.info = {"HDFS": {"NameNode": "hdfs://hostname:port",
                               "Web UI": "http://host_ip:port"}}
-        self.to_dict = lambda: {"cluster": "info"}
 
 
-class SaharaClusterTest(common.HeatTestCase):
+class SaharaClusterTest(HeatTestCase):
     def setUp(self):
         super(SaharaClusterTest, self).setUp()
         self.patchobject(sc.constraints.CustomConstraint, '_is_valid'
@@ -68,16 +67,14 @@ class SaharaClusterTest(common.HeatTestCase):
         self.sahara_mock = mock.MagicMock()
         self.patchobject(sahara.SaharaClientPlugin, '_create'
                          ).return_value = self.sahara_mock
-        self.patchobject(sahara.SaharaClientPlugin, 'validate_hadoop_version'
-                         ).return_value = None
         self.cl_mgr = self.sahara_mock.clusters
         self.fake_cl = FakeCluster()
 
         self.t = template_format.parse(cluster_stack_template)
 
     def _init_cluster(self, template):
-        self.stack = utils.parse_stack(template)
-        cluster = self.stack['super-cluster']
+        stack = utils.parse_stack(template)
+        cluster = stack['super-cluster']
         return cluster
 
     def _create_cluster(self, template):
@@ -96,23 +93,17 @@ class SaharaClusterTest(common.HeatTestCase):
         expected_kwargs = {'cluster_template_id': 'some_cluster_template_id',
                            'user_keypair_id': 'admin',
                            'default_image_id': 'some_image_id',
-                           'net_id': 'some_network_id',
-                           'use_autoconfig': None}
+                           'net_id': 'some_network_id'}
         self.cl_mgr.create.assert_called_once_with(*expected_args,
                                                    **expected_kwargs)
         self.cl_mgr.get.assert_called_once_with(self.fake_cl.id)
 
     def test_cluster_delete(self):
         cluster = self._create_cluster(self.t)
-        self.cl_mgr.get.side_effect = [
-            self.fake_cl,
-            sahara.sahara_base.APIException(error_code=404)]
-        self.cl_mgr.get.reset_mock()
         scheduler.TaskRunner(cluster.delete)()
         self.assertEqual((cluster.DELETE, cluster.COMPLETE),
                          cluster.state)
         self.cl_mgr.delete.assert_called_once_with(self.fake_cl.id)
-        self.assertEqual(2, self.cl_mgr.get.call_count)
 
     def test_cluster_create_fails(self):
         cfg.CONF.set_override('action_retry_limit', 0)
@@ -121,8 +112,7 @@ class SaharaClusterTest(common.HeatTestCase):
         self.cl_mgr.get.return_value = FakeCluster(status='Error')
         create_task = scheduler.TaskRunner(cluster.create)
         ex = self.assertRaises(exception.ResourceFailure, create_task)
-        expected = ('ResourceInError: resources.super-cluster: '
-                    'Went to status Error due to "Unknown"')
+        expected = 'ResourceInError: Went to status Error due to "Unknown"'
         self.assertEqual(expected, six.text_type(ex))
 
     def test_cluster_delete_fails(self):
@@ -130,7 +120,7 @@ class SaharaClusterTest(common.HeatTestCase):
         self.cl_mgr.delete.side_effect = sahara.sahara_base.APIException()
         delete_task = scheduler.TaskRunner(cluster.delete)
         ex = self.assertRaises(exception.ResourceFailure, delete_task)
-        expected = "APIException: resources.super-cluster: None"
+        expected = "APIException: None"
         self.assertEqual(expected, six.text_type(ex))
         self.cl_mgr.delete.assert_called_once_with(self.fake_cl.id)
 
@@ -141,33 +131,6 @@ class SaharaClusterTest(common.HeatTestCase):
         scheduler.TaskRunner(cluster.delete)()
         self.cl_mgr.delete.assert_called_once_with(self.fake_cl.id)
 
-    def test_cluster_check_delete_complete_error(self):
-        cluster = self._create_cluster(self.t)
-        self.cl_mgr.get.side_effect = [
-            self.fake_cl,
-            sahara.sahara_base.APIException()]
-        self.cl_mgr.get.reset_mock()
-        delete_task = scheduler.TaskRunner(cluster.delete)
-        ex = self.assertRaises(exception.ResourceFailure, delete_task)
-        expected = "APIException: resources.super-cluster: None"
-        self.assertEqual(expected, six.text_type(ex))
-        self.cl_mgr.delete.assert_called_once_with(self.fake_cl.id)
-        self.assertEqual(2, self.cl_mgr.get.call_count)
-
-    def test_cluster_delete_cluster_in_error(self):
-        cluster = self._create_cluster(self.t)
-        self.cl_mgr.get.side_effect = [
-            self.fake_cl,
-            FakeCluster(status='Error')]
-        self.cl_mgr.get.reset_mock()
-        delete_task = scheduler.TaskRunner(cluster.delete)
-        ex = self.assertRaises(exception.ResourceFailure, delete_task)
-        expected = ('ResourceInError: resources.super-cluster: '
-                    'Went to status Error due to "Unknown"')
-        self.assertEqual(expected, six.text_type(ex))
-        self.cl_mgr.delete.assert_called_once_with(self.fake_cl.id)
-        self.assertEqual(2, self.cl_mgr.get.call_count)
-
     def test_cluster_resolve_attribute(self):
         cluster = self._create_cluster(self.t)
         self.cl_mgr.get.reset_mock()
@@ -175,8 +138,7 @@ class SaharaClusterTest(common.HeatTestCase):
                          cluster._resolve_attribute('info'))
         self.assertEqual(self.fake_cl.status,
                          cluster._resolve_attribute('status'))
-        self.assertEqual({"cluster": "info"}, cluster.FnGetAtt('show'))
-        self.assertEqual(3, self.cl_mgr.get.call_count)
+        self.assertEqual(2, self.cl_mgr.get.call_count)
 
     def test_cluster_resource_mapping(self):
         cluster = self._init_cluster(self.t)
@@ -187,8 +149,7 @@ class SaharaClusterTest(common.HeatTestCase):
         self.assertIsInstance(cluster, sc.SaharaCluster)
 
     def test_cluster_create_no_image_anywhere_fails(self):
-        self.t['resources']['super-cluster']['properties'].pop(
-            'default_image_id')
+        self.t['resources']['super-cluster']['properties'].pop('image')
         self.sahara_mock.cluster_templates.get.return_value = mock.Mock(
             default_image_id=None)
         cluster = self._init_cluster(self.t)
@@ -209,25 +170,3 @@ class SaharaClusterTest(common.HeatTestCase):
                                cluster.validate)
         self.assertEqual("neutron_management_network must be provided",
                          six.text_type(ex))
-
-    def test_validation_error_for_deprecated_properties(self):
-        tmpl = '''
-heat_template_version: 2013-05-23
-description: Hadoop Cluster by Sahara
-resources:
-  super-cluster:
-    type: OS::Sahara::Cluster
-    properties:
-      name: super-cluster
-      plugin_name: vanilla
-      hadoop_version: 2.3.0
-      cluster_template_id: some_cluster_template_id
-      image: some_image
-      default_image_id: test_image_id
-      key_name: admin
-      neutron_management_network: some_network
-      '''
-        ct = self._init_cluster(template_format.parse(tmpl))
-        ex = self.assertRaises(exception.ResourcePropertyConflict, ct.validate)
-        msg = 'Cannot define the following properties at the same time: '
-        self.assertIn(msg, six.text_type(ex))
