@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -13,19 +13,15 @@
 
 
 import mox
-from testtools import skipIf
+import swiftclient.client as sc
 
+from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
-from heat.engine import resource
 from heat.engine.resources import swift
 from heat.engine import scheduler
-from heat.openstack.common.importutils import try_import
 from heat.tests.common import HeatTestCase
-from heat.tests import fakes
 from heat.tests import utils
 
-swiftclient = try_import('swiftclient.client')
 
 swift_template = '''
 {
@@ -69,23 +65,21 @@ swift_template = '''
 
 
 class swiftTest(HeatTestCase):
-    @skipIf(swiftclient is None, 'unable to import swiftclient')
     def setUp(self):
         super(swiftTest, self).setUp()
-        self.m.CreateMock(swiftclient.Connection)
-        self.m.StubOutWithMock(swiftclient.Connection, 'post_account')
-        self.m.StubOutWithMock(swiftclient.Connection, 'put_container')
-        self.m.StubOutWithMock(swiftclient.Connection, 'delete_container')
-        self.m.StubOutWithMock(swiftclient.Connection, 'head_container')
-        self.m.StubOutWithMock(swiftclient.Connection, 'get_auth')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-
-        utils.setup_dummy_db()
+        self.m.CreateMock(sc.Connection)
+        self.m.StubOutWithMock(sc.Connection, 'post_account')
+        self.m.StubOutWithMock(sc.Connection, 'put_container')
+        self.m.StubOutWithMock(sc.Connection, 'delete_container')
+        self.m.StubOutWithMock(sc.Connection, 'head_container')
+        self.m.StubOutWithMock(sc.Connection, 'get_auth')
+        self.stub_keystoneclient()
 
     def create_resource(self, t, stack, resource_name):
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = swift.SwiftContainer(
             'test_resource',
-            t['Resources'][resource_name],
+            resource_defns[resource_name],
             stack)
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
@@ -96,9 +90,10 @@ class swiftTest(HeatTestCase):
         t = template_format.parse(swift_template)
         t['Resources']['SwiftContainer']['Properties']['name'] = 'the_name'
         stack = utils.parse_stack(t)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = swift.SwiftContainer(
             'test_resource',
-            t['Resources']['SwiftContainer'],
+            resource_defns['SwiftContainer'],
             stack)
 
         self.assertEqual('the_name', rsrc.physical_resource_name())
@@ -131,14 +126,12 @@ class swiftTest(HeatTestCase):
             "x-container-bytes-used": "17680980",
             "content-type": "text/plain; charset=utf-8"}
 
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name, {}).AndReturn(None)
-        swiftclient.Connection.head_container(
+        sc.Connection.head_container(
             mox.IgnoreArg()).MultipleTimes().AndReturn(headers)
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -156,23 +149,18 @@ class swiftTest(HeatTestCase):
         self.assertEqual('17680980', rsrc.FnGetAtt('BytesUsed'))
         self.assertEqual(headers, rsrc.FnGetAtt('HeadContainer'))
 
-        self.assertRaises(swift.exception.InvalidTemplateAttribute,
+        self.assertRaises(exception.InvalidTemplateAttribute,
                           rsrc.FnGetAtt, 'Foo')
 
-        self.assertRaises(resource.UpdateReplace,
-                          rsrc.handle_update, {}, {}, {})
-
         scheduler.TaskRunner(rsrc.delete)()
         self.m.VerifyAll()
 
     def test_public_read(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {'X-Container-Read': '.r:*'}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -184,14 +172,12 @@ class swiftTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_public_read_write(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {'X-Container-Write': '.r:*',
              'X-Container-Read': '.r:*'}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -204,15 +190,13 @@ class swiftTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_container_headers(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {'X-Container-Meta-Web-Error': 'error.html',
              'X-Container-Meta-Web-Index': 'index.html',
              'X-Container-Read': '.r:*'}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -222,13 +206,11 @@ class swiftTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_account_headers(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(container_name, {})
-        swiftclient.Connection.post_account(
+        sc.Connection.put_container(container_name, {})
+        sc.Connection.post_account(
             {'X-Account-Meta-Temp-Url-Key': 'secret'}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -238,14 +220,30 @@ class swiftTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_delete_exception(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndRaise(
-            swiftclient.ClientException('Test delete failure'))
+        sc.Connection.delete_container(container_name).AndRaise(
+            sc.ClientException('Test delete failure'))
+
+        self.m.ReplayAll()
+        t = template_format.parse(swift_template)
+        stack = utils.parse_stack(t)
+        rsrc = self.create_resource(t, stack, 'SwiftContainer')
+        self.assertRaises(exception.ResourceFailure,
+                          scheduler.TaskRunner(rsrc.delete))
+
+        self.m.VerifyAll()
+
+    def test_delete_not_found(self):
+        container_name = utils.PhysName('test_stack', 'test_resource')
+        sc.Connection.put_container(
+            container_name,
+            {}).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndRaise(
+            sc.ClientException('Its gone',
+                               http_status=404))
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -256,11 +254,8 @@ class swiftTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_delete_retain(self):
-
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         # first run, with retain policy
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             utils.PhysName('test_stack', 'test_resource'),
             {}).AndReturn(None)
 
@@ -278,15 +273,13 @@ class swiftTest(HeatTestCase):
 
     def test_default_headers_not_none_empty_string(self):
         '''Test that we are not passing None when we have a default
-        empty string or swiftclient will pass them as string None. see
+        empty string or sc will pass them as string None. see
         bug lp:1259571.
         '''
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name, {}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -295,4 +288,4 @@ class swiftTest(HeatTestCase):
         scheduler.TaskRunner(rsrc.delete)()
         self.m.VerifyAll()
 
-        self.assertEqual({}, rsrc.metadata)
+        self.assertEqual({}, rsrc.metadata_get())
