@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -15,12 +15,13 @@
 import datetime
 
 import mox
+from oslo.utils import timeutils
 
 from heat.common import exception
-import heat.db.api as db_api
+from heat.db import api as db_api
 from heat.engine import parser
+from heat.engine import template
 from heat.engine import watchrule
-from heat.openstack.common import timeutils
 from heat.tests.common import HeatTestCase
 from heat.tests import utils
 
@@ -39,24 +40,22 @@ class DummyAction(object):
 class WatchRuleTest(HeatTestCase):
     stack_id = None
 
-    @classmethod
-    def setUpDatabase(cls):
-        if cls.stack_id is not None:
+    def setUpDatabase(self):
+        if self.stack_id is not None:
             return
         # Create a dummy stack in the DB as WatchRule instances
         # must be associated with a stack
-        utils.setup_dummy_db()
         ctx = utils.dummy_context()
         ctx.auth_token = 'abcd1234'
-        empty_tmpl = {"template": {}}
-        tmpl = parser.Template(empty_tmpl)
+        empty_tmpl = {'HeatTemplateFormatVersion': '2012-12-12'}
+        tmpl = template.Template(empty_tmpl)
         stack_name = 'dummystack'
         dummy_stack = parser.Stack(ctx, stack_name, tmpl)
         dummy_stack.state_set(dummy_stack.CREATE, dummy_stack.COMPLETE,
                               'Testing')
         dummy_stack.store()
 
-        cls.stack_id = dummy_stack.id
+        self.stack_id = dummy_stack.id
 
     def setUp(self):
         super(WatchRuleTest, self).setUp()
@@ -269,7 +268,6 @@ class WatchRuleTest(HeatTestCase):
         new_state = self.wr.get_alarm_state()
         self.assertEqual('ALARM', new_state)
 
-    @utils.wr_delete_after
     def test_load(self):
         # Insert two dummy watch rules into the DB
         rule = {u'EvaluationPeriods': u'1',
@@ -309,7 +307,6 @@ class WatchRuleTest(HeatTestCase):
             self.assertEqual(datetime.timedelta(seconds=int(rule['Period'])),
                              wr.timeperiod)
 
-    @utils.wr_delete_after
     def test_store(self):
         rule = {u'EvaluationPeriods': u'1',
                 u'AlarmActions': [u'WebServerRestartPolicy'],
@@ -330,7 +327,6 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(watchrule.WatchRule.NODATA, dbwr.state)
         self.assertEqual(rule, dbwr.rule)
 
-    @utils.wr_delete_after
     def test_evaluate(self):
         rule = {'EvaluationPeriods': '1',
                 'MetricName': 'test_metric',
@@ -388,7 +384,6 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(now, self.wr.last_evaluated)
         self.assertEqual([], actions)
 
-    @utils.wr_delete_after
     def test_evaluate_suspend(self):
         rule = {'EvaluationPeriods': '1',
                 'MetricName': 'test_metric',
@@ -418,7 +413,6 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(self.wr.SUSPENDED, self.wr.state)
         self.assertEqual([], actions)
 
-    @utils.wr_delete_after
     def test_rule_actions_alarm_normal(self):
         rule = {'EvaluationPeriods': '1',
                 'MetricName': 'test_metric',
@@ -446,7 +440,6 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual([], actions)
         self.m.VerifyAll()
 
-    @utils.wr_delete_after
     def test_rule_actions_alarm_alarm(self):
         rule = {'EvaluationPeriods': '1',
                 'MetricName': 'test_metric',
@@ -481,7 +474,6 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(['DummyAction'], actions)
         self.m.VerifyAll()
 
-    @utils.wr_delete_after
     def test_rule_actions_alarm_two_actions(self):
         rule = {'EvaluationPeriods': '1',
                 'MetricName': 'test_metric',
@@ -509,7 +501,6 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(['DummyAction', 'DummyAction'], actions)
         self.m.VerifyAll()
 
-    @utils.wr_delete_after
     def test_rule_actions_ok_alarm(self):
         rule = {'EvaluationPeriods': '1',
                 'MetricName': 'test_metric',
@@ -550,7 +541,6 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(['DummyAction'], actions)
         self.m.VerifyAll()
 
-    @utils.wr_delete_after
     def test_rule_actions_nodata(self):
         rule = {'EvaluationPeriods': '1',
                 'MetricName': 'test_metric',
@@ -589,7 +579,6 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(['DummyAction'], actions)
         self.m.VerifyAll()
 
-    @utils.wr_delete_after
     def test_create_watch_data(self):
         rule = {u'EvaluationPeriods': u'1',
                 u'AlarmDescription': u'test alarm',
@@ -619,7 +608,6 @@ class WatchRuleTest(HeatTestCase):
         # correctly get a list of all datapoints where watch_rule_id ==
         # watch_rule.id, so leave it as a single-datapoint test for now.
 
-    @utils.wr_delete_after
     def test_create_watch_data_suspended(self):
         rule = {u'EvaluationPeriods': u'1',
                 u'AlarmDescription': u'test alarm',
@@ -643,7 +631,6 @@ class WatchRuleTest(HeatTestCase):
         dbwr = db_api.watch_rule_get_by_name(self.ctx, 'create_data_test')
         self.assertEqual([], dbwr.watch_data)
 
-    @utils.wr_delete_after
     def test_create_watch_data_match(self):
         rule = {u'EvaluationPeriods': u'1',
                 u'AlarmDescription': u'test alarm',
@@ -665,7 +652,6 @@ class WatchRuleTest(HeatTestCase):
                                                       u'group_x'}]}}
         self.assertTrue(watchrule.rule_can_use_sample(self.wr, data))
 
-    @utils.wr_delete_after
     def test_create_watch_data_match_2(self):
         rule = {u'EvaluationPeriods': u'1',
                 u'AlarmDescription': u'test alarm',
